/*===========================================================================*
 * FILE:                        C D E V _ F W . H
 *===========================================================================*
 *
 * COPYRIGHT (C) 2007-2011  BY
 *          GE INTELLIGENT PLATFORMS, INC. SANTA BARBARA, CALIFORNIA
 *          ALL RIGHTS RESERVED.
 *
 *          THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *          COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *          THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *          OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *          AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *          SOFTWARE IS HEREBY TRANSFERRED.
 *
 *          THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *          NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY GE 
 *          INTELLIGENT PLATFORMS.
 *
 *===========================================================================*
 *
 * FUNCTION:   Firmware array declaration header file for CEI-x30 products.
 * 
 *---------------------------------------------------------------------------*
 *          
 * HISTORY:   
 *
 *   Date     By   Comments
 * --------  ----  ------------------------------------------------------
 * 12/07/07  BDW   Created.
 * 03/26/08  BDW   Default to uncompressed firmware build, define the
 *                 directive COMPRESSED_FW to use compressed header files.
 * 04/02/09  BDW   Only truly define the CEI-830 firmware load for Integrity
 * 04/07/09  BDW   Remove the use of COMPRESSED_FW and compressed f/w files
 * 09/30/10  BDW   Moved the CEI-530 f/w array delcaration so it is included 
 *                 when compiled for the LabVIEW Real-Time target
 * 09/16/11  BDW   Added CEI-430A firmware array declaration fpga_430A[] by
 *                 including the header file fpga430A.h.
 *===========================================================================*/

#ifndef CDEV_FW_H
#define CDEV_FW_H

/*===========================================================================* 
 *  Supporting uncompressed firmware files here:
 *
 *  NOTE:  to reduce the overall size of the API, you may eliminate firmware
 *         support for unused boards by replacing the standard include "*.h"
 *         with #include "fpgax30n.h", as demonstrated with the LabVIEW
 *         Real-Time build (when the directive LABVIEW_RT is defined).
 *
 *
 *  NOTE:  for Integrity, only define the CEI-830 uncompressed firmware load
 *
 *
 *===========================================================================*/
#ifdef INTEGRITY_PCI_PPC
   static CEI_UCHAR  const fpga_830[] = {
      #include "fpga830.h" 
   };

   /* all other firmware is defined as 1 element arrays */ 
   static CEI_UCHAR  const fpga_830_rx[] = {
      #include "fpgax30n.h" 
   };
   static CEI_UINT32 const fpga_630[] = {
      #include "fpgax30n.h" 
   };
   static CEI_UCHAR  const fpga_430[] = {
      #include "fpgax30n.h" 
   };
   static CEI_UCHAR  const fpga_430A[] = {
      #include "fpgax30n.h" 
   };
   static CEI_UINT32 const fpga_530[] = {
      #include "fpgax30n.h" 
   };
   static CEI_UINT32 const fpga_A30[] = {
      #include "fpgax30n.h" 
   };
   static CEI_UINT32 const fpga_ec[] = {
      #include "fpgax30n.h" 
   };
#else
   static CEI_UCHAR  const fpga_830[] = { 
      #include "fpga830.h" 
   };
   static CEI_UCHAR  const fpga_830_rx[] = {
      #include "fpga830rx.h" 
   };
   static CEI_UINT32 const fpga_630[] = {
      #include "fpga630.h" 
   };
   static CEI_UINT32 const fpga_530[] = {
      #include "fpga530.h" 
   };
     
   /* For LabVIEW Real-Time, only include the RCEI-530, CEI-830, R830RX and RAR-CPCI firmware above.
    * For the 430, A30 and RAR-EC include a 1 element array instead.
    */
   #if defined (NIVISA_PXI)    
      static CEI_UINT32 const fpga_ec[] = {
         #include "fpgax30n.h" 
      };
      static CEI_UCHAR  const fpga_430[] = {
         #include "fpgax30n.h" 
      };
      static CEI_UCHAR  const fpga_430A[] = {
         #include "fpgax30n.h" 
      };
      static CEI_UINT32 const fpga_A30[] = {
         #include "fpgax30n.h" 
      };
   #else
      /* The real stuff */
      static CEI_UINT32 const fpga_ec[] = {
         #include "fpga_ec.h" 
      };
      static CEI_UCHAR  const fpga_430[] = {
         #include "fpga430.h" 
      };
      static CEI_UCHAR  const fpga_430A[] = {
         #include "fpga430A.h" 
      };
      static CEI_UINT32 const fpga_A30[] = {
         #include "fpgaA30.h" 
      };
   #endif
#endif

#endif /* ifndef CDEV_FW_H */

