/*===========================================================================*
 * FILE:                  C E I _ I N S T A L L . H
 *===========================================================================*
 *
 *                   *** do not distribute this file ***
 *
 *===========================================================================*
 *
 *      COPYRIGHT (C) 1998 - 2011 BY
 *      GE INTELLIGENT PLATFORMS EMBEDDED SYSTEMS, INC.
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY GE 
 *      INTELLIGENT PLATFORMS EMBEDDED SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:
 *  
 *      Defines the interface to the CEI installation utility library. 
 *
 * HISTORY:
 *
 *   Date     By   Vers                        Comments
 * --------  ----  ----  -----------------------------------------------------
 * 10/07/02  skb   1.10  Initial version.  Refer to the cei_install.cpp 
 *                       comment header for revision history.
 *
 *===========================================================================*/
#ifndef CEI_INSTALL_H
#define CEI_INSTALL_H


// make sure standard data types are defined
#ifndef CEI_KERNEL_BUILD
#include <windows.h> 
#include <stdio.h>
#endif

// include common data type definitions
#include "cei_types.h"


// make sure the build environment is recognizable
#if (defined(WIN32) || defined(_WIN32))

   // define the DLL function export tag
   #define DLL_EXPORTED _stdcall

   // check if this is a Visual Studio build
   #if (defined(_MSC_VER))

      // make sure we're not compiling with Unicode character set
      #if (defined(_UNICODE))

         // terminate the current build
         #error "Unicode character set is not supported."

      #endif

      // disable unused var and snprintf warnings (already secure, thanks anyway)
      #pragma warning(push)
      #pragma warning(disable:4100)
      #pragma warning(disable:4996)

   #endif

#else

   // terminate the current build
   #error "Unsupported build configuration."

#endif


// define: CEI_INSTALL_VERSION
// usage:  The current installation library version.
#define CEI_INSTALL_VERSION            "4.05"

// define: CEI_INSTALL_MAX_FILE_PATH_LEN
// usage:  The maximum length (in characters) of a string containing the
//         full path to a file on the target machine.
#define CEI_INSTALL_MAX_FILE_PATH_LEN  512

// define: CEI_INSTALL_MAX_VERSION_LEN
// usage:  The maximum length (in characters) of a string containing the
//         version string associated with a file or product on the target
//         machine.
#define CEI_INSTALL_MAX_VERSION_LEN    32

// define: CEI_INSTALL_MAX_FILE_TIME_LEN
// usage:  The maximum length (in characters) of a string containing the
//         time/date stamp associated with a file on the target machine.
#define CEI_INSTALL_MAX_FILE_TIME_LEN  32

// define: CEI_INSTALL_MAX_SERIAL_LEN
// usage:  The maximum length (in characters) of a string containing a product
//         serial number. 
#define CEI_INSTALL_MAX_SERIAL_LEN     32

// define: CEI_INSTALL_MAX_DRIVER_TYPE_LEN
// usage:  The maximum length (in characters) of a driver type string.
#define CEI_INSTALL_MAX_DRV_TYPE_LEN   32

// define: CEI_INSTALL_MAX_DRIVER_NAME_LEN
// usage:  The maximum length (in characters) of a driver name string.
#define CEI_INSTALL_MAX_DRV_NAME_LEN   32

// define: CEI_INSTALL_MAX_FILES 
// usage:  The maximum number of CEI-related files installed on a target
//         machine.
#define CEI_INSTALL_MAX_FILES          1024

// define: CEI_INSTALL_MAX_KEY_PATH_LEN
// usage:  The maximum length (in characters) of a string containing the
//         full path to a CEI-related registry key on the target machine.
#define CEI_INSTALL_MAX_KEY_PATH_LEN   512

// define: CEI_INSTALL_MAX_KEY_DATA_LEN
// usage:  The maximum length (in bytes) of a data buffer associated with
//         a CEI-related registry key on the target machine.
#define CEI_INSTALL_MAX_KEY_DATA_LEN   128

// define: CEI_INSTALL_MAX_REG_KEYS
// usage:  The maximum number of CEI-related registry keys on a target
//         machine.
#define CEI_INSTALL_MAX_REG_KEYS       256

// define: CEI_INSTALL_MAX_SW_SUPP_DEVS
// usage:  The maximum number of hardware devices (e.g. CEI-220, CEI-420, 
//         CEI-715, etc.) that any software product can support.
#define CEI_INSTALL_MAX_SW_SUPP_DEVS   64

// define: CEI_INSTALL_MAX_PRODUCTS
// usage:  The maximum number of CEI products installed on any given target
//         target machine.
#define CEI_INSTALL_MAX_PRODUCTS       64

// define: CEI_INSTALL_MAX_PROD_DESC_LEN
// usage:  The maximum length (in characters) of a string containing a 
//         description of a CEI product.
#define CEI_INSTALL_MAX_PROD_DESC_LEN  64

// define: CEI_INSTALL_MAX_ERR_DESC_LEN
// usage:  The maximum length (in characters) of a string containing an
//         installation library error description.
#define CEI_INSTALL_MAX_ERR_DESC_LEN   512

// define: CEI_MAX_RES_ID_LEN
// usage:  The maximum length (in characters) of a case-sensitive string that 
//         uniquely identifies a particular resource instance across an entire 
//         machine.
#define CEI_MAX_RES_ID_LEN             64

// define: CEI_MAX_DEV_MEM_RGNS
// usage:  The maximum number of memory regions supported by a device.
#define CEI_MAX_DEV_MEM_RGNS           32

// define: CEI_MAX_INT_REGS
// usage:  The maximum number of interrupt status or disable register processing
//         structures supported for a single card.
#define CEI_MAX_INT_REGS               32

// define: CEI_INSTALL_LIB_NAME
// usage:  This contains the name of the installation library.
#if defined(_WIN64) 
#define CEI_INSTALL_LIB_NAME           "cei_install_64.dll"
#else
#define CEI_INSTALL_LIB_NAME           "cei_install.dll"
#endif

// define: <x>_VER
// usage:  The version constant associated with the current version of the 
//         structure <x>.
#define CEI_PRODUCT_LIST_VER           3
#define CEI_FILE_LIST_VER              2
#define CEI_REG_KEY_LIST_VER           1
#define CEI_DEVICE_MEM_VER             1

// define: CEI_OS_<x>
// usage:  These bitfield values (family, architecture, and version) are used 
//         to describe the currently installed Windows version.  Refer to 
//         CEI_DLL_GetSystemVersionEx() for details.
#define CEI_OS_FAM_9X                  0x0010000000000000
#define CEI_OS_FAM_NT                  0x0020000000000000
#define CEI_OS_FAM_2000                0x0040000000000000
#define CEI_OS_FAM_VISTA               0x0080000000000000
#define CEI_OS_ARCH_32                 0x0000100000000000
#define CEI_OS_ARCH_64                 0x0000200000000000
#define CEI_OS_95_GOLD                 0x0000000000000001
#define CEI_OS_95_OSR2                 0x0000000000000002
#define CEI_OS_98_STD                  0x0000000000000004
#define CEI_OS_98_SE                   0x0000000000000008
#define CEI_OS_ME                      0x0000000000000010
#define CEI_OS_NT                      0x0000000000000020
#define CEI_OS_2000                    0x0000000000000040
#define CEI_OS_XP                      0x0000000000000080
#define CEI_OS_SERVER_2003             0x0000000000000100 
#define CEI_OS_VISTA                   0x0000000000000200 
#define CEI_OS_SERVER_2008             0x0000000000000400
#define CEI_OS_WIN7                    0x0000000000000800


// type:   CEI_FILE_ENTRY
// usage:  This structure contains the full path, the version string, and the 
//         time/date stamp associated with a CEI-related file on the 
//         target machine.
typedef struct 
{
    CEI_CHAR path[CEI_INSTALL_MAX_FILE_PATH_LEN];
    CEI_CHAR version[CEI_INSTALL_MAX_VERSION_LEN];
    CEI_CHAR timestamp[CEI_INSTALL_MAX_FILE_TIME_LEN];
} CEI_FILE_ENTRY;

// type:   CEI_FILE_LIST
// usage:  This structure contains a list of CEI-related files on the 
//         target machine. Each of the first 'numFiles' elements in the 
//         'files' array contains a file entry.
typedef struct 
{
    CEI_INT32 numFiles;
    CEI_FILE_ENTRY files[CEI_INSTALL_MAX_FILES];
} CEI_FILE_LIST;

// type:   CEI_KEY_ENTRY
// usage:  This structure contains the full path, the data type (currently 
//         either REG_SZ, REG_DWORD, or REG_NONE), and the data associated 
//         with a CEI-related registry item on the target machine. If 
//         'dataType' is REG_SZ, the 'data' buffer should be interpreted as a
//         null-terminated ASCII string. If 'dataType' is REG_DWORD, the first
//         four bytes of the 'data' buffer contain the 32-bit value associated
//         with the item. If 'dataType' is REG_NONE, the registry item exists
//         on the target machine but the data type is not supported by the 
//         installation library (ignore the contents of the 'data' buffer in 
//         this case).
typedef struct 
{
    CEI_CHAR path[CEI_INSTALL_MAX_KEY_PATH_LEN];
    CEI_CHAR dataType;
    CEI_CHAR data[CEI_INSTALL_MAX_KEY_DATA_LEN];
} CEI_KEY_ENTRY;

// type:   CEI_REG_KEY_LIST
// usage:  This structure contains a list of CEI-related registry keys on
//         the target machine. Each of the first 'numKeys' elements in the 
//         'keys' array contains a key entry.
typedef struct 
{
    CEI_INT32 numKeys;
    CEI_KEY_ENTRY keys[CEI_INSTALL_MAX_REG_KEYS];
} CEI_REG_KEY_LIST;

// type:   CEI_INSTALL_ERR
// usage:  This standard string type is used to relay error information
//         back to the client application.
typedef CEI_CHAR CEI_INSTALL_ERR[CEI_INSTALL_MAX_ERR_DESC_LEN];

// type:   CEI_RES_ID
// usage:  This standard string type is used to uniquely identify a particular
//         resource instance across an entire machine.  Resource ID strings are
//         case-sensitive.
typedef CEI_CHAR CEI_RES_ID[CEI_MAX_RES_ID_LEN];

// type:   CEI_DEV_HANDLE
// usage:  This type represents a handle to a device instance.
typedef HANDLE CEI_DEV_HANDLE,*pCEI_DEV_HANDLE;

// type:   CEI_BUS_TYPE
// usage:  These are the different bus types targeted by CEI hardware.
enum CEI_BUS_TYPE 
{
    CEI_BUS_ISA = 1,
    CEI_BUS_PCI = 5,
    CEI_BUS_VME = 6,
    CEI_BUS_PCMCIA = 8,
    CEI_BUS_NONE = 32
};

// type:   CEI_RES_FLAG
// usage:  These are the different types of resources that can be required by
//         various CEI hardware devices. These flags are or'd together when a
//         device requires multiple different resource types.
enum CEI_RES_FLAG 
{
    CEI_RES_NONE = 0,
    CEI_RES_MEM = 1,
    CEI_RES_IO = 2,
    CEI_RES_IRQ = 4
};

// type:   CEI_DEVICE_TYPE
// usage:  This structure contains the device type and desc of a certain
//         CEI board type (e.g. typeID = CEI_HW_CEI_220, desc = "CEI-220").
typedef struct 
{
    CEI_INT32 typeID;
    CEI_CHAR  desc[CEI_INSTALL_MAX_PROD_DESC_LEN];
} CEI_DEVICE_TYPE;

// type:   CEI_PRODUCT_DESC
// usage:  This structure describes a particular product that is currently 
//         installed on the target machine. Fields beginning with 'hw' (e.g.
//         'hwDeviceType' are valid if 'isHardware' is true. If 'isHardware'
//         is false, fields beginning with 'sw' (e.g. 'swNumSupportedDevs')
//         are valid. For elements of the form '<element>Valid', the structure
//         member <element> (e.g. 'hwDeviceNum') is only valid if the 
//         <element>Valid (e.g. 'hwDeviceNumValid') member is set to true.
typedef struct 
{
    CEI_INT32 productID;
    CEI_CHAR  name[CEI_INSTALL_MAX_PROD_DESC_LEN];
    CEI_CHAR  version[CEI_INSTALL_MAX_VERSION_LEN];
    CEI_INT32 serialValid;
    CEI_CHAR  serialNumber[CEI_INSTALL_MAX_SERIAL_LEN];
    CEI_INT32 isHardware;
    CEI_INT32 swNumSupportedDevs;
    CEI_DEVICE_TYPE swSupportedDevs[CEI_INSTALL_MAX_SW_SUPP_DEVS];
    CEI_INT32 swHardwareDependencyValid;
    CEI_DEVICE_TYPE swHardwareDependency;
    CEI_DEVICE_TYPE hwDeviceType;
    CEI_INT32 hwDeviceNumValid;
    CEI_INT32 hwDeviceNum;
    CEI_INT32 hwBusType;
    CEI_INT32 hwResourceReqs;
    CEI_INT32 hwMemoryAddress;
    CEI_INT32 hwMemoryCount;
    CEI_INT32 hwIoPortAddress;
    CEI_INT32 hwIoPortCount;
    CEI_INT32 hwInterrupt;
    CEI_CHAR  hwDriverType[CEI_INSTALL_MAX_DRV_TYPE_LEN];
    CEI_CHAR  hwDriverVer[CEI_INSTALL_MAX_VERSION_LEN];
    CEI_CHAR  hwDriverNamePrimary[CEI_INSTALL_MAX_DRV_NAME_LEN];
    CEI_CHAR  hwDriverNameSecondary[CEI_INSTALL_MAX_DRV_NAME_LEN];
} CEI_PRODUCT_DESC;

// type:   CEI_PRODUCT_LIST
// usage:  This structure contains a list of CEI products installed on the
//         target machine.
typedef struct 
{
    CEI_INT32 numProducts;
    CEI_PRODUCT_DESC product[CEI_INSTALL_MAX_PRODUCTS];
} CEI_PRODUCT_LIST;

// type:  CEI_GLOBAL_SHARED_OBJECT_TYPE
// usage: These are the types of shared global named objects supported by the
//        CEI_DLL_OpenGlobalNamedObject and CEI_DLL_CloseGlobalNamedObject
//        functions.
enum CEI_GLOBAL_SHARED_OBJECT_TYPE
{
   CEI_GSO_MUTEX = 1,
   CEI_GSO_EVENT = 2,
   CEI_GSO_SHMEM = 3,
};

// type:   CEI_FOLDER_ID
// usage:  These folders are supported by the CEI_DLL_GetFolderPath routine.
enum CEI_FOLDER_ID
{
   CFI_SYSTEM_X86 = 1,
   CFI_SYSTEM_X64 = 2,
   CFI_PROGRAM_FILES_X86 = 3,
   CFI_PROGRAM_FILES_X64 = 4,
   CFI_COMMON_DOCUMENTS = 5,
};

// type:   CEI_CLR_REG_OPTION
// usage:  These flags are used to control the interrupt clearing operation
//         that is performed by the driver.  Refer to CEI_INT_STATUS_REG_V1 
//         description for details.
enum CEI_CLR_REG_OPTION
{
   CEI_CRO_AND = 1,
   CEI_CRO_SET = 2,
   CEI_CRO_NONE = 4,
   CEI_CRO_USE_CHK_REG = 8,
   CEI_CRO_PAG_REG_EN = 16,
};

// type:   CEI_CHK_REG_OPTION
// usage:  These flags are used to control optional additional functionality 
//         associated with an interrupt checking operation that is performed by
//         the driver.  Refer to CEI_INT_STATUS_REG_V1 description for details.
enum CEI_CHK_REG_OPTION
{
   CEI_XRO_NONE = 0,
   CEI_XRO_PROP_CHK_EN = 1,
};

// type:   CEI_DIS_REG_OPTION
// usage:  These flags are used to control the interrupt disable operation
//         that is performed by the driver.  Refer to CEI_INT_DISABLE_REG_V1
//         description for details.
enum CEI_DIS_REG_OPTION
{
   CEI_DRO_AND = 1,
   CEI_DRO_SET = 2,
};

// type:   CEI_CAP_TYPE
// usage:  These flags describe capabilities of the low-level interface which are
//         not supported on all platforms.  At run-time, callers can determine
//         whether a particular capability is supported using LL_GetCapability.
enum CEI_CAP_TYPE
{
   CEI_CAP_HW_INTS = 1,
   CEI_CAP_CONTIG_DMA = 2,
   CEI_CAP_HW_INTS_MULTI_PROC = 3,
};

// type:   CEI_WAIT_INT_RESULT
// usage:  These flags describe the result of an interrupt wait operation.  Refer
//         to LL_WaitForInterrupt for details.
enum CEI_WAIT_INT_RESULT
{
   CEI_WIR_INT_OCCURRED = 0,
   CEI_WIR_INT_DISABLED = 1,
};

// type:   CEI_DEVICE_MEM
// usage:  This structure contains the number of memory regions supported by a
//         particular device and the size and user-mode address of each region.
typedef struct
{
   CEI_UINT32 num_rgns;     // (o) the number of memory regions supported by
                            // the device
   CEI_UINT32 fill;         // (r) reserved
   CEI_UINT32 rgn_len_bytes[CEI_MAX_DEV_MEM_RGNS]; // (o) length (in bytes) of
                            // each memory region
   pCEI_VOID  rgn_usr_addr[CEI_MAX_DEV_MEM_RGNS]; // (o) user-mode mapping of
                            // the physical base address for each memory region -
                            // these addresses can be used to access each memory
                            // region from a user-mode process
} CEI_DEVICE_MEM,*pCEI_DEVICE_MEM;

// type:   CEI_ONBOARD_REG
// usage:  This structure contains the location and size of a particular on-board
//         register.  All fields marked (i) must be initialized by the caller.  
//         All fields marked (r) are reserved for use by the CEI-Install library 
//         and driver, and should not be referenced by the caller.  The address 
//         field rgn_usr_addr is represented using a 64-bit value - if the caller
//         is a 32-bit application/library, store the 32-bit address value in the
//         low 32 bits of rgn_usr_addr.
typedef struct
{
   CEI_UINT64 rgn_usr_addr; // (i) user-mode mapping of the base address of the 
                            // device memory region that contains the register
                            // (returned by a prior call to LL_MapMemoryRegions)
   CEI_UINT32 offset_bytes; // (i) offset (in bytes) of the register within the 
                            // memory region specified in rgn_usr_addr
   CEI_UINT32 len_bytes;    // (i) length of the register in bytes (1/2/4/8)
   CEI_UINT64 rgn_phy_addr; // (r) reserved
   CEI_UINT64 kern_addr;    // (r) reserved
} CEI_ONBOARD_REG,*pCEI_ONBOARD_REG;

// type:   CEI_INT_STATUS_REG_V1
// usage:  This structure contains parameters used to process an on-board
//         interrupt status register.  All fields marked (i) must be initialized
//         by the caller before enabling interrupts.  All fields marked (r) are
//         reserved for use by the CEI-Install library and driver, and should not
//         be referenced by the caller.
typedef struct
{
   CEI_UINT64 cap_reg_mask; // (i) if chk_cap_reg is non-zero, this interrupt
                            // status register is only processed if the value
                            // of the capability register bitwise ANDed with the
                            // low cap_reg.len_bytes of cap_reg_mask is non-zero
                            // (this field is ignored if chk_cap_reg is zero)
   CEI_ONBOARD_REG chk_reg; // (i) location and size of the chk_reg register - 
                            // this register contains a bit field which is non-
                            // zero when the board generates an interrupt
   CEI_UINT64 chk_reg_opt;  // (i) combination of CEI_XRO_xxx flags (bitwise ORed
                            // together) which control optional additional 
                            // functionality associated with the interrupt 
                            // checking operation for this register - valid 
                            // options are CEI_XRO_NONE (no additional optional
                            // interrupt checking functionality is requested) or 
                            // CEI_XRO_PROP_CHK_EN (normal interrupt checking and
                            // clearing are performed, but the interrupt is only
                            // propagated to user-mode if the current contents of
                            // the chk_reg register bitwise ANDed with the high
                            // four bytes of chk_reg_opt is non-zero)
   CEI_UINT64 chk_reg_mask; // (i) when an interrupt occurs, a non-destructive
                            // bitwise AND (&) operation is performed on the low
                            // chk_reg.len_bytes of this mask value and the 
                            // current contents of the chk_reg register - if non-
                            // zero, then our driver claims ownership of the 
                            // interrupt
   CEI_UINT64 chk_reg_val;  // (r) reserved for future use
   CEI_ONBOARD_REG clr_reg; // (i) location and size of the clr_reg register - 
                            // this register contains a bit field to be cleared 
                            // when chk_reg indicates that the board has 
                            // generated an interrupt (ignored if clr_reg_opt 
                            // includes CEI_CRO_USE_CHK_REG or CEI_CRO_NONE)
   CEI_UINT64 clr_reg_opt;  // (i) combination of CEI_CRO_xxx flags (bitwise ORed
                            // together) which control execution of the interrupt
                            // clearing operation for this register - valid
                            // options are CEI_CRO_USE_CHK_REG (which directs the
                            // interrupt handler to perform the clearing oper
                            // on chk_reg instead of clr_reg), CEI_CRO_PAG_REG_EN
                            // (see pag_reg for a discussion of this option),
                            // CEI_CRO_NONE (no clearing operation is performed
                            // for this register) and CEI_CRO_AND/CEI_CRO_SET 
                            // (see clr_reg_mask for a discussion of these 
                            // options) - the clr_reg field is ignored if 
                            // clr_reg_opt includes CEI_CRO_USE_CHK_REG or 
                            // CEI_CRO_NONE
   CEI_UINT64 clr_reg_mask; // (i) when chk_reg indicates that the board has
                            // generated an interrupt, the low clr_reg.len_bytes
                            // of this mask value are either (a) written to the
                            // clr_reg register (if clr_reg_opt includes
                            // CEI_CRO_SET), or (b) bitwise ANDed with the 
                            // current value of the clr_reg register and written
                            // back to the clr_reg register (if clr_reg_opt
                            // includes CEI_CRO_AND) - if clr_reg_opt does not 
                            // include CEI_CRO_SET or CEI_CRO_AND, clr_reg_mask 
                            // is ignored 
   CEI_ONBOARD_REG pag_reg; // (i) location and size of the pag_reg register - 
                            // if clr_reg_opt includes CEI_CRO_PAG_REG_EN, the 
                            // following logic is used to clear an interrupt: 
                            // (1) save the value of pag_reg, (2) write the low 
                            // pag_reg.len_bytes of pag_reg_val to pag_reg,
                            // (3) clear the int (see clr_reg_mask description 
                            // for details), and (4) restore the saved value of 
                            // pag_reg - if clr_reg_opt does not include 
                            // CEI_CRO_PAG_REG_EN, this field is ignored and only
                            // operation (3) is executed to clear an interrupt
   CEI_UINT64 pag_reg_val;  // (i) the low pag_reg.len_bytes of this value are
                            // written to the pag_reg register as detailed in
                            // pag_reg documentation above (this field is ignored
                            // unless clr_reg_opt includes CEI_CRO_PAG_REG_EN)
   CEI_UINT32 int_counter;  // (r) reserved
   CEI_UINT32 sig_counter;  // (r) reserved
   CEI_UINT64 listen_list;  // (r) reserved
   CEI_UINT64 spare_a;      // (r) reserved
   CEI_UINT64 spare_b;      // (r) reserved
   CEI_UINT64 spare_c;      // (r) reserved
} CEI_INT_STATUS_REG_V1,*pCEI_INT_STATUS_REG_V1;

// type:   CEI_INT_DISABLE_REG_V1
// usage:  This structure contains parameters used to update the value of a 
//         particular on-board register in order to disable device interrupt
//         generation capability.  The device's interrupt generation capability
//         is automatically disabled by the driver immediately before the last
//         process uninstalls the device's interrupt handler (preventing 
//         potential system lock-up due to the device generating an unhandled 
//         interrupt).  All fields marked (i) must be initialized by the caller 
//         before enabling interrupts.  All fields marked (r) are reserved for 
//         use by the CEI-Install library and driver, and should not be 
//         referenced by the caller.
typedef struct
{
   CEI_UINT64 cap_reg_mask; // (i) if chk_cap_reg is non-zero, this interrupt
                            // disable register is only processed if the value
                            // of the capability register bitwise ANDed with the
                            // low cap_reg.len_bytes of cap_reg_mask is non-zero
                            // (this field is ignored if chk_cap_reg is zero)
   CEI_ONBOARD_REG dis_reg; // (i) location and size of the dis_reg register - 
                            // this register contains a bit field to be cleared 
                            // when disabling device interrupts
   CEI_UINT64 dis_reg_opt;  // (i) contains CEI_DRO_xxx flag which controls
                            // execution of the interrupt disable operation for 
                            // this register - valid options are currently 
                            // CEI_DRO_AND or CEI_DRO_SET (see dis_reg_mask for 
                            // a discussion of these options)
   CEI_UINT64 dis_reg_mask; // (i) when disabling device interrupts, the low 
                            // dis_reg.len_bytes of this value are either (a)
                            // written to the dis_reg register (if dis_reg_opt is
                            // CEI_DRO_SET), or (b) bitwise ANDed with the 
                            // current value of the dis_reg register and written
                            // back to the dis_reg register (if dis_reg_opt is
                            // CEI_DRO_AND)
   CEI_UINT64 spare_a;      // (r) reserved
   CEI_UINT64 spare_b;      // (r) reserved
   CEI_UINT64 spare_c;      // (r) reserved
} CEI_INT_DISABLE_REG_V1,*pCEI_INT_DISABLE_REG_V1;

// type:   CEI_INT_CONTROL_V1
// usage:  This structure contains parameters supporting interrupt management.
//         All fields marked (i) must be initialized by the caller before 
//         enabling interrupts.  All fields marked (r) are reserved for use by 
//         the CEI-Install library and driver, and should not be referenced by 
//         the caller.  
typedef struct
{
   CEI_UINT32 struct_ver;   // (r) reserved
   CEI_UINT32 spare_a;      // (r) reserved
   CEI_INSTALL_ERR err_dsc; // (r) reserved
   CEI_RES_ID device_id;    // (r) reserved
   CEI_UINT64 entry_ptr;    // (r) reserved
   CEI_UINT64 spare_b;      // (r) reserved
   CEI_UINT64 spare_c;      // (r) reserved
   CEI_UINT64 spare_d;      // (r) reserved
   CEI_UINT64 chk_cap_reg;  // (i) set to non-zero to enable capability register
                            // checking for all interrupt registers (see desc of
                            // the cap_reg_mask field in CEI_INT_STATUS_REG_V1
                            // and CEI_INT_DISABLE_REG_V1 structures for 
                            // details), zero otherwise
   CEI_ONBOARD_REG cap_reg; // (i) location and size of the cap_reg register
                            // (this field is ignored if chk_cap_reg is zero)
   CEI_UINT64 add_chk_cnt;  // (i) directs the interrupt handler to cycle
                            // through all interrupt status register processing
                            // structures repeatedly until add_chk_cnt
                            // iterations through the list can be completed
                            // without detecting an interrupt (legacy flag
                            // supporting the ISA-1553) - set this field to 0
                            // for normal operation
   CEI_UINT64 num_status_reg; // (i) number of int status register processing
                            // structures required to fully monitor int status
   CEI_INT_STATUS_REG_V1 status_reg[CEI_MAX_INT_REGS]; // (i) list of interrupt
                            // status register processing structures
   CEI_UINT64 num_disable_reg; // (i) number of int disable register processing
                            // structures required to fully disable the device's
                            // ability to generate interrupts
   CEI_INT_DISABLE_REG_V1 disable_reg[CEI_MAX_INT_REGS]; // (i) list of int 
                            // disable register processing structures
} CEI_INT_CONTROL_V1,*pCEI_INT_CONTROL_V1;

// type:   CEI_INT_CONTROL_VER_VX
// usage:  Constant version indicator associated with each version of the 
//         CEI_INT_CONTROL structure.
enum CEI_INT_CONTROL_VER_VX
{
   CEI_INT_CONTROL_VER_V1 = 1,
   CEI_INT_CONTROL_VER_LAST = 1,
};


// type:   PTR_CEI_DLL_<x> 
// usage:  These types represent pointers to functions that are exported by
//         the installation library via dynamic linking.  For documentation of
//         each function, refer to function comment headers below.
typedef CEI_INT32 (DLL_EXPORTED *PTR_CEI_DLL_INIT_LIBRARY)(CEI_CHAR *clientLibraryVer);
typedef CEI_INT32 (DLL_EXPORTED *PTR_CEI_DLL_RUN_ERROR_PROBE)(CEI_CHAR *errorList,CEI_INT32 errorListCapacity,CEI_INT32 *numErrors);
typedef CEI_INT32 (DLL_EXPORTED *PTR_CEI_DLL_RUN_INSTALL_ERROR_CHECK)(CEI_VOID *desc,CEI_CHAR *outputFilename,CEI_INT32 *numErrors);
typedef CEI_INT32 (DLL_EXPORTED *PTR_CEI_DLL_GET_FILE_LIST_EX)(CEI_UINT32 listVer,CEI_FILE_LIST *fileList);
typedef CEI_INT32 (DLL_EXPORTED *PTR_CEI_DLL_GET_REGISTRY_KEY_LIST_EX)(CEI_UINT32 listVer,CEI_REG_KEY_LIST *keyList);
typedef CEI_INT32 (DLL_EXPORTED *PTR_CEI_DLL_GET_PRODUCT_LIST_EX)(CEI_UINT32 listVer,CEI_PRODUCT_LIST *list,CEI_INT32 failIfUnknownDevs);
typedef CEI_VOID (DLL_EXPORTED *PTR_CEI_DLL_GET_LAST_ERROR)(CEI_CHAR *errorText,CEI_INT32 errorTextCapacity);
typedef CEI_VOID (DLL_EXPORTED *PTR_CEI_DLL_GET_VERSION)(CEI_CHAR *versionText,CEI_INT32 versionTextCapacity);
typedef CEI_INT32 (DLL_EXPORTED *PTR_CEI_DLL_OPEN_LOGICAL_DEVICE_WINDOWS)(CEI_INT32 logicalDevId,CEI_INT32 optionFlags,HANDLE *driverHandle,CEI_INSTALL_ERR err);
typedef CEI_INT32 (DLL_EXPORTED *PTR_CEI_DLL_CLOSE_LOGICAL_DEVICE_WINDOWS)(HANDLE *driverHandle,CEI_INT32 optionFlags,CEI_INSTALL_ERR err);
typedef CEI_INT32 (DLL_EXPORTED *PTR_CEI_DLL_GET_SYSTEM_VERSION_EX)(CEI_INT64 *osVer,CEI_CHAR *osVerDesc,CEI_INT32 osVerDescMax);
typedef CEI_INT32 (DLL_EXPORTED *PTR_CEI_DLL_IS_ADMIN)(CEI_INT32 *is_elevated,CEI_INSTALL_ERR err);
typedef CEI_INT32 (DLL_EXPORTED *PTR_CEI_DLL_SETUP_SHARED_FILE)(CEI_INT32 is_install,CEI_CHAR *source_file,CEI_CHAR *target_file,CEI_INT32 *fail_share,CEI_INSTALL_ERR err);
typedef CEI_INT32 (DLL_EXPORTED *PTR_CEI_DLL_GET_FOLDER_PATH)(CEI_INT32 folder_id,CEI_CHAR *folder_buf,CEI_INT32 folder_buf_max,CEI_INSTALL_ERR err);
typedef CEI_INT32 (DLL_EXPORTED *PTR_CEI_DLL_OPEN_GLOBAL_NAMED_OBJECT)(CEI_UINT32 obj_type,CEI_UINT64 obj_param,pCEI_CHAR obj_name,pCEI_UINT32 preexisted_ptr,pCEI_VOID *block_ptr,HANDLE *obj_handle,CEI_INSTALL_ERR err);
typedef CEI_INT32 (DLL_EXPORTED *PTR_CEI_DLL_CLOSE_GLOBAL_NAMED_OBJECT)(HANDLE obj_handle,pCEI_UINT32 persists_ptr,CEI_INSTALL_ERR err);
typedef CEI_INT32 (DLL_EXPORTED *PTR_LL_OPEN_DEVICE_BY_LOGICAL_ID)(CEI_UINT32 logical_dev_id,pCEI_DEV_HANDLE dev_handle_ptr,CEI_INSTALL_ERR err);
typedef CEI_INT32 (DLL_EXPORTED *PTR_LL_CLOSE_DEVICE)(CEI_DEV_HANDLE dev_handle,CEI_INSTALL_ERR err);
typedef CEI_INT32 (DLL_EXPORTED *PTR_LL_MAP_MEMORY_REGIONS)(CEI_DEV_HANDLE dev_handle,CEI_UINT32 dev_mem_ver,CEI_UINT64 options,pCEI_DEVICE_MEM dev_mem_ptr,pCEI_UINT32 preexisted_ptr,CEI_INSTALL_ERR err);
typedef CEI_INT32 (DLL_EXPORTED *PTR_LL_UNMAP_MEMORY_REGIONS)(CEI_DEV_HANDLE dev_handle,pCEI_UINT32 persists_ptr,CEI_INSTALL_ERR err);
typedef CEI_INT32 (DLL_EXPORTED *PTR_LL_HOOK_INTERRUPT)(CEI_DEV_HANDLE dev_handle,pCEI_VOID int_ctl_ptr,pCEI_UINT32 preexisted_ptr,CEI_INSTALL_ERR err);
typedef CEI_INT32 (DLL_EXPORTED *PTR_LL_UNHOOK_INTERRUPT)(CEI_DEV_HANDLE dev_handle,pCEI_UINT32 persists_ptr,CEI_INSTALL_ERR err);
typedef CEI_INT32 (DLL_EXPORTED *PTR_LL_WAIT_FOR_INTERRUPT)(CEI_DEV_HANDLE dev_handle,CEI_UINT64 wait_on_regs,pCEI_UINT32 wait_result,pCEI_UINT64 int_rcvd_regs,pCEI_UINT64 chk_reg_val,CEI_INSTALL_ERR err);
typedef CEI_INT32 (DLL_EXPORTED *PTR_LL_INIT_INTERRUPT_CONTROL_STRUCT)(CEI_DEV_HANDLE dev_handle,CEI_UINT32 int_ctl_ver,CEI_UINT32 int_ctl_size,pCEI_VOID int_ctl_ptr,CEI_INSTALL_ERR err);
typedef CEI_INT32 (DLL_EXPORTED *PTR_LL_MAP_CONTIGUOUS_DMA_BUFFER)(CEI_DEV_HANDLE dev_handle,CEI_UINT32 min_size,CEI_UINT32 allow_phys_64,CEI_RES_ID buf_unique_id,pCEI_VOID *user_addr_ptr,pCEI_UINT64 phys_addr_ptr,pCEI_UINT32 actual_size_ptr,pCEI_UINT32 preexisted_ptr,CEI_INSTALL_ERR err);
typedef CEI_INT32 (DLL_EXPORTED *PTR_LL_UNMAP_CONTIGUOUS_DMA_BUFFER)(CEI_DEV_HANDLE dev_handle,pCEI_VOID user_addr,pCEI_UINT32 persists_ptr,CEI_INSTALL_ERR err);
typedef CEI_INT32 (DLL_EXPORTED *PTR_LL_DMA_TRANSFER_BEGIN)(CEI_DEV_HANDLE dev_handle,pCEI_VOID user_addr,CEI_INSTALL_ERR err);
typedef CEI_INT32 (DLL_EXPORTED *PTR_LL_DMA_TRANSFER_END)(CEI_DEV_HANDLE dev_handle,pCEI_VOID user_addr,CEI_INSTALL_ERR err);
typedef CEI_INT32 (DLL_EXPORTED *PTR_LL_READ_CONFIG_SPACE)(CEI_DEV_HANDLE dev_handle,CEI_UINT32 offset,CEI_UINT32 num_bytes,pCEI_UINT32 value_ptr,CEI_INSTALL_ERR err);
typedef CEI_INT32 (DLL_EXPORTED *PTR_LL_GET_CAPABILITY)(CEI_DEV_HANDLE dev_handle,CEI_UINT32 cap_type,pCEI_UINT32 cap_value_ptr,CEI_INSTALL_ERR err);


// type:   CEI_INSTALL_MODULE
// usage:  Contains pointers to all functions exported by the installation
//         library when dynamic linking is used.
typedef struct 
{
    pCEI_VOID LibraryHandle;
    PTR_CEI_DLL_INIT_LIBRARY InitLibrary;
    PTR_CEI_DLL_RUN_ERROR_PROBE RunErrorProbe;
    PTR_CEI_DLL_RUN_INSTALL_ERROR_CHECK RunInstallErrorCheck;
    PTR_CEI_DLL_GET_FILE_LIST_EX GetFileListEx;
    PTR_CEI_DLL_GET_REGISTRY_KEY_LIST_EX GetRegistryKeyListEx;
    PTR_CEI_DLL_GET_PRODUCT_LIST_EX GetProductListEx;
    PTR_CEI_DLL_GET_LAST_ERROR GetLastError;
    PTR_CEI_DLL_GET_VERSION GetVersion;
    PTR_CEI_DLL_OPEN_LOGICAL_DEVICE_WINDOWS OpenLogicalDeviceWindows;
    PTR_CEI_DLL_CLOSE_LOGICAL_DEVICE_WINDOWS CloseLogicalDeviceWindows;
    PTR_CEI_DLL_GET_SYSTEM_VERSION_EX GetSystemVersionEx;
    PTR_CEI_DLL_IS_ADMIN IsAdmin;
    PTR_CEI_DLL_SETUP_SHARED_FILE SetupSharedFile;
    PTR_CEI_DLL_GET_FOLDER_PATH GetFolderPath;
    PTR_CEI_DLL_OPEN_GLOBAL_NAMED_OBJECT OpenGlobalNamedObject;
    PTR_CEI_DLL_CLOSE_GLOBAL_NAMED_OBJECT CloseGlobalNamedObject;
    PTR_LL_OPEN_DEVICE_BY_LOGICAL_ID LL_OpenDeviceByLogicalId;
    PTR_LL_CLOSE_DEVICE LL_CloseDevice;
    PTR_LL_MAP_MEMORY_REGIONS LL_MapMemoryRegions;
    PTR_LL_UNMAP_MEMORY_REGIONS LL_UnmapMemoryRegions;
    PTR_LL_HOOK_INTERRUPT LL_HookInterrupt;
    PTR_LL_UNHOOK_INTERRUPT LL_UnhookInterrupt;
    PTR_LL_WAIT_FOR_INTERRUPT LL_WaitForInterrupt;
    PTR_LL_INIT_INTERRUPT_CONTROL_STRUCT LL_InitInterruptControlStruct;
    PTR_LL_MAP_CONTIGUOUS_DMA_BUFFER LL_MapContiguousDmaBuffer;
    PTR_LL_UNMAP_CONTIGUOUS_DMA_BUFFER LL_UnmapContiguousDmaBuffer;
    PTR_LL_DMA_TRANSFER_BEGIN LL_DmaTransferBegin;
    PTR_LL_DMA_TRANSFER_END LL_DmaTransferEnd;
    PTR_LL_READ_CONFIG_SPACE LL_ReadConfigSpace;
    PTR_LL_GET_CAPABILITY LL_GetCapability;
} CEI_INSTALL_MODULE;


// include dynamic-link support (if requested)
#ifdef CEI_INSTALL_DYNAMIC_LINK_SUPPORT


/*===========================================================================*
 * ENTRY POINT: CEI_DLL_GetDynamicLink
 *===========================================================================*
 *
 * FUNCTION:    Return a pointer to a structure that can be used to interface
 *              dynamically with the installation utility library (removing
 *              the need to statically link to the library).  Dynamically 
 *              linking to the library allows the caller to present meaningful
 *              error messages in the event that the cei_install library is 
 *              missing or incompatible.
 *
 * RETURN VAL:  NULL    failed to bind to the installation library (error desc
 *                      stored in 'err')
 *              other   pointer to library interface structure (see definition
 *                      of CEI_INSTALL_MODULE for details)
 *
 * DESCRIPTION: If we haven't already bound to the installation library and
 *              exported functions, attempt to load and initialize the
 *              library, obtain pointers to library functions, and return a 
 *              pointer to the resulting interface structure (which is stored
 *              statically in this function for repeat use).  If we're already
 *              bound as a result of a prior successful call to this function,
 *              a pointer to the same interface structure is returned.  Store 
 *              error description in 'err' upon failure.
 *
 *===========================================================================*/
CEI_INSTALL_MODULE *CEI_DLL_GetDynamicLink
(
    CEI_INT32 optionFlags,   // (i) currently unused (must specify 0)
    CEI_INSTALL_ERR err      // (o) receives error description (upon failure)
)
{
    CEI_INT32 i;             // loop counter
    CEI_INSTALL_ERR lastErr; // receives last library error desc

    // receives pointers to dynamically-exported functions
    static CEI_INSTALL_MODULE ceiLib;

    // true if we're already bound to the libray, false otherwise
    static CEI_INT32 libInit = FALSE;

    // receives true upon structure size error (volatile to exclude from 
    // optimization and prevent pursuant 'unreachable code' warning)
    volatile CEI_INT32 is_struct_err; 

    // name and pointer for each dynamically-exported library function
    static struct
    {
        CEI_CHAR *funcName;
        CEI_VOID *funcPtr;
    } libFunc[] = 
    { 
        { "CEI_DLL_InitLibrary",               &ceiLib.InitLibrary                   },
        { "CEI_DLL_RunErrorProbe",             &ceiLib.RunErrorProbe                 },
        { "CEI_DLL_RunInstallErrorCheck",      &ceiLib.RunInstallErrorCheck          },
        { "CEI_DLL_GetFileListEx",             &ceiLib.GetFileListEx                 },
        { "CEI_DLL_GetRegistryKeyListEx",      &ceiLib.GetRegistryKeyListEx          },
        { "CEI_DLL_GetProductListEx",          &ceiLib.GetProductListEx              },
        { "CEI_DLL_GetLastError",              &ceiLib.GetLastError                  },
        { "CEI_DLL_GetVersion",                &ceiLib.GetVersion                    },
        { "CEI_DLL_OpenLogicalDeviceWindows",  &ceiLib.OpenLogicalDeviceWindows      },
        { "CEI_DLL_CloseLogicalDeviceWindows", &ceiLib.CloseLogicalDeviceWindows     },
        { "CEI_DLL_GetSystemVersionEx",        &ceiLib.GetSystemVersionEx            },
        { "CEI_DLL_IsAdmin",                   &ceiLib.IsAdmin                       },
        { "CEI_DLL_SetupSharedFile",           &ceiLib.SetupSharedFile               },
        { "CEI_DLL_GetFolderPath",             &ceiLib.GetFolderPath                 },
        { "CEI_DLL_OpenGlobalNamedObject",     &ceiLib.OpenGlobalNamedObject         },
        { "CEI_DLL_CloseGlobalNamedObject",    &ceiLib.CloseGlobalNamedObject        },
        { "LL_OpenDeviceByLogicalId",          &ceiLib.LL_OpenDeviceByLogicalId      },
        { "LL_CloseDevice",                    &ceiLib.LL_CloseDevice                },
        { "LL_MapMemoryRegions",               &ceiLib.LL_MapMemoryRegions           },
        { "LL_UnmapMemoryRegions",             &ceiLib.LL_UnmapMemoryRegions         },
        { "LL_HookInterrupt",                  &ceiLib.LL_HookInterrupt              },
        { "LL_UnhookInterrupt",                &ceiLib.LL_UnhookInterrupt            },
        { "LL_WaitForInterrupt",               &ceiLib.LL_WaitForInterrupt           },
        { "LL_InitInterruptControlStruct",     &ceiLib.LL_InitInterruptControlStruct },
        { "LL_MapContiguousDmaBuffer",         &ceiLib.LL_MapContiguousDmaBuffer     },
        { "LL_UnmapContiguousDmaBuffer",       &ceiLib.LL_UnmapContiguousDmaBuffer   },
        { "LL_DmaTransferBegin",               &ceiLib.LL_DmaTransferBegin           },
        { "LL_DmaTransferEnd",                 &ceiLib.LL_DmaTransferEnd             },
        { "LL_ReadConfigSpace",                &ceiLib.LL_ReadConfigSpace            },
        { "LL_GetCapability",                  &ceiLib.LL_GetCapability              }
    };

    // nothing to do if we're already bound to the library
    if (libInit == TRUE)
        return &ceiLib;

    // clear alignment error flag
    is_struct_err = 0;

    // determine if we have a structure size or alignment error    
    is_struct_err |= (sizeof(CEI_FILE_ENTRY) != 576);
    is_struct_err |= (sizeof(CEI_FILE_LIST) != 589828);
    is_struct_err |= (sizeof(CEI_KEY_ENTRY) != 641);
    is_struct_err |= (sizeof(CEI_REG_KEY_LIST) != 164100);
    is_struct_err |= (sizeof(CEI_DEVICE_TYPE) != 68);
    is_struct_err |= (sizeof(CEI_PRODUCT_DESC) != 4800);
    is_struct_err |= (sizeof(CEI_PRODUCT_LIST) != 307204);
    is_struct_err |= (sizeof(CEI_ONBOARD_REG) != 32);
    is_struct_err |= (sizeof(CEI_INT_STATUS_REG_V1) != 192);
    is_struct_err |= (sizeof(CEI_INT_DISABLE_REG_V1) != 80);
    is_struct_err |= (sizeof(CEI_INT_CONTROL_V1) != 9384);
    #ifdef _WIN64
    is_struct_err |= (sizeof(CEI_DEVICE_MEM) != 392);
    is_struct_err |= (sizeof(CEI_INSTALL_MODULE) != 248);
    #else
    is_struct_err |= (sizeof(CEI_DEVICE_MEM) != 264);
    is_struct_err |= (sizeof(CEI_INSTALL_MODULE) != 124);
    #endif

    // make sure structure size and alignment looks good
    if (is_struct_err)
    {
        // store null-terminated error description and bail out
        _snprintf(err,CEI_INSTALL_MAX_ERR_DESC_LEN,"failed to bind to installation library '%s' (details: structure size or alignment error detected)",CEI_INSTALL_LIB_NAME);
        err[CEI_INSTALL_MAX_ERR_DESC_LEN - 1] = '\0';
        return NULL;
    }

    // attempt to load the library (handle closed automatically upon process shutdown)
    ceiLib.LibraryHandle = LoadLibrary(CEI_INSTALL_LIB_NAME);
    if (ceiLib.LibraryHandle == NULL)
    {
        // store null-terminated error description and bail out
        _snprintf(err,CEI_INSTALL_MAX_ERR_DESC_LEN,"failed to load installation library '%s' (last error = %ld) - reinstalling the product may resolve this error, or contact our technical support staff (support.avionics.ip@ge.com) for assistance",CEI_INSTALL_LIB_NAME,GetLastError());
        err[CEI_INSTALL_MAX_ERR_DESC_LEN - 1] = '\0';
        return NULL;
    }

    // resolve each dynamically-exported function
    for (i=0;i < (sizeof(libFunc) / sizeof(libFunc[0]));i++)
    {
        // attempt to resolve this function
        *((FARPROC *)libFunc[i].funcPtr) = GetProcAddress((HMODULE)ceiLib.LibraryHandle,libFunc[i].funcName);
        if ((*((FARPROC *)libFunc[i].funcPtr)) != NULL)
            continue;

        // store null-terminated error description and bail out
        _snprintf(err,CEI_INSTALL_MAX_ERR_DESC_LEN,"failed to locate the function '%s' in '%s' (last error = %ld) - reinstalling the product may resolve this error, or contact our technical support staff (support.avionics.ip@ge.com) for assistance",libFunc[i].funcName,CEI_INSTALL_LIB_NAME,GetLastError());
        err[CEI_INSTALL_MAX_ERR_DESC_LEN - 1] = '\0';
        return NULL;
    }

    // attempt to initialize the library
    if (ceiLib.InitLibrary(CEI_INSTALL_VERSION) == FALSE)
    {
        // store null-terminated error description and bail out
        ceiLib.GetLastError(lastErr,sizeof(lastErr));
        _snprintf(err,CEI_INSTALL_MAX_ERR_DESC_LEN,"failed to initialize '%s' (details: %s)",CEI_INSTALL_LIB_NAME,lastErr);
        err[CEI_INSTALL_MAX_ERR_DESC_LEN - 1] = '\0';
        return NULL;
    }

    // operation successful - return pointer to interface structure
    libInit = TRUE;
    return &ceiLib;
}

#endif


// begin the C-style linkage block (if necessary)
#ifdef __cplusplus
    extern "C" {
#endif


/*===========================================================================*
 * ENTRY POINT: CEI_DLL_RunInstallErrorCheck
 *===========================================================================*
 *
 * FUNCTION:    Build a list of potential error conditions affecting the
 *              specified pending installation.
 *
 * RETURN VAL:  TRUE    error-check completed successfully
 *              FALSE   failed to complete the error-check
 *
 * DESCRIPTION: Build a list of error and warning conditions that should be 
 *              announced to the user prior to executing the specified 
 *              installation. The newline-separated list is written to the 
 *              file associated with the given fully-qualified path.
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED CEI_DLL_RunInstallErrorCheck
(
    CEI_VOID *desc,           // (i) points to a structure describing the
                              //     installation being attempted (see the
                              //     generic installation script header file
                              //     condor.h for a description of this 
                              //     structure)
    CEI_CHAR *outputFilename, // (i) full path to the file that will receive 
                              //     the newline-separated list of error and 
                              //     warning conditions
    CEI_INT32 *numErrors      // (o) receives the number of newline-separated
                              //     entries written to the file
);


/*===========================================================================*
 * ENTRY POINT: CEI_DLL_InitLibrary
 *===========================================================================*
 *
 * FUNCTION:    Initialize the installation utility library.  This routine 
 *              must be called prior to calling any other installation library
 *              routines (except for CEI_DLL_RunInstallErrorCheck, which is
 *              called only from InstallShield script).
 *
 * RETURN VAL:  TRUE    installation utility library initialized successfully
 *              FALSE   failed to initialize the installation utility library
 *
 * DESCRIPTION: Check the installation utility library version that the caller
 *              is expecting.  This allows the library to indicate failure if
 *              an incompatibility exists (for example, if the client was 
 *              built against a more recent version of the installation 
 *              utility library than is currently running).
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED CEI_DLL_InitLibrary
(
    CEI_CHAR *clientLibraryVer  // (i) the constant CEI_INSTALL_VERSION from 
                                //     the header file cei_install.h
);


/*===========================================================================*
 * ENTRY POINT: CEI_DLL_RunErrorProbe
 *===========================================================================*
 *
 * FUNCTION:    Build a list of potential or actual error conditions that 
 *              affect existing CEI installations on the target machine.
 *
 * RETURN VAL:  TRUE    error-check completed successfully
 *              FALSE   failed to complete the error-check
 *
 * DESCRIPTION: Build a string containing a newline-separated list of error
 *              and warning conditions that affect current installations on 
 *              the target machine.
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED CEI_DLL_RunErrorProbe
(
    CEI_CHAR *errorList,         // (o) client buffer that receives the
                                 //     newline-separated list of error and
                                 //     warning conditions
    CEI_INT32 errorListCapacity, // (i) maximum number of bytes that can be
                                 //     written to the 'errorList' buffer
    CEI_INT32 *numErrors         // (o) receives number of newline-separated
                                 //     entries written to the buffer
);


/*===========================================================================*
 * ENTRY POINT: CEI_DLL_GetFileListEx
 *===========================================================================*
 *
 * FUNCTION:    Build a list of CEI-related files currently installed on the
 *              target machine.
 *
 * RETURN VAL:  TRUE    the file list was created successfully
 *              FALSE   failed to create the file list
 *
 * DESCRIPTION: The structure referenced by the supplied pointer is filled in
 *              with a list that enumerates each CEI-related file found on the
 *              target machine.
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED CEI_DLL_GetFileListEx
(
    CEI_UINT32 listVer,      // (i) the constant CEI_FILE_LIST_VER from the
                             //     header file cei_install.h
    CEI_FILE_LIST *fileList  // (o) pointer to client structure that receives
                             //     list of CEI-related files
);


/*===========================================================================*
 * ENTRY POINT: CEI_DLL_GetRegistryKeyListEx
 *===========================================================================*
 *
 * FUNCTION:    Build a list of CEI-related registry entries currently found
 *              on the target machine.
 *
 * RETURN VAL:  TRUE    the registry entry list was created successfully
 *              FALSE   failed to create the registry entry list
 *
 * DESCRIPTION: The structure referenced by the supplied pointer is filled in
 *              with a list that enumerates each CEI-related registry entry
 *              found on the target machine.
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED CEI_DLL_GetRegistryKeyListEx
(
    CEI_UINT32 listVer,       // (i) the constant CEI_REG_KEY_LIST_VER from
                              //     the header file cei_install.h
    CEI_REG_KEY_LIST *keyList // (o) pointer to client structure that receives
                              //     list of CEI-related registry entries
);


/*===========================================================================*
 * ENTRY POINT: CEI_DLL_GetProductListEx
 *===========================================================================*
 *
 * FUNCTION:    Build a list of CEI products installed on the target machine.
 *
 * RETURN VAL:  TRUE    product list was created successfully
 *              FALSE   failed to create the product list
 *
 * DESCRIPTION: Enumerate all CEI products installed on the target machine
 *              and fill out the given CEI_PRODUCT_LIST structure with the 
 *              resulting list of installed products.
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED CEI_DLL_GetProductListEx
(
    CEI_UINT32 listVer,         // (i) the constant CEI_PRODUCT_LIST_VER from
                                //     the header file cei_install.h
    CEI_PRODUCT_LIST *list,     // (o) receives list of installed CEI products
    CEI_INT32 failIfUnknownDevs // (i) if true, this function halts and 
                                //     indicates failure if an unknown device
                                //     is detected within a known hardware 
                                //     product group, otherwise unknown 
                                //     devices are appended to the product 
                                //     list (the hwDeviceType.typeID field in
                                //     the CEI_PRODUCT_DESC struct is set to
                                //     CEI_HW_UNKNOWN_DEVICE)
);


/*===========================================================================*
 * ENTRY POINT: CEI_DLL_GetLastError
 *===========================================================================*
 *
 * FUNCTION:    Store a description of the most recent internal error that
 *              prevented a utility routine from completing successfully (if
 *              any). 
 *
 * RETURN VAL:  none
 *
 * DESCRIPTION: Copy the string describing the most recent internal error into
 *              the supplied client buffer.  The stored string is truncated if
 *              the client buffer is too small.
 *
 *===========================================================================*/
CEI_VOID DLL_EXPORTED CEI_DLL_GetLastError
(
    CEI_CHAR *errorText,         // (o) client buffer that receives the error
                                 //     description string
    CEI_INT32 errorTextCapacity  // (i) maximum number of bytes that can be
                                 //     written to the 'errorText' buffer
);


/*===========================================================================*
 * ENTRY POINT: CEI_DLL_GetVersion
 *===========================================================================*
 *
 * FUNCTION:    Store the current library version (e.g. "1.10") in the
 *              supplied buffer.
 *
 * RETURN VAL:  none
 *
 * DESCRIPTION: Copy the current installation library version string into the 
 *              supplied client buffer.  The stored string is truncated if the
 *              client buffer is too small.
 *
 *===========================================================================*/
CEI_VOID DLL_EXPORTED CEI_DLL_GetVersion
(
    CEI_CHAR *versionText,        // (o) client buffer that receives the 
                                  //     installation library version string
    CEI_INT32 versionTextCapacity // (i) maximum number of bytes that can be
                                  //     written to the 'versionText' buffer
);


/*===========================================================================*
 * ENTRY POINT: CEI_DLL_CalcUpperAddr 
 *===========================================================================*
 *
 * FUNCTION:    Calculate the upper limit of the specified memory region.
 *
 * RETURN VAL:  0       the upper limit was calculated successfully
 *              -1      failed to calculate the upper limit
 *
 * DESCRIPTION: Given the base address and the size of a memory region, 
 *              return the string representation of the upper limit of the 
 *              memory region.
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED CEI_DLL_CalcUpperAddr 
(
    CEI_CHAR *baseAddr,      // (i) string representation of the base address
                             //     of the memory region
    CEI_CHAR *regionSize,    // (i) string representation of the size of the 
                             //     memory region
    CEI_CHAR *endingAddr     // (o) receives the string representation of the 
                             //     memory region upper limit
);


/*===========================================================================*
 * ENTRY POINT: CEI_DLL_FormatUninstallString
 *===========================================================================*
 *
 * FUNCTION:    This routine formats the executable and parameter uninstall 
 *              strings required to uninstall an existing CEI S/W product from
 *              the system using the InstallShield LaunchAppAndWait script
 *              function.
 *
 * RETURN VAL:  TRUE    all returned strings were successfully defined
 *              FALSE   an error occurred attempting to define the return 
 *                      strings
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED CEI_DLL_FormatUninstallString 
( 
    CEI_CHAR *regKeyBoardId, // (i) Windows Registry key path for the 
                             //     installation product's Hardware 
                             //     identification key. Typically defined as 
                             //     something like: "Software\Condor 
                             //     Engineering\CEI-x20-SW\Hardware\Board0"
    CEI_CHAR *appString,     // (o) InstallShield uninstall executable file  
                             //     path/name string, typically defined as: 
                             //     "C:\WINDOWS\IsUninst.exe"
    CEI_CHAR *paramString,   // (o) uninstall executable parameter list 
                             //     string, typically defined something like: 
                             //     -f"C:\Program Files\Condor Engineering\
                             //     CEI-715-SW\Condor Engineering CEI-715 
                             //     Board0 (Dev 0)"
    CEI_CHAR *devNum,        // (o) the device number character assigned to a 
                             //     current API single-board installation, 
                             //     "0" to "9"
    CEI_CHAR *productIdent   // (o) the product identifier assigned to a 
                             //     current API single-board installation, 
                             //     such as "CEI-520"
);


/*===========================================================================*
 * ENTRY POINT: CEI_DLL_IsAdmin
 *===========================================================================*
 *
 * FUNCTION:    Check for admin privileges.
 *
 * RETURN VAL:  TRUE    admin status determined successfully
 *              FALSE   failed to determine admin status - error description 
 *                      stored in 'err'
 *
 * DESCRIPTION: Determine if the current process is running with elevated
 *              admin privileges.  This function is not supported under 
 *              Windows 95/98/Me or Windows NT.
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED CEI_DLL_IsAdmin
( 
    CEI_INT32 *is_elevated,  // (o) upon successful return, receives true if 
                             //     we're running as an admin with elevated 
                             //     privileges, false otherwise
    CEI_INSTALL_ERR err      // (o) receives error description (upon failure)
);


/*===========================================================================*
 * ENTRY POINT: CEI_DLL_SetupSharedFile
 *===========================================================================*
 *
 * FUNCTION:    Install/uninstall the given shared file.
 *
 * RETURN VAL:  TRUE    shared file setup completed successfully
 *              FALSE   failed to setup shared file - error description 
 *                      stored in 'err'
 *
 * DESCRIPTION: Install/uninstall the given shared file to/from the given
 *              target location and update the shared file's counter value in
 *              the CEI_SHARED_DLL_KEY_32 key accordingly.  If uninstalling,
 *              the shared file is only removed when the resulting shared file
 *              count reaches zero.  If installing and target_file already 
 *              exists, target_file is only overwritten with source_file if
 *              source_file has a more recent version or has the same version
 *              and a more recent file timestamp.
 *
 *              The caller must be running with elevated admin privileges.  
 *              This function is not supported under Windows 95/98/Me or
 *              Windows NT.  
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED CEI_DLL_SetupSharedFile
( 
    CEI_INT32 is_install,   // (i) true to install file, false to uninstall
    CEI_CHAR *source_file,  // (i) if 'is_install' is true, contains full 
                            //     path to the file to be installed (e.g., 
                            //     'c:\temp\my_install\myfile.sys') - ignored
                            //     if 'is_install' is false
    CEI_CHAR *target_file,  // (i) contains full post-install/pre-uninstall 
                            //     file path, may begin with '%sys32%' to
                            //     denote the 32-bit system file folder or
                            //     '%sys64%' to denote the 64-bit system file
                            //     folder (e.g., '%sys64%\drivers\myfile.sys')
    CEI_INT32 *fail_share,  // (o) if the operation fails due to a permission
                            //     or sharing violation (e.g., if target_file 
                            //     is currently open within another program or
                            //     is not writable), 'fail_share' is set to 
                            //     true, otherwise 'fail_share' is set to 
                            //     false - this field is only valid if
                            //     CEI_DLL_SetupSharedFile returns false
    CEI_INSTALL_ERR err     // (o) receives error description (upon failure)
);


/*===========================================================================*
 * ENTRY POINT: CEI_DLL_GetFolderPath
 *===========================================================================*
 *
 * FUNCTION:    Obtain the current path to the given folder.
 *
 * RETURN VAL:  TRUE    path obtained successfully
 *              FALSE   failed to obtain path - error desc stored in 'err'
 *
 * DESCRIPTION: Store the current full path to folder 'folder_id' in the 
 *              supplied 'folder_buf' buffer.  The path is returned without
 *              a trailing backslash (e.g., 'c:\Program Files').
 *
 *              The following 'folder_id' values are currently supported:
 *
 *                           id                       desc
 *                 ---------------------  -----------------------------
 *                 CFI_SYSTEM_X86         32-bit system folder
 *                 CFI_SYSTEM_X64         64-bit system folder
 *                 CFI_PROGRAM_FILES_X86  32-bit 'Program Files' folder
 *                 CFI_PROGRAM_FILES_X64  64-bit 'Program Files' folder
 *                 CFI_COMMON_DOCUMENTS   Base common documents folder
 *
 *              Note that attempting to obtain the path to a 64-bit folder 
 *              (e.g., CFI_PROGRAM_FILES_X64) under a 32-bit operating system
 *              causes an error indication.  Check if the current operating
 *              system is 64-bit (using CEI_DLL_GetSystemVersionEx) before 
 *              calling this routine.
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED CEI_DLL_GetFolderPath
( 
    CEI_INT32 folder_id,      // (i) obtain path to this folder (CFI_xxx)
    CEI_CHAR *folder_buf,     // (o) buffer that receives the folder path
    CEI_INT32 folder_buf_max, // (i) maximum number of bytes that can be
                              //     written to the 'folder_buf' buffer
    CEI_INSTALL_ERR err       // (o) receives error description (upon failure)
);


/*===========================================================================*
 * ENTRY POINT: CEI_DLL_OpenLogicalDeviceWindows
 *===========================================================================*
 *
 * FUNCTION:    Open a WinRT device associated with a logical device ID.
 *
 * RETURN VAL:  TRUE    driver handle opened successfully
 *              FALSE   failed to open driver handle - error description 
 *                      stored in 'err'
 *
 * DESCRIPTION: This routine is provided for backwards compatibility only. 
 *              It has been deprecated and should no longer be used.
 *
 *              Instead, use the LL_<x> family of routines that comprise the 
 *              common driver-independent API/driver interface under Windows.
 *
 *              This routine only supports devices which are associated with 
 *              a WinRT driver.
 *
 *              Call CEI_DLL_CloseLogicalDeviceWinRT to close a driver handle
 *              that was opened using this function.
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED CEI_DLL_OpenLogicalDeviceWindows
(
    CEI_INT32 logicalDevId,  // (i) open a handle to the driver associated with
                             //     the device instance identified by this 
                             //     logical device ID
    CEI_INT32 optionFlags,   // (i) currently unused (must specify 0)
    HANDLE *driverHandle,    // (o) receives driver handle (upon successful 
                             //     return)
    CEI_INSTALL_ERR err      // (o) receives error description (upon failure)
);


/*===========================================================================*
 * ENTRY POINT: CEI_DLL_CloseLogicalDeviceWindows
 *===========================================================================*
 *
 * FUNCTION:    Close the given WinRT driver handle.
 *
 * RETURN VAL:  TRUE    driver handle closed successfully
 *              FALSE   failed to close driver handle - error description 
 *                      stored in 'err'
 *
 * DESCRIPTION: This routine is provided for backwards compatibility only. 
 *              It has been deprecated and should no longer be used.
 *
 *              Instead, use the LL_<x> family of routines that comprise the 
 *              common driver-independent API/driver interface under Windows.
 *
 *              This routine can only be used to close a driver handle that
 *              was previously opened using CEI_DLL_OpenLogicalDeviceWindows.
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED CEI_DLL_CloseLogicalDeviceWindows
(
    HANDLE *driverHandle,    // (i) driver handle to close
    CEI_INT32 optionFlags,   // (i) currently unused (must specify 0)
    CEI_INSTALL_ERR err      // (o) receives error description (upon failure)
);


/*===========================================================================*
 * ENTRY POINT: CEI_DLL_GetSystemVersionEx
 *===========================================================================*
 *
 * FUNCTION:    Determine the current Windows operating system version. 
 *
 * RETURN VAL:  TRUE    operating system info determined successfully
 *              FALSE   failed to determine operating system version info    
 *
 * DESCRIPTION: Store current operating system version bitfield and textual 
 *              description.
 *
 *              A bitfield containing the operating system family, version,
 *              and architecture is returned in 'osVer'.  The currently-
 *              supported family and version values are:
 *
 *                    family              version
 *                 -------------     ------------------
 *                 CEI_OS_FAM_9X     CEI_OS_95_GOLD
 *                                   CEI_OS_95_OSR2
 *                                   CEI_OS_98_STD
 *                                   CEI_OS_98_SE
 *                                   CEI_OS_ME
 *                 CEI_OS_FAM_NT     CEI_OS_NT
 *                 CEI_OS_FAM_2000   CEI_OS_2000
 *                                   CEI_OS_XP
 *                                   CEI_OS_SERVER_2003
 *                 CEI_OS_FAM_VISTA  CEI_OS_VISTA
 *                                   CEI_OS_SERVER_2008
 *                                   CEI_OS_WIN7
 *             
 *              The currently-supported target architecture values are:
 *
 *                  architecture                desc
 *                 --------------    --------------------------
 *                 CEI_OS_ARCH_64    operating system is 64-bit
 *                 CEI_OS_ARCH_32    operating system is 32-bit
 *                         
 *              All flags can be extracted from 'osVer' using the bitwise and 
 *              operator.
 *
 *              A textual description of the current operating system version
 *              (e.g., "Windows XP") is returned in 'osVerDesc'.
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED CEI_DLL_GetSystemVersionEx
(
   CEI_INT64 *osVer,         // (o) receives current operating system version 
                             //     bitfield (see above for details), or null
                             //     if no bitfield is required
   CEI_CHAR *osVerDesc,      // (o) client buffer that receives a textual desc
                             //     of the current operating system version, 
                             //     or null if no desc is required
   CEI_INT32 osVerDescMax    // (i) maximum number of bytes that can be
                             //     written to the 'osVerDesc' buffer
);


/*===========================================================================*
 * ENTRY POINT: CEI_DLL_OpenGlobalNamedObject
 *===========================================================================*
 *
 * FUNCTION:    Open a shared global named object.
 *
 * RETURN VAL:  TRUE    operation completed successfully
 *              FALSE   operation failed - error description stored in 'err'
 *
 * DESCRIPTION: Open a handle to the given type of shared global named object.
 *
 *              The Windows SDK function used by this routine to open the 
 *              handle depends upon the object type, as listed below:
 *
 *                  object type    opened using this Windows SDK routine
 *                 -------------   -------------------------------------
 *                 CEI_GSO_MUTEX   CreateMutex
 *                 CEI_GSO_EVENT   CreateEvent
 *                 CEI_GSO_SHMEM   CreateFileMapping
 *
 *              Thus, a handle opened using CEI_DLL_OpenGlobalNamedObject may
 *              subsequently be passed by the caller to any Windows SDK 
 *              routine expecting a handle opened by the associated Create<x> 
 *              routine listed above.
 *
 *              Use CEI_DLL_CloseGlobalNamedObject to close any handle opened
 *              using CEI_DLL_OpenGlobalNamedObject.
 *
 *              Thus far, to open a shared global named object under Windows,
 *              common practice has typically been to call Create<x> and pass
 *              a null SECURITY_ATTRIBUTES parameter and an object name that 
 *              does not include a name space prefix.  The null attribute 
 *              parameter assigns a default security descriptor to the object,
 *              and the lack of a name space prefix assigns the object to the 
 *              default name space.
 *
 *              Changes introduced in recent versions of Windows make this 
 *              practice undesirable.  Objects created in the default name
 *              space are no longer truly system-global.  They are only global
 *              relative to the current user's login session.  For example, 
 *              two different users running at the same time on the same 
 *              machine (e.g., fast user switching) may successfully create
 *              identically-named shared global objects that are completely
 *              independent of each other (if created in the default name
 *              space).  If the application expects the shared object to be 
 *              system-global (and, for example, uses it to synchronize access
 *              to a single common hardware device), carnage is inevitable.
 *
 *              The CEI_DLL_OpenGlobalNamedObject routine addresses this issue
 *              by opening objects in the session-global name space (that is,
 *              "Global\" is prepended to the supplied object name) and by 
 *              opening each object with an "all access to any user" security 
 *              descriptor.  Thus, opened objects are truly system-global.
 *
 *              There is one exception, however.  Recent versions of Windows 
 *              do not allow a user app to open a named shared memory object
 *              in the session-global name space unless the app was run as 
 *              admin.  Since requiring admin rights in all cases is not
 *              practical, CEI_DLL_OpenGlobalNamedObject opens shared memory
 *              objects in session-local name space by default (no admin  
 *              rights required) but checks if an identically-named shared
 *              memory object already exists in another session.  If it does,
 *              the open fails and the user is advised to enable 'shared
 *              memory terminal services support mode' (which directs 
 *              CEI_DLL_OpenGlobalNamedObject to open shared memory objects in
 *              session-global name space (admin rights required).  Mutex and
 *              event objects are always opened in session-global name space
 *              by CEI_DLL_OpenGlobalNamedObject (admin rights not required).
 *
 *              Under Windows NT or earlier, CEI_DLL_OpenGlobalNamedObject 
 *              passes the supplied object name through unchanged to the 
 *              associated Create<x> call along with a default (null) 
 *              SECURITY_ATTRIBUTES parameter (shared named objects are
 *              system-global under Windows NT or earlier).
 *              
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED CEI_DLL_OpenGlobalNamedObject
(
    CEI_UINT32 obj_type,        // (i) open a shared global named object of
                                //     this type - supported obj_type values
                                //     are CEI_GSO_MUTEX (mutex), CEI_GSO_EVENT
                                //     (event), and CEI_GSO_SHMEM (shared 
                                //     memory block)
    CEI_UINT64 obj_param,       // (i) if obj_type is CEI_GSO_MUTEX, specifies
                                //     the bInitialOwner param to CreateMutex
                                //     in the low 32 bits - if obj_type is 
                                //     CEI_GSO_EVENT, specifies the 
                                //     bManualReset and bInitialState params to
                                //     CreateEvent in the high and low 32 bits,
                                //     respectively - if obj_type is 
                                //     CEI_GSO_SHMEM, specifies the desired
                                //     size (in bytes) of the memory block 
    pCEI_CHAR obj_name,         // (i) name of the shared global object
    pCEI_UINT32 preexisted_ptr, // (o) upon successful return, receives true if
                                //     the shared global named object already 
                                //     existed, false otherwise
    pCEI_VOID *block_ptr,       // (o) upon successful return, receives a 
                                //     pointer to the memory block if obj_type 
                                //     is CEI_GSO_SHMEM - ignored otherwise
    HANDLE *obj_handle,         // (o) receives object handle (upon success)
    CEI_INSTALL_ERR err         // (o) receives error desc (upon failure)
);


/*===========================================================================*
 * ENTRY POINT: CEI_DLL_CloseGlobalNamedObject
 *===========================================================================*
 *
 * FUNCTION:    Close a shared global named object handle.
 *
 * RETURN VAL:  TRUE    operation completed successfully
 *              FALSE   operation failed - error description stored in 'err'
 *
 * DESCRIPTION: Close the given handle to a shared global named object that 
 *              was previously opened using CEI_DLL_OpenGlobalNamedObject.
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED CEI_DLL_CloseGlobalNamedObject
(
    HANDLE obj_handle,          // (i) object handle previously opened using
                                //     CEI_DLL_OpenGlobalNamedObject
    pCEI_UINT32 persists_ptr,   // (o) upon successful return, receives true
                                //     if another handle to the shared global 
                                //     object remains open, false otherwise
    CEI_INSTALL_ERR err         // (o) receives error desc (upon failure)
);


/*===========================================================================*
 * ENTRY POINT: LL_OpenDeviceByLogicalId
 *===========================================================================*
 *
 * FUNCTION:    Open the device associated with a specific logical device ID.
 *
 * RETURN VAL:  TRUE    operation completed successfully 
 *              FALSE   operation failed - error description stored in 'err'
 *
 * DESCRIPTION: When installing the software distribution for a particular 
 *              board using our common API/driver installer under Windows, the
 *              user specifies the board type (e.g., CEI-520A, PCC-D1553,
 *              etc.) and a unique static logical device ID (0, 1, etc.). 
 *              When accessing a particular board via a product API, the user
 *              passes the associated logical device ID.
 *
 *              This routine opens a handle to the device associated with the
 *              board instance corresponding to the given logical device ID. 
 *              The resulting handle can then be used by the caller to execute
 *              other LL_xxx routines.
 *
 *              Under Windows NT (and under WDM for all non-PnP boards, when
 *              supported), this routine opens a handle to the device using a
 *              call to WinRTOpenDevice, passing the supplied device id 
 *              directly to WinRT.
 *
 *              For PnP boards under WDM and 64-bit Windows, all current board
 *              API installations are enumerated and the desired board type
 *              and instance (first, second, etc.) is determined using the
 *              supplied logical device ID.  All physically-installed instances
 *              of the desired board type are enumerated and a handle to the
 *              desired device instance is opened.  When searching for the
 *              desired device instance (first, second, etc.), PCI devices of
 *              the same type are enumerated in increasing order by PCI bus,
 *              device, and function, while PCMCIA devices of the same type are
 *              enumerated in the order the instances were reported by Windows.
 *              Note that PCMCIA is not currently supported under 64-bit
 *              Windows.
 *
 *              Call LL_CloseDevice to close any device handle opened with this
 *              function.
 * 
 * EXAMPLE:     CEI_DEV_HANDLE my_dev; // receives handle to the desired device
 *              CEI_INSTALL_ERR err;   // receives error desc upon failure
 *              ...
 *              // open handle to logical device id 0
 *              if (!LL_OpenDeviceByLogicalId(0,&my_dev,err))
 *                 return MyErrorHandler(err);
 *              ...
 *              // execute calls to other LL_xxx routines using my_dev
 *              ...
 *              // close the device handle
 *              if (!LL_CloseDevice(my_dev,err))
 *                 return MyErrorHandler(err);
 *              ...
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED LL_OpenDeviceByLogicalId
(
    CEI_UINT32 logical_dev_id,      // (i) open a handle to the device instance 
                                    //     identified by this logical device ID
    pCEI_DEV_HANDLE dev_handle_ptr, // (o) receives device handle (upon 
                                    //     successful return)
    CEI_INSTALL_ERR err             // (o) receives error desc (upon failure)
);


/*===========================================================================*
 * ENTRY POINT: LL_CloseDevice
 *===========================================================================*
 *
 * FUNCTION:    Close a device handle.
 *
 * RETURN VAL:  TRUE    operation completed successfully 
 *              FALSE   operation failed - error description stored in 'err'
 *
 * DESCRIPTION: Close the given device handle which was previously opened 
 *              using a call to LL_OpenDeviceByLogicalId.
 *
 * EXAMPLE:     Refer to LL_OpenDeviceByLogicalId.
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED LL_CloseDevice
(
    CEI_DEV_HANDLE dev_handle,      // (i) handle of device instance
    CEI_INSTALL_ERR err             // (o) receives error desc (upon failure)
);


/*===========================================================================*
 * ENTRY POINT: LL_MapMemoryRegions
 *===========================================================================*
 *
 * FUNCTION:    Map device memory regions.
 *
 * RETURN VAL:  TRUE    operation completed successfully 
 *              FALSE   operation failed - error description stored in 'err'
 *
 * DESCRIPTION: Map each device memory region into the address space of the
 *              calling process. 
 *              
 *              Obtain the number of memory regions supported by the given 
 *              device, and the size and user-mode address of each region.
 *
 *              When the user-mode process no longer requires access to the 
 *              device's memory region(s), call LL_UnmapMemoryRegions to unmap
 *              each memory region from the address space of the calling 
 *              process.
 *
 * EXAMPLE:     CEI_DEV_HANDLE my_dev; // obtained via LL_OpenDeviceByLogicalId
 *              CEI_DEVICE_MEM my_mem; // receives memory region config info
 *              CEI_UINT32 preexisted; // receives true if another process 
 *                                     // already has the region(s) mapped 
 *              CEI_UINT32 persists;   // receives true if another process 
 *                                     // still has the region(s) mapped 
 *              CEI_INSTALL_ERR err;   // receives error desc upon failure
 *              ...
 *              // map all device memory regions to user-space
 *              if (!LL_MapMemoryRegions(my_dev,CEI_DEVICE_MEM_VER,0,&my_mem,
 *                                       &preexisted,err))
 *                 return MyErrorHandler(err);
 *              
 *              // display device memory region config info
 *              printf("device supports %d memory regions:\n",my_mem.num_rgns);
 *              for (i=0;i < my_mem.num_rgns;i++)
 *              {
 *                 printf("rgn %d:\n",i);
 *                 printf("   length: %d bytes\n", my_mem.rgn_len_bytes[i]);
 *                 printf("   user address: 0x%x\n",my_mem.rgn_usr_addr[i]);
 *              }
 *              
 *              // indicate whether another process currently has the region(s) mapped
 *              if (preexisted)
 *                 printf("another process already has the device's region(s) mapped\n");
 *              else
 *                 printf("no other processes have the device's region(s) mapped\n");
 *              ...
 *              ...
 *              // unmap device memory regions
 *              if (!LL_UnmapMemoryRegions(my_dev,&persists,err))
 *                 return MyErrorHandler(err);
 *              
 *              // indicate whether another process still has the region(s) mapped
 *              if (persists)
 *                 printf("another process still has the device's region(s) mapped\n");
 *              else
 *                 printf("no other processes have the device's region(s) mapped\n");
 *              ...
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED LL_MapMemoryRegions
(
    CEI_DEV_HANDLE dev_handle,   // (i) handle of device instance
    CEI_UINT32 dev_mem_ver,      // (i) caller should pass the constant 
                                 //     CEI_DEVICE_MEM_VER (not the value of 
                                 //     the constant)
    CEI_UINT64 options,          // (i) currently unused, must be 0
    pCEI_DEVICE_MEM dev_mem_ptr, // (o) upon successful return, receives device 
                                 //     memory region configuration info (refer 
                                 //     to CEI_DEVICE_MEM documentation for 
                                 //     details)
    pCEI_UINT32 preexisted_ptr,  // (o) upon successful return, receives TRUE 
                                 //     if another process currently has the 
                                 //     device's memory region(s) mapped, FALSE
                                 //     otherwise
    CEI_INSTALL_ERR err          // (o) receives error desc (upon failure)
);


/*===========================================================================*
 * ENTRY POINT: LL_UnmapMemoryRegions
 *===========================================================================*
 *
 * FUNCTION:    Unmap device memory regions.
 *
 * RETURN VAL:  TRUE    operation completed successfully 
 *              FALSE   operation failed - error description stored in 'err'
 *
 * DESCRIPTION: Unmap all device memory regions (mapped earlier with 
 *              LL_MapMemoryRegions) from the address space of the calling 
 *              process.
 *
 *              Call this function when the user-mode process no longer 
 *              requires access to the device's memory region(s).
 *
 * EXAMPLE:     Refer to LL_MapMemoryRegions.
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED LL_UnmapMemoryRegions
(
    CEI_DEV_HANDLE dev_handle, // (i) handle of device instance
    pCEI_UINT32 persists_ptr,  // (o) upon successful return, receives TRUE if
                               //     another process still has the device's
                               //     memory region(s) mapped, FALSE otherwise
    CEI_INSTALL_ERR err        // (o) receives error desc (upon failure)
);


/*===========================================================================*
 * ENTRY POINT: LL_HookInterrupt
 *===========================================================================*
 *
 * FUNCTION:    Hook a device's hardware interrupt.
 *
 * RETURN VAL:  TRUE    operation completed successfully 
 *              FALSE   operation failed - error description stored in 'err'
 *
 * DESCRIPTION: Enable the calling process to monitor hardware interrupts on 
 *              the given device.
 *
 *              All processes utilizing hardware interrupts on a particular 
 *              device must call LL_HookInterrupt to initiate interrupt 
 *              control, LL_WaitForInterrupt to wait for interrupts to occur,
 *              and LL_UnhookInterrupt when interrupts are no longer required
 *              by the calling process. 
 *
 *              Only the first process to call LL_HookInterrupt for a 
 *              particular device (the "master" process) actually installs the 
 *              device interrupt handler.  Other processes ("minor" processes) 
 *              may subsequently startup and call LL_HookInterrupt to enable 
 *              interrupt monitoring via calls to LL_WaitForInterrupt, but the
 *              same "master" process that first called LL_HookInterrupt must 
 *              remain running until all "minor" processes have finished 
 *              monitoring interrupts and called LL_UnhookInterrupt or 
 *              terminated.  Only at this point may the "master" process call
 *              LL_UnhookInterrupt (disabling device interrupts and 
 *              uninstalling the actual interrupt handler).
 *
 *              All interrupt management functionality is configured using the 
 *              supplied CEI_INT_CONTROL_V1 structure.  For full details, refer
 *              to CEI_INT_CONTROL_V1, CEI_INT_STATUS_REG_V1, and 
 *              CEI_INT_DISABLE_REG_V1 documentation.  
 *
 *              If another process has already installed an interrupt handler 
 *              for the device in question (that is, the calling process is a 
 *              "minor" process), all parameters specified within 
 *              CEI_INT_CONTROL_V1 by both processes must match.  If not, the
 *              calling process's call to LL_HookInterrupt fails.
 *
 *              Before writing to the interrupt control structure, it must 
 *              first be initialized using a call to 
 *              LL_InitInterruptControlStruct.
 *
 *              When the calling process no longer needs to monitor hardware
 *              interrupts on the given device, call LL_UnhookInterrupt.
 *
 * EXAMPLE:     CEI_DEV_HANDLE my_dev;     // obtained via 
 *                                         // LL_OpenDeviceByLogicalId
 *              CEI_DEVICE_MEM my_mem;     // obtained via LL_MapMemoryRegions
 *              CEI_UINT32 ints_supported; // receives h/w int support status
 *              CEI_INT_CONTROL_V1 my_ic;  // initialize to configure int mgmt
 *              CEI_UINT32 preexisted;     // receives true if another process 
 *                                         // is already using the device's int
 *              CEI_UINT32 persists;       // receives true if another process
 *                                         // is still using the device's int
 *              CEI_INSTALL_ERR err;       // receives error desc upon failure
 *              ...
 *              // query hardware interrupt support capability
 *              if (!LL_GetCapability(my_dev,CEI_CAP_HW_INTS,&ints_supported,err))
 *                 return MyErrorHandler(err);
 *              
 *              // bail out if hardware interrupts are not supported
 *              if (!ints_supported)
 *                 return MyErrorHandler("Hardware interrupts are not supported.");
 *              
 *              // initialize the interrupt control structure
 *              if (!LL_InitInterruptControlStruct(my_dev,CEI_INT_CONTROL_VER_V1,
 *                                                 sizeof(my_ic),&my_ic,err))
 *                 return MyErrorHandler(err);
 *              
 *              // set main interrupt control parameters
 *              my_ic.chk_cap_reg = FALSE; // no capability register checking
 *              my_ic.add_chk_cnt = 0;     // no special ISA-1553 processing
 *              my_ic.num_status_reg = 1;  // device uses single int status reg
 *              my_ic.num_disable_reg = 1; // device uses single int disable reg
 *              
 *              // the device in this example has a single 4-byte interrupt 
 *              // status register at byte offset 0x400000 in the second memory
 *              // region, the low-bit is non-zero upon interrupt, no 
 *              // chk_reg_opt option flags needed
 *              my_ic.status_reg[0].chk_reg.rgn_usr_addr = my_mem.rgn_usr_addr[1];
 *              my_ic.status_reg[0].chk_reg.offset_bytes = 0x400000;
 *              my_ic.status_reg[0].chk_reg.len_bytes = 4;
 *              my_ic.status_reg[0].chk_reg_opt = CEI_XRO_NONE;
 *              my_ic.status_reg[0].chk_reg_mask = 1;
 *              
 *              // upon interrupt, bitwise AND the same register with 0xFFFFFFFC
 *              my_ic.status_reg[0].clr_reg_opt = CEI_CRO_USE_CHK_REG | CEI_CRO_AND;
 *              my_ic.status_reg[0].clr_reg_mask = 0xFFFFFFFC;
 *              
 *              // the device in this example has a single 4-byte interrupt 
 *              // disable register at byte offset 0x4C in the first memory 
 *              // region, bitwise AND the register with 0xFFFF0000 to disable
 *              // device interrupts
 *              my_ic.disable_reg[0].dis_reg.rgn_usr_addr = my_mem.rgn_usr_addr[0];
 *              my_ic.disable_reg[0].dis_reg.offset_bytes = 0x4C;
 *              my_ic.disable_reg[0].dis_reg.len_bytes = 4;
 *              my_ic.disable_reg[0].dis_reg_opt = CEI_DRO_AND;
 *              my_ic.disable_reg[0].dis_reg_mask = 0xFFFF0000;
 *              
 *              // install the interrupt handler
 *              if (!LL_HookInterrupt(my_dev,&my_ic,&preexisted,err))
 *                 return MyErrorHandler(err);
 *              
 *              // spin off a new thread which implements the following code:
 *              // {
 *              //    CEI_UINT64 wait_on_regs; // bitfield specifying which int
 *              //                             // status register(s) to wait on 
 *              //    CEI_UINT32 wait_result;  // receives wait operation result
 *              //    CEI_UINT64 int_on_regs;  // receives bitfield indicating 
 *              //                             // which int status register(s) 
 *              //                             // generated the int
 *              //    CEI_INSTALL_ERR err;     // receives error desc upon 
 *              //                             // failure
 *              //
 *              //    // only one int status reg in this example, wait on it
 *              //    wait_on_regs = 1;
 *              //
 *              //    // handle ints until the process calls LL_UnhookInterrupt
 *              //    while (TRUE)
 *              //    {
 *              //       // wait for a pending interrupt
 *              //       if (!LL_WaitForInterrupt(my_dev,wait_on_regs,&wait_result,
 *              //                                &int_on_regs,NULL,err))
 *              //          return MyErrorHandler(err);
 *              //
 *              //       // bail out if the process called LL_UnhookInterrupt
 *              //       if (wait_result == CEI_WIR_INT_DISABLED)
 *              //          break;
 *              //       else if (wait_result != CEI_WIR_INT_OCCURRED)
 *              //          return MyErrorHandler("Unexpected interrupt wait result.");
 *              //
 *              //       // an int has occurred, handle it - in this example, 
 *              //       // int_on_regs is always 1 since there is only one
 *              //       // int status register
 *              //       MyInterruptHandler();
 *              //    }
 *              // }
 *              
 *              // if this is the first process to hook the int for this
 *              // device, we need to configure the device to enable ints
 *              if (!preexisted)
 *                 MyEnableDeviceInterrupts();
 *              ...
 *              ...
 *              ...
 *              // indicate that this process is done monitoring the device 
 *              // int - any pending LL_WaitForInterrupt calls associated with
 *              // threads belonging to this process return with the result
 *              // parameter set to CEI_WIR_INT_DISABLED - if this is the last 
 *              // process to unhook, device ints are disabled and the int
 *              // handler is uninstalled
 *              if (!LL_UnhookInterrupt(my_dev,&persists,err)
 *                 return MyErrorHandler(err);
 *              ...
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED LL_HookInterrupt
(
    CEI_DEV_HANDLE dev_handle,  // (i) handle of device instance
    pCEI_VOID int_ctl_ptr,      // (i) pointer to interrupt control parameter 
                                //     structure (refer to CEI_INT_CONTROL_V1 
                                //     documentation)
    pCEI_UINT32 preexisted_ptr, // (o) upon successful return, receives TRUE if
                                //     another process is already currently 
                                //     using the device's interrupt, FALSE 
                                //     otherwise
    CEI_INSTALL_ERR err         // (o) receives error desc (upon failure)
);


/*===========================================================================*
 * ENTRY POINT: LL_UnhookInterrupt
 *===========================================================================*
 *
 * FUNCTION:    Unhook a device's hardware interrupt.
 *
 * RETURN VAL:  TRUE    operation completed successfully 
 *              FALSE   operation failed - error description stored in 'err'
 *
 * DESCRIPTION: Execute after calling LL_HookInterrupt when the calling 
 *              process no longer needs to monitor interrupts on the given 
 *              device.
 *
 *              If no other running processes currently require hardware 
 *              interrupt monitoring capability on the given device (that is, 
 *              any other processes that have called LL_HookInterrupt for the 
 *              device have either called LL_UnhookInterrupt or terminated), 
 *              device interrupts are disabled and the interrupt handler is 
 *              uninstalled.  Otherwise, device interrupts are not disabled 
 *              and the interrupt handler remains installed.
 *
 *              Refer to LL_HookInterrupt for a discussion of the limitations
 *              associated with multi-process hardware interrupt support.
 *
 *              Prior to returning, LL_UnhookInterrupt terminates any pending
 *              LL_WaitForInterrupt operations associated with thread(s) 
 *              belonging to the calling process (such pending calls return 
 *              with result parameter set to CEI_WIR_INT_DISABLED).
 *
 * EXAMPLE:     Refer to LL_HookInterrupt.
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED LL_UnhookInterrupt
(
    CEI_DEV_HANDLE dev_handle, // (i) handle of device instance
    pCEI_UINT32 persists_ptr,  // (o) upon successful return, receives TRUE if 
                               //     another process is still using the 
                               //     device's interrupt, FALSE otherwise
    CEI_INSTALL_ERR err        // (o) receives error desc (upon failure)
);


/*===========================================================================*
 * ENTRY POINT: LL_WaitForInterrupt
 *===========================================================================*
 *
 * FUNCTION:    Wait for a device to generate an interrupt.
 *
 * RETURN VAL:  TRUE    operation completed successfully 
 *              FALSE   operation failed - error description stored in 'err'
 *
 * DESCRIPTION: Execute after calling LL_HookInterrupt to wait for the device
 *              to generate a pending interrupt. 
 *
 *              The function call returns as soon as (a) the device has 
 *              generated a pending interrupt associated with one or more of 
 *              the specified interrrupt status register(s), or (b) the calling
 *              process calls LL_UnhookInterrupt.
 *
 *              Multiple threads per process may simultaneously call 
 *              LL_WaitForInterrupt, but at most one thread (per process) at a
 *              time may wait for a pending interrupt on a particular interrupt
 *              status register.  That is, no two threads (per process) may 
 *              simultaneously wait for a pending interrupt on the same 
 *              interrupt status register.
 *
 *              Refer to LL_HookInterrupt for a discussion of the limitations 
 *              associated with multi-process hardware interrupt support.
 *
 * EXAMPLE:     Refer to LL_HookInterrupt.
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED LL_WaitForInterrupt
(
    CEI_DEV_HANDLE dev_handle, // (i) handle of device instance
    CEI_UINT64 wait_on_regs,   // (i) bitfield specifying which interrupt 
                               //     status register(s) to wait on - set bit i
                               //     to wait for pending interrupt on 
                               //     status_reg[i] (status_reg array was 
                               //     defined by the calling process within the
                               //     CEI_INT_CONTROL_V1 param structure for 
                               //     LL_HookInterrupt)
    pCEI_UINT32 wait_result,   // (o) upon successful return, receives wait 
                               //     operation result - possible values are 
                               //     CEI_WIR_INT_OCCURRED (a pending interrupt
                               //     occured) or CEI_WIR_INT_DISABLED (the 
                               //     calling process called LL_UnhookInterrupt)
    pCEI_UINT64 int_rcvd_regs, // (o) upon successful return, receives bitfield
                               //     indicating which interrupt status 
                               //     register(s) specified in wait_on_regs 
                               //     generated the pending interrupt (if
                               //     result is CEI_WIR_INT_OCCURRED) or 0 (if
                               //     result is CEI_WIR_INT_DISABLED)
    pCEI_UINT64 chk_reg_val,   // (o) must specify NULL (not currently used)
    CEI_INSTALL_ERR err        // (o) receives error desc (upon failure)
);


/*===========================================================================*
 * ENTRY POINT: LL_InitInterruptControlStruct
 *===========================================================================*
 *
 * FUNCTION:    Initialize an interrupt control structure.
 *
 * RETURN VAL:  TRUE    operation completed successfully 
 *              FALSE   operation failed - error description stored in 'err'
 *
 * DESCRIPTION: Initialize the given interrupt control structure.  This 
 *              routine must be used to initialize the contents of an 
 *              interrupt control structure before the caller writes any 
 *              structure members.
 *
 * EXAMPLE:     Refer to LL_HookInterrupt.
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED LL_InitInterruptControlStruct
(
    CEI_DEV_HANDLE dev_handle, // (i) handle of device instance
    CEI_UINT32 int_ctl_ver,    // (i) caller should pass the constant 
                               //     CEI_INT_CONTROL_VER_V1 (not the value of
                               //     the constant)
    CEI_UINT32 int_ctl_size,   // (i) caller should pass the size (in bytes) of
                               //     the structure pointed to by int_ctl_ptr
    pCEI_VOID int_ctl_ptr,     // (o) contents of this interrupt control 
                               //     structure are initialized (upon 
                               //     successful return)
    CEI_INSTALL_ERR err        // (o) receives error desc (upon failure)
);


/*===========================================================================*
 * ENTRY POINT: LL_MapContiguousDmaBuffer
 *===========================================================================*
 *
 * FUNCTION:    Map a contiguous DMA buffer.
 *
 * RETURN VAL:  TRUE    operation completed successfully 
 *              FALSE   operation failed - error description stored in 'err'
 *
 * DESCRIPTION: Map a physically contiguous DMA buffer into the address space
 *              of the calling process.  
 *
 *              A particular DMA buffer is uniquely identified using a caller-
 *              supplied CEI_RES_ID string (e.g., "bt3_dev0_ch0_dmabuf").  All 
 *              processes accessing the same DMA buffer must call 
 *              LL_MapContiguousDmaBuffer to map the buffer into user-space and 
 *              LL_UnmapContiguousDmaBuffer to unmap the buffer when it is no 
 *              longer required by the calling process.
 *
 *              Only the first process to call LL_MapContiguousDmaBuffer for a 
 *              particular device DMA buffer (the "master" process) actually 
 *              allocates the buffer. Other processes ("minor" processes) may 
 *              subsequently startup and call LL_MapContiguousDmaBuffer to 
 *              access the same buffer, but the same "master" process that 
 *              first allocated the buffer must remain running until all 
 *              "minor" processes have finished accessing the buffer and called
 *              LL_UnmapContiguousDmaBuffer or terminated.  Only at this point 
 *              may the "master" process call LL_UnmapContiguousDmaBuffer 
 *              (releasing the memory allocated for the DMA buffer).
 *
 *              If the caller is the first process to map the DMA buffer 
 *              associated with the given unique CEI_RES_ID string, a new 
 *              physically contiguous DMA buffer (at least min_size bytes in 
 *              length) is allocated and mapped into the address space of the 
 *              calling process.
 *
 *              If another process subsequently attempts to map the same DMA 
 *              buffer, the already-allocated buffer is mapped into the 
 *              address space of the calling process.  If the minimum buffer 
 *              size requested by the caller is greater than the size of the 
 *              already-allocated buffer, the calling process's call to 
 *              LL_MapContiguousDmaBuffer fails.
 *
 *              The physical and user-mode addresses of the buffer are 
 *              returned, as is the actual size of the allocated buffer.
 *
 *              When the calling process no longer requires access to the DMA 
 *              buffer, call LL_UnmapContiguousDmaBuffer.
 *
 * EXAMPLE:     CEI_DEV_HANDLE my_dev;    // obtained via 
 *                                        // LL_OpenDeviceByLogicalId
 *              CEI_CHAR *user_dma_addr;  // receives user addr of DMA buffer
 *              CEI_UINT64 phys_dma_addr; // receives phys addr of DMA buffer
 *              CEI_UINT32 actual_size;   // receives actual size of DMA buffer
 *              CEI_UINT32 dma_supported; // receives DMA support status
 *              CEI_UINT32 preexisted;    // receives true if another process 
 *                                        // already has the DMA buffer mapped
 *              CEI_UINT32 persists;      // receives true if another process 
 *                                        // still has the DMA buffer mapped 
 *              CEI_INSTALL_ERR err;      // receives error desc upon failure
 *              ...
 *              // query contiguous buffer DMA support capability
 *              if (!LL_GetCapability(my_dev,CEI_CAP_CONTIG_DMA,&dma_supported,err))
 *                 return MyErrorHandler(err);
 *              
 *              // bail out if contiguous buffer DMA is not supported
 *              if (!dma_supported)
 *                 return MyErrorHandler("Contiguous buffer DMA is not supported.");
 *              
 *              // map contiguous 8KB DMA buffer into this process's addr space
 *              if (!LL_MapContiguousDmaBuffer(my_dev,8192,FALSE,"my_unique_dma_buf_id",
 *                                             &user_dma_addr,&phys_dma_addr,
 *                                             &actual_size,&preexisted,err))
 *                 return MyErrorHandler(err);
 *              ...
 *              ...
 *              ...
 *              // configure the device DMA controller with 32-bit physical
 *              // address and buffer size
 *              MyConfigureDmaController((CEI_UINT32)phys_dma_addr,actual_size);
 *              
 *              // execute prior to every DMA transfer
 *              if (!LL_DmaTransferBegin(my_dev,user_dma_addr,err))
 *                 return MyErrorHandler(err);
 *              
 *              // execute my device DMA transfer
 *              MyExecuteDmaTransfer();
 *              
 *              // execute upon completion of every DMA transfer 
 *              if (!LL_DmaTransferEnd(my_dev,user_dma_addr,err))
 *                 return MyErrorHandler(err);
 *              ...
 *              ...
 *              ...
 *              // release the DMA buffer
 *              if (!LL_UnmapContiguousDmaBuffer(my_dev,user_dma_addr,&persists,err))
 *                 return MyErrorHandler(err);
 *              ...
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED LL_MapContiguousDmaBuffer
(
    CEI_DEV_HANDLE dev_handle,   // (i) handle of device instance
    CEI_UINT32 min_size,         // (i) minimum DMA buffer size (in bytes)
    CEI_UINT32 allow_phys_64,    // (i) must specify FALSE (not currently used)
    CEI_RES_ID buf_unique_id,    // (i) string that uniquely identifies the DMA
                                 //     buffer across processes (e.g., 
                                 //     "bt3_dev0_ch0_dmabuf")
    pCEI_VOID *user_addr_ptr,    // (o) upon successful return, receives the 
                                 //     user-mode address of the DMA buffer - 
                                 //     this can be used to access the buffer 
                                 //     in the calling process
    pCEI_UINT64 phys_addr_ptr,   // (o) upon successful return, receives the 
                                 //     physical address of the buffer in the 
                                 //     low 32 bits and the high 32 bits are 
                                 //     unused - this 32-bit address can be 
                                 //     used to configure the device's DMA 
                                 //     controller
    pCEI_UINT32 actual_size_ptr, // (o) upon successful return, receives the 
                                 //     actual size (in bytes) of the allocated
                                 //     buffer 
    pCEI_UINT32 preexisted_ptr,  // (o) upon successful return, receives TRUE 
                                 //     if another process currently has the 
                                 //     DMA buffer mapped, FALSE otherwise
    CEI_INSTALL_ERR err          // (o) receives error desc (upon failure)
);


/*===========================================================================*
 * ENTRY POINT: LL_UnmapContiguousDmaBuffer
 *===========================================================================*
 *
 * FUNCTION:    Unmap a contiguous DMA buffer.
 *
 * RETURN VAL:  TRUE    operation completed successfully 
 *              FALSE   operation failed - error description stored in 'err'
 *
 * DESCRIPTION: Execute after calling LL_MapContiguousDmaBuffer when the 
 *              calling process no longer requires access to the referenced 
 *              DMA buffer.
 *
 *              If no other running processes currently require access to the 
 *              referenced DMA buffer (that is, any other processes that have 
 *              called LL_MapContiguousDmaBuffer for the buffer have either 
 *              called LL_UnmapContiguousDmaBuffer to unmap the buffer or 
 *              terminated), the buffer memory is released.
 *
 *              Refer to LL_MapContiguousDmaBuffer for a discussion of the 
 *              limitations associated with multi-process DMA buffer support.
 *
 * EXAMPLE:     Refer to LL_MapContiguousDmaBuffer.
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED LL_UnmapContiguousDmaBuffer
(
    CEI_DEV_HANDLE dev_handle, // (i) handle of device instance
    pCEI_VOID user_addr,       // (i) user-mode DMA buffer address returned by
                               //     a prior call to LL_MapContiguousDmaBuffer
    pCEI_UINT32 persists_ptr,  // (o) upon successful return, receives TRUE if
                               //     another process still has the DMA buffer 
                               //     mapped, FALSE otherwise
    CEI_INSTALL_ERR err        // (o) receives error desc (upon failure)
);


/*===========================================================================*
 * ENTRY POINT: LL_DmaTransferBegin
 *===========================================================================*
 *
 * FUNCTION:    Mark the beginning of a DMA transfer.
 *
 * RETURN VAL:  TRUE    operation completed successfully 
 *              FALSE   operation failed - error description stored in 'err'
 *
 * DESCRIPTION: Call this function immediately before executing any device DMA
 *              transfer which will utilize a DMA buffer returned by a previous
 *              call to LL_MapContiguousDmaBuffer.
 *
 *              This routine flushes the caches of all CPUs to guarantee cache
 *              synchronization with the DMA buffer.
 *
 * EXAMPLE:     Refer to LL_MapContiguousDmaBuffer.
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED LL_DmaTransferBegin
(
    CEI_DEV_HANDLE dev_handle, // (i) handle of device instance
    pCEI_VOID user_addr,       // (i) user-mode DMA buffer address returned by
                               //     a prior call to LL_MapContiguousDmaBuffer
    CEI_INSTALL_ERR err        // (o) receives error desc (upon failure)
);


/*===========================================================================*
 * ENTRY POINT: LL_DmaTransferEnd
 *===========================================================================*
 *
 * FUNCTION:    Mark the ending of a DMA transfer.
 *
 * RETURN VAL:  TRUE    operation completed successfully 
 *              FALSE   operation failed - error description stored in 'err'
 *
 * DESCRIPTION: Call this function immediately after executing any device DMA
 *              transfer which utilized a DMA buffer returned by a previous 
 *              call to LL_MapContiguousDmaBuffer.
 *
 *              This routine flushes all I/O caches and updates the caches of 
 *              all CPUs to guarantee cache synchronization with the DMA 
 *              buffer.
 *
 * EXAMPLE:     Refer to LL_MapContiguousDmaBuffer.
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED LL_DmaTransferEnd
(
    CEI_DEV_HANDLE dev_handle, // (i) handle of device instance
    pCEI_VOID user_addr,       // (i) user-mode DMA buffer address returned by
                               //     a prior call to LL_MapContiguousDmaBuffer
    CEI_INSTALL_ERR err        // (o) receives error desc (upon failure)
);


/*===========================================================================*
 * ENTRY POINT: LL_ReadConfigSpace
 *===========================================================================*
 *
 * FUNCTION:    Read device configuration space.
 *
 * RETURN VAL:  TRUE    operation completed successfully 
 *              FALSE   operation failed - error description stored in 'err'
 *
 * DESCRIPTION: Read the given number of bytes at the specified offset in the
 *              device's configuration space.
 *
 * EXAMPLE:     CEI_DEV_HANDLE my_dev; // obtained via LL_OpenDeviceByLogicalId
 *              CEI_UINT32 my_dev_id;  // receives PCI device id
 *              CEI_INSTALL_ERR err;   // receives error desc upon failure
 *              ...
 *              // read the PCI device id associated with the device
 *              if (!LL_ReadConfigSpace(my_dev,2,2,&my_dev_id,err))
 *                 return MyErrorHandler(err);
 *              ...
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED LL_ReadConfigSpace
(
    CEI_DEV_HANDLE dev_handle, // (i) handle of device instance
    CEI_UINT32 offset,         // (i) start reading at this byte offset in 
                               //     configuration space
    CEI_UINT32 num_bytes,      // (i) byte count to read (1, 2, or 4)
    pCEI_UINT32 value_ptr,     // (o) receives desired value (upon successful 
                               //     return)
    CEI_INSTALL_ERR err        // (o) receives error desc (upon failure)
);


/*===========================================================================*
 * ENTRY POINT: LL_GetCapability
 *===========================================================================*
 *
 * FUNCTION:    Determine if a capability is currently supported.
 *
 * RETURN VAL:  TRUE    operation completed successfully 
 *              FALSE   operation failed - error description stored in 'err'
 *
 * DESCRIPTION: Certain capabilities of the low-level interface are not 
 *              supported on all platforms.  This routine allows the caller to
 *              determine whether a particular capability is supported on the 
 *              current platform.
 *
 * EXAMPLE:     Refer to LL_HookInterrupt.
 *
 *===========================================================================*/
CEI_INT32 DLL_EXPORTED LL_GetCapability
(
    CEI_DEV_HANDLE dev_handle, // (i) handle of device instance
    CEI_UINT32 cap_type,       // (i) determine if this capability is currently
                               //     supported - available cap_type values are 
                               //     CEI_CAP_HW_INTS (hardware interrupts) and
                               //     CEI_CAP_CONTIG_DMA (contiguous buffer DMA)
    pCEI_UINT32 cap_value_ptr, // (o) receives TRUE if cap_type is supported on 
                               //     the current platform, FALSE otherwise 
                               //     (upon successful return)
    CEI_INSTALL_ERR err        // (o) receives error desc (upon failure)
);


// terminate the C-style linkage block
#ifdef __cplusplus
}
#endif


// restore the previously-saved warning state (if necessary)
#ifdef _MSC_VER
#pragma warning(pop)
#endif


#endif
