/* 
Copyright 2010 GE Intelligent Platforms Embedded Systems

modification history
--------------------
 7 Jun 2004   Created
15 Jan 2007   Modified for VxWorks 6.3 RTP Driver
25 Feb 2008   Unify KM and RTP options
10 Mar 2008   Create VxWorks 5.5 version
13 Aug 2009   For some reason we separated PCI Mapping from PCI Installation

DESCRIPTION

This file contains descriptions for the GE Intelligent Platforms PCI/PMC installation


*/

Folder FOLDER_GEIP_X86_AVIONICS {
       NAME        GE Intelligent Platforms Avionics Products
       _CHILDREN   FOLDER_HARDWARE
       CHILDREN    INCLUDE_GEIP_PCI_MAP \
                   INCLUDE_GEIP_PCI_INSTALL
       DEFAULTS    INCLUDE_GEIP_PCI_MAP \
                   INCLUDE_GEIP_PCI_INSTALL
}

Component INCLUDE_GEIP_PCI_MAP {
        NAME          GE Intelligent Platforms Kernel Module Mapping
        SYNOPSIS      Include this option to use Kernel Module mapping for GE Intelligent Platforms PCI/PMC Boards
        HDR_FILES     lowlevel.h \
                      CondorVXWRTPDrv.h \
                      ./drv/pci/pciConfigLib.h
        CONFIGLETTES  CondorVxWRTPDrv.c
        REQUIRES      INCLUDE_PCI_PARAMS
        CFG_PARAMS    VXW_PCI_X86 \
                      VXW_X86_MAP_ADD \
                      _RTP_PHYS_MEM_MAP_ \
                      _RTP_SHARED_MEM_MAP_ \
                      _RTP_VM_PAGE_UNAMP_ \
                      MAX_BTA \
                      SPIN_LOCK_PROTECT \
                      IRQ_OFFSET \
                      vxwdebug
        INIT_RTN      mapCondorPCIAddress(); 
        _INIT_ORDER   usrInit
        INIT_BEFORE   INCLUDE_KERNEL
}

Component INCLUDE_GEIP_PCI_INSTALL {
        NAME          GE Intelligent Platforms Board Installation
        SYNOPSIS      Include this option to install GE IP PCI/PMC Boards
        HDR_FILES     lowlevel.h \
                      CondorVXWRTPDrv.h \
                      ./drv/pci/pciConfigLib.h
        CONFIGLETTES  CondorVxWRTPDrv.c
        REQUIRES      INCLUDE_PCI_PARAMS
        CFG_PARAMS    VXW_PCI_X86 \
                      vxwdebug
        INIT_RTN      gefesInitPCI(); 
        _INIT_ORDER   usrRoot
}

Parameter VXW_PCI_X86 {
        NAME          Define PCI Compile for x86 and Pentium
        SYNOPSIS      This define is set when building the API for pcPentium SBCs
        TYPE          exists
        DEFAULT       TRUE
}

Parameter VXW_X86_MAP_ADD {
        NAME          Adds device memory into the sysPhysMemDesc table
        SYNOPSIS      The memory regions for the board need to be manually added to the sysPhysMemDesc table early during the boot process.  This defines enables adding board memory regions to that table.  All versions pcPentium require this call up to VxWorks 6.6.  If you are using VxWorks 6.6 or higher this is no longer required.  The default if TRUE.  For VxWorks 6.6 or higher set to FALSE.  
        TYPE          exists
        DEFAULT       TRUE
}

Parameter vxwdebug {
        NAME          Enables debug print. Options: 0=print-off - 1=print-on
        SYNOPSIS      Setting this option to 1 will provide debug prints during boot and initialization.  The default is zero (0).  Set this to one (1) if you need to debug the boot or initialization process.
        TYPE          uint
        DEFAULT       0
}

Parameter IRQ_OFFSET {
        NAME          Option IRQ offset. Required by some BSPs. Default = 0
        SYNOPSIS      Some BSPs require an offset added to the IRQ value set in the PCI Configuration register.  The default is zero (0).  If your BSP specifies an offset, set this parameter to that offset value.
        TYPE          uint
        DEFAULT       0
}

Parameter MAX_BTA {
        NAME          Defines maximum number of devices - Default 4
        SYNOPSIS      This parameter specifies the maximum number of devices.  For non-1553 cards, this represents the maximum number of boards in the system.  For 1553 cards, this refers to the number of logical devices (channels) on all 1553 boards in the system.  The default is four (4). This allows one 4-channel board or four 1-channel boards for example.  If your system needs to support more channels, then increase this value.
        TYPE          uint
        DEFAULT       4
}

Parameter _RTP_PHYS_MEM_MAP_ {
        NAME          Defines physical memory mapping
        SYNOPSIS      If you are using a Real Time Process (RTP) you can have either Physical Memory mapping or Shared Memory mapping.  Physical Memory mapping uses the physical address of the board.  Shared Memory Mapping creates a shared memory region and uses that address.  When using shared memory, interrupt processing changes to accommodate RTP restrictions on the use of shared memory in ISR.  If you application requires hardware interrupts you may want to select physical memory mapping.  The default is FALSE.  Change to TRUE to use this option.
        TYPE          exists
        DEFAULT       FALSE
}

Parameter _RTP_SHARED_MEM_MAP_ {
        NAME          Defines shared memory mapping
        SYNOPSIS      If you are using a Real Time Process (RTP) you can have either Physical Memory mapping or Shared Memory mapping.  Physical Memory mapping uses the physical address of the board.  Shared Memory Mapping creates a shared memory region and uses that address.  When using shared memory, interrupt processing changes to accommodate RTP restrictions on the use of shared memory in ISR.  If you application requires hardware interrupts you may want to select physical memory mapping.  The default is FALSE.  Change to TRUE to use this option.
        TYPE          exists
        DEFAULT       FALSE
}

Parameter _RTP_VM_PAGE_UNAMP_ {
        NAME          Ensures memory mapped into a singe context 
        SYNOPSIS      If you use shared memory mapping with your RTP processes, the memory region can only be mapped into a single context.  Some BSPs that auto-configure PCI devices add the board's memory to the sysPhysMemDesc table during boot.  If that is case, then the share memory allocation will fail.  Set this to TRUE to un-map the boards memory region and allow the shared memory allocation.  The default is FALSE.
        TYPE          exists
        DEFAULT       FALSE
}

Parameter SPIN_LOCK_PROTECT {
        NAME          For VxWorks 6.6. Adds Spinlocks for SMP processing
        SYNOPSIS      Use this when you are using SMP to place Spinlocks around critical sections of code.  Spin locks only apply to VxWorks 6.6 or higher running on multi-processor systems.
        TYPE          exists
        DEFAULT       FALSE
}
