/*============================================================================*
 * FILE:                  C o n d o r V X W R T P D r v . c
 *============================================================================*
 *
 * COPYRIGHT (C) 1997 - 2010 BY
 *          GE INTELLIGENT PLATFORMS, INC., SANTA BARBARA, CALIFORNIA
 *          ALL RIGHTS RESERVED.
 *
 *          THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *          COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *          THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *          OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *          AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *          SOFTWARE IS HEREBY TRANSFERRED.
 *
 *          THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *          NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY GE
 *          INTELLIGENT PLATFORMS.
 *
 *===========================================================================*
 *
 * FUNCTION:   CondorVXWRTPDrv.c
 *
 *===========================================================================*/

/* Revision:   1.21
   Date        Modification Notes
  ----------   ---------------------------------------------------------------
  02/29/2008   Unified PowerPC and x86 installation for Kernel Module and RTP
  10/22/2008   Added VxW_GetCommercialInfo in support of all Commercial APIs
  03/12/2009   Added support for the new IRQ_OFFSET and SPIN_LOCK_PROTECT 
               defines with values assigned in the CDF file.
  08/13/2009   Merged Commerical and 1553 product VxWorks driver - lets be common
  07/19/2010   Trying to keep up with unannounced 1553 VxWorks driver changes
                  Changed gefDrvVer routine name to be geipDrvVer
                  Changed gefHelp routine name to be geipHelp
                  Changed gefBoardShow routine name to be geipBoardShow
  
*/

#include "CondorVXWRTPDrv.h"
#include <sysLib.h>

UINT VxW_GetCommercialInfo(UINT device, 
                           UINT *bar0addr,
                           UINT *bar2addr,
                           UINT *manufId,
                           UINT *deviceId,
                           UINT *irq
                           )
{
   *bar0addr = pci_state[device].membase[0];
   *bar2addr = pci_state[device].membase[2];    
   *manufId  = pci_state[device].manufacturerID;    
   *deviceId = pci_state[device].CondorDeviceID;
   *irq      = pci_state[device].irq;
   return 1;
}

void geipDrvVer()
{
   printf("GE Intellligent Platforms Common VxWorks Drvier version %3.2f\n",CONDOR_VXW_DRV_VER);
   printf("GE Intellligent Platforms Common VxWorks Driver date (%s)\n\n",CONDOR_VXW_DRV_DATE);
}

void geipHelp()
{
   printf("       GE Intelligent Platforms Avionics Debug functions\n\n");
   printf("VxW_GetCommercialInfo(device...) -- Returns commercial avionics board info\n");
   printf("VxW_Get1553CardID(device) -- Returns the 1553 board type\n");
   printf("VxW_GetMQID(device)       -- Returns interrupt message queue ID\n");
   printf("VxW_Get1553HIF(device)    -- Returns the 1553 Host Interface value\n");
   printf("VxW_GetDevChan(device)    -- Returns number channels on the device\n");
   printf("VxW_GetDevIRQ(device)     -- Returns the devices IRQ (int line)\n");
   printf("geipBoardShow             -- Lists all GEIP Avionic boards\n");
   printf("memDescshow               -- Lists contents of sysPhysMemDesc table\n");
   printf("mapCondorPCIAddress       -- Shows mapping parameter (debug mode only)\n");
   printf("gefes_config_1553_devices(dev) -- Shows device configuration(debug mode only)\n");
   printf("gefesInitPCI              -- Show Device and interrupt configuration \n");
   printf("geipDrvVer                -- Display driver version and release date\n\n");
}

float VxW_GetDrvVer()
{
   return CONDOR_VXW_DRV_VER;
}

char * VxW_GetDrvDate()
{
   return CONDOR_VXW_DRV_DATE;
}

UINT VxW_Get1553CardID(UINT device )
{
   return pci_state[device].dev_id;
}

MSG_Q_ID VxW_GetMQID(UINT device)
{
   return pci_state[device].msg_q_id;
}

UINT VxW_Get1553HIF(int dev)
{
   return pci_state[dev].hif;
}

UINT VxW_GetDevChan(UINT device)
{
   return pci_state[device].dev_chan;
}

UINT VxW_GetDevIRQ(UINT device)
{
   return pci_state[device].irq;
}

UINT VxW_GetDevID(UINT device)
{
   return dev_id[device];
}

UINT get_device(UINT cardnum)
{
   return cardNumberIndex[cardnum];
}

void get_dev_id(UINT *devid)
{
   int i;
   for (i=0; i<MAX_BTA; i++)
     devid[i] = dev_id[i];
}

#ifdef _RTP_SHARED_MEM_MAP_
int rtpInterruptConnect(int device)
{
   char thread_name[40], mq_name[40];

#ifdef SPIN_LOCK_PROTECT
   spinLockIsrInit(&spinlock[device],0); 
#endif    
    
   irq_value[device] = VxW_GetDevIRQ(device);
   pci_state[device].irq = irq_value[device];
   
   isrSemaphore[device] = semBCreate (SEM_Q_FIFO, SEM_EMPTY);
   if (isrSemaphore[device] == NULL)
      return ERROR;
   
   sprintf(mq_name,"/msg_q_%d",device);
   pci_state[device].msg_q_id = msgQOpen(mq_name,
                                         100,
                                         4,
                                         MSG_Q_FIFO,
                                         OM_CREATE,
                                         NULL);
      
   if (pci_state[device].msg_q_id == NULL)
   {
      dprint("Message Queue Create Error on %s",mq_name);
      dprint("msgQOpen fails with %x",errno);
      return 0x444;
   }

   sprintf(thread_name,"isrThread_%d",device);

   isrTask_id[device] = taskSpawn (thread_name, 
                                   1, 
                                   0, 
                                   20000, 
                                   (FUNCPTR)intProcess,
                                   (int)pci_state[device].membase[pci_state[device].mem_base_reg],
                                   (int)pci_state[device].dev_chan,
                                   (int)pci_state[device].dev_id,
                                   (int)pci_state[device].msg_q_id,
                                   (int)isrSemaphore[device],
                                   device,
                                   7,
                                   8,
                                   9,
                                   0);  
   if (isrTask_id[device] == 0)
      return ERROR;
      
   int_level[device] = irq_value[device];

#ifdef VXW_PCI_X86
   sysIntDisablePIC(irq_value[device]);
   if(pciIntConnect(INUM_TO_IVEC(irq_value[device] + INT_NUM_IRQ0 + IRQ_OFFSET),isrProcess,device) == ERROR)
      return ERROR;
   sysIntEnablePIC(irq_value[device]);
#else
#if defined (VXW_PCI_INT_CON)
   dprint("pciIntConnect with int line %x\n", int_level[device]);
   if(pciIntConnect(INUM_TO_IVEC(int_level[device] + IRQ_OFFSET),intProcess,device))
      return ERROR;
        
   if(intEnable(int_level[device]) == ERROR)
      return ERROR;
#elif defined (VXW_INT_LINE_CON)
   dprint("intConnect using int line %x\n", int_level[device]);
   if(intConnect((void *)(int_level[device] + IRQ_OFFSET),intProcess,device))
      return ERROR;

   if(intEnable(int_level[device]) == ERROR)
      return ERROR;
 #else
   dprint("Using intConnnect on isrProcess for IRQ %x\n",int_level[device]);
   if(intConnect(INUM_TO_IVEC(INT_VEC_IRQ0 + int_level[device] + IRQ_OFFSET),isrProcess,device))
      return ERROR;
        
   if(intEnable(int_level[device]) == ERROR)
      return ERROR;
 #endif
#endif

   return 0;
}

#else /* _RTP_SHARED_MEM_MAP_ */

int rtpInterruptConnect(int device)
{
   char mq_name[40];
   
   irq_value[device] = VxW_GetDevIRQ(device);
   pci_state[device].irq = irq_value[device];
#ifdef SPIN_LOCK_PROTECT
   spinLockIsrInit(&spinlock[device],0); 
#endif 

#if _WRS_VXWORKS_MAJOR == 6  
   sprintf(mq_name,"/msg_q_%d",device);
   pci_state[device].msg_q_id = msgQOpen(mq_name,100,4,MSG_Q_FIFO,OM_CREATE,NULL);
#else
   pci_state[device].msg_q_id = msgQCreate(100,4,MSG_Q_FIFO);
#endif  /* VXW_RTP_MODE */
   if(pci_state[device].msg_q_id == NULL)
   {
      dprint("Message Queue Create Error on %s",mq_name);
      dprint("msgQOpen fails with %x",errno);
      return 0x444;
   }		
      
   int_level[device] = irq_value[device];

#ifdef VXW_PCI_X86
   sysIntDisablePIC(irq_value[device]);
/*   if(pciIntConnect(INUM_TO_IVEC(irq_value[device] + INT_NUM_IRQ0 + IRQ_OFFSET),intProcess,device) == ERROR) */
   if(pciIntConnect (INUM_TO_IVEC (INT_NUM_GET (irq_value[device])),intProcess,device) == ERROR) {
	   dprint("pciIntConnect ERROR %x\n",errno);
	   dprint("IVEC = %x\n",INUM_TO_IVEC(irq_value[device] + INT_NUM_IRQ0 + IRQ_OFFSET));
      return ERROR;
   }
   sysIntEnablePIC(irq_value[device]);
   dprint("pciIntConnect for IVEC %x\n",INUM_TO_IVEC(irq_value[device] + INT_NUM_IRQ0 + IRQ_OFFSET));
   dprint("INT_NUM_GET(irq) = %x\n",INT_NUM_GET(irq_value[device]));
#else
 #ifdef VXW_PCI_INT_CON
   dprint("pciIntConnect with int line %x\n", int_level[device]);
   if(pciIntConnect(INUM_TO_IVEC(int_level[device] + IRQ_OFFSET),intProcess,device))
      return ERROR;

/*   sysIntEnablePIC(irq_value[device]);*/
   if(intEnable(int_level[device]) == ERROR)
        return ERROR;
#elif defined (VXW_INT_LINE_CON)
   dprint("int line %x\n", int_level[device]);
   int_level[device] = int_level[device] + INT_VEC_IRQ0;
   dprint("intConnect using int line %x\n", int_level[device]);
   if(intConnect((void *)(int_level[device] + IRQ_OFFSET),intProcess,device))
      return ERROR;
        
   if(intEnable(int_level[device]) == ERROR)
      return ERROR;
 #else /* VXW_PCI_INT_CON */
   intDisable(int_level[device]);
   dprint("Using intConnnect on intProcess for IRQ %x\n",int_level[device]);
   if(intConnect(INUM_TO_IVEC(INT_VEC_IRQ0 + int_level[device] + IRQ_OFFSET),intProcess,device))
      return ERROR;
        
   if(intEnable(int_level[device]) == ERROR)
      return ERROR;
 #endif /* VXW_PCI_INT_CON */
#endif
   return 0;
}
#endif /* _RTP_SHARED_MEM_MAP_ */

int cnicInterruptConnect(int* params)
{
   /* params is pointer to three 32-bit ints:  [0] = pPdx, [1] = pPdx->sys_board_num, [2] = pointer to ISR */
   int pPdx = params[0];
   int device = params[1];
   int isr = params[2];
   int irqLevel = VxW_GetDevIRQ(device);

   irq_value[device] = irqLevel;
/*   pci_state[device].irq = irqLevel;
*/
   dprint("Entering cnicInterruptConnect :\n",0);
   dprint("  device = %d\n", device);
   dprint("  isr = 0x%X\n", isr);
   dprint("  irqLevel = %d\n", irqLevel);

#if defined(VXW_PCI_X86)

   sysIntDisablePIC(irqLevel);
   dprint("cnicInterruptConnect: calling pciIntConnect\n", 0);

   if (pciIntConnect(INUM_TO_IVEC(irq_value[device] + INT_NUM_IRQ0 + IRQ_OFFSET), (VOIDFUNCPTR)isr, pPdx) == ERROR)
   {
      dprint("cnicInterruptConnect: error calling pciIntConnect - errno = %d\n", errno);
      return ERROR;
   }
   dprint("cnicInterruptConnect: success calling pciIntConnect\n",0);
   sysIntEnablePIC(irqLevel);

#else
   #if defined (VXW_PCI_INT_CON)

      dprint("cnicInterruptConnect: calling pciIntConnect\n", 0);
      if (pciIntConnect(INUM_TO_IVEC(irqLevel + IRQ_OFFSET), (VOIDFUNCPTR)isr, pPdx))
      {
         dprint("cnicInterruptConnect: error calling pciIntConnect\n", 0);
         return ERROR;
      }
           
      if (intEnable(irqLevel) == ERROR)
      {
         dprint("cnicInterruptConnect: error calling intEnable\n", 0);
         return ERROR;
      }

   #else /* VXW_PCI_INT_CON */

      dprint("cnicInterruptConnect: calling intConnect\n", 0);
      if (intConnect(INUM_TO_IVEC(INT_VEC_IRQ0 + irqLevel + IRQ_OFFSET), (VOIDFUNCPTR)isr, pPdx))
      {
         dprint("cnicInterruptConnect: error calling intConnect\n", 0);
         return ERROR;
      }
      dprint("cnicInterruptConnect: success calling intConnect\n", 0);
           
      if (intEnable(irqLevel) == ERROR)
      {
         dprint("cnicInterruptConnect: error calling intEnable\n", 0);
         return ERROR;
      }
      dprint("cnicInterruptConnect: success calling intEnable\n", 0);

   #endif /* VXW_PCI_INT_CON */
#endif

   return 0;
}


/****************************************************************
*
*  PROCEDURE - mapCondorPCIAddress()
*
*  PARAMETERS - None
*
*  RETURNS
*     BTD_OK          - Mapped address with no error
*     ERROR           - Error Mapping PCI addresses
****************************************************************/
int mapCondorPCIAddress(void)
{
   int i,j, status,unit, memRegion, instance;
   int busNumber;
   int deviceNumber;
   int functionNumber;

   UINT32 mem, mask, page_addr, page_offset;
   pciregion_state * pPCIState;
   unsigned short *addr;
#ifdef _RTP_SHARED_MEM_MAP_
   unsigned *mapped_addr[6];
   char mem_reg_name[80];
#endif /* _RTP_SHARED_MEM_MAP_ */

   status = SYSTEM_OK;

   dprint("mapCondorPCIAddress in debug mode %d\n",0);
   unit = 0;
   for (i = 0; i < num_boards; i++)
   {
      for (instance = 0; instance < MAX_BTA; instance++)
      {
         if(pciFindDevice(CONDOR_ENGINEERING,board_to_map[i],instance,&busNumber,&deviceNumber,&functionNumber) == ERROR)
            break;

         /* read address */
         pPCIState = &pci_state[0];

         for (j = 0; j < MAX_BTA; j++, pPCIState++)
         {
            if ((j != unit) && (pPCIState->busNumber == busNumber) &&
                (pPCIState->deviceNumber == deviceNumber) && (pPCIState->functionNumber == functionNumber))
               continue;
         }
      
         pci_state[unit].busNumber = busNumber;
         pci_state[unit].CondorDeviceID = board_to_map[i];
         pci_state[unit].functionNumber = functionNumber;
         pci_state[unit].deviceNumber = deviceNumber;
         pci_state[unit].bustype = 2;
         pci_state[unit].mem_base_reg = 0;
         pci_state[unit].region_count = 0;
         pci_state[unit].mapping = PHYSICAL_MAP; /* default to physical mapping */
      
         dprint("PCI device found on bus %d\n",busNumber);
         dprint("PCI device number = %x\n",board_to_map[i]);

         for (memRegion = 0; memRegion < MAX_MEMORY_REGIONS; memRegion++)
         {
            /* determine region size */
            pciConfigInLong(busNumber, deviceNumber,functionNumber,  addresses[memRegion], &mem);
            pciConfigOutLong(busNumber, deviceNumber,functionNumber,addresses[memRegion], (unsigned)~0);
            pciConfigInLong(busNumber, deviceNumber,functionNumber,addresses[memRegion], &mask);
            pciConfigOutLong(busNumber, deviceNumber,functionNumber,addresses[memRegion], mem);
         
            if(mem & PCI_BASE_ADDRESS_SPACE)
               continue;
            
            pci_state[unit].membase[memRegion] = mem & PCI_MEMBASE_MASK;    
            pci_state[unit].size[memRegion] = ~mask+1;

            if(!mask)
               continue;

            pci_state[unit].region_count++;
            
            page_addr = pci_state[unit].membase[memRegion] & 0xfffff000;
            dprint("page_addr = %x\n", page_addr);
            page_offset = pci_state[unit].membase[memRegion] - page_addr;
            dprint("page_offset = %x\n", page_offset);

            if(pci_state[unit].size[memRegion] < PAGE_SIZE)
            {
               dprint("PCI unadjusted region size = %lx\n",pci_state[unit].size[memRegion]);
               pci_state[unit].size[memRegion] = PAGE_SIZE;
            }
            
            if(pci_state[unit].size[memRegion] == 0x800000 ||pci_state[unit].size[memRegion] == 0x400000)
            {
               pci_state[unit].mem_base_reg = memRegion;
               dprint("mem_base_region = %lx\n",pci_state[unit].mem_base_reg);
            }   

#if defined (VXW_X86_MAP_ADD)
   #if defined(_RTP_PHYS_MEM_MAP_) 
            if((mem & PCI_BASE_ADDRESS_SPACE) == 0) /* map only memory regions not I/O space */
            { 
               dprint("Calling sysMmuMapAdd for x86 unit = %d\n",unit);
               if(sysMmuMapAdd((void *)mem,pci_state[unit].size[memRegion],MMU_VALID_PROT_MASK,MMU_RW_FOR_ALL) == ERROR) 
                  return ERROR;
            }
   #else  /* _RTP_PHYS_MEM_MAP_ */          
            if((mem & PCI_BASE_ADDRESS_SPACE) == 0) /* map only memory regions not I/O space */
            { 
               dprint("Calling sysMmuMapAdd for x86 unit = %d\n",unit);
               if(sysMmuMapAdd((void *)mem,pci_state[unit].size[memRegion],VM_STATE_MASK_FOR_ALL,VM_STATE_FOR_PCI) == ERROR) 
                  return ERROR;
            }
   #endif /* _RTP_PHYS_MEM_MAP_ */        
#endif  /* VXW_X86_MAP_ADD */
         
#if defined(_RTP_PHYS_MEM_MAP_) && !defined (VXW_X86_MAP_ADD)
            dprint("vmStateSet addr = %x\n",page_addr);
            dprint("vmStateSet size = %x\n",pci_state[unit].size[memRegion]);
            dprint("vmStateSet StateMask = %x\n",MMU_VALID_PROT_MASK);
            dprint("vmStateSet State = %x\n", MMU_RW_FOR_ALL);
            status = vmStateSet (NULL, page_addr, pci_state[unit].size[memRegion], MMU_VALID_PROT_MASK, MMU_RW_FOR_ALL);      
            if(status == ERROR)
            {
               dprint("vmStateSet Error = %x\n",errno);
            }
#elif defined(_RTP_SHARED_MEM_MAP_)
   #ifdef _RTP_VM_PAGE_UNAMP_
            dprint("VM_PAGE_UNMAP page_addr = %x\n", page_addr);
            dprint("VM_PAGE_UNMAP membase = %x\n", pci_state[unit].membase[memRegion]);
            dprint("VM_PAGE_UNMAP PCI region size = %lx\n",pci_state[unit].size[memRegion]);

            VM_PAGE_UNMAP(NULL, page_addr, pci_state[unit].size[memRegion]);
   #endif        
            sprintf(mem_reg_name,"dev%d_%d",unit,memRegion);
            pci_state[unit].sd_id[memRegion] = sdCreate(mem_reg_name,SD_LINGER,pci_state[unit].size[memRegion],(off_t64)pci_state[unit].membase[memRegion],SD_ATTR_RW | SD_CACHE_OFF,(void *)&mapped_addr[memRegion]);
            
            if(pci_state[unit].sd_id[memRegion]==NULL)
            {
               dprint("sdCreate Failed with %x\n",errno);
               return 9999;
            }
         
            pci_state[unit].membase[memRegion] = (unsigned long)(mapped_addr[memRegion] + page_offset);
            pci_state[unit].mapping = VIRTUAL_MAP;
#endif /* _RTP_PHYS_MEM_MAP_ */

            dprint("Found PCI memory region %d\n", memRegion);
            dprint("PCI base address = %lx\n",pci_state[unit].membase[memRegion]);
            dprint("PCI region size = %lx\n",pci_state[unit].size[memRegion]);      
            
         }     
      
         pciConfigInWord (busNumber, deviceNumber, functionNumber, PCI_CFG_VENDOR_ID, (UINT16 *)(&pci_state[unit].manufacturerID));  
         pciConfigInWord (busNumber, deviceNumber, functionNumber, PCI_CFG_DEVICE_ID, (UINT16 *)(&pci_state[unit].CondorDeviceID));  
         pciConfigInByte (busNumber, deviceNumber, functionNumber, PCI_CFG_DEV_INT_LINE, &pci_state[unit].irq);

         addr = (unsigned short *)pci_state[unit].membase[pci_state[unit].mem_base_reg];
         dprint("Mem base addr = %x\n",addr);

         dprint("Complete with unit %d\n",unit);
         unit++;
      }  
   }
   bd_mapped = 1;
   dprint("mapCondorPCIAddress Complete %d\n\n",unit);
   number_of_gefes_device = unit;
   return unit;
}

int gefes_config_1553_devices(int bd_indx)
{
   UINT btype,nchan;
   int ci;
   unsigned short host_interface;
   UINT chndx;
   UINT card_table[] = {0,4,8,12};
   
   dprint("gefes_config_1553_devices %d\n",bd_indx);
   chndx = card_table[bd_indx];
   
   dprint("base addr = %x\n",pci_state[bd_indx].membase[pci_state[bd_indx].mem_base_reg]);
   host_interface = *(unsigned short *)(pci_state[bd_indx].membase[pci_state[bd_indx].mem_base_reg]);
   flipw(&host_interface);
   dprint("Host_interface = %x\n",host_interface);                   

   dprint("Condor Device ID = %x\n",pci_state[bd_indx].CondorDeviceID);
   pci_state[bd_indx].hif=host_interface;
   if((host_interface & 0x000f) != 0)
   {
      dprint("1553 Qboard host interface = %x\n",host_interface);
      btype = (host_interface & 0x3e)>>1;
      nchan = (host_interface & 0x7c0)>>6;
      dev_chan[bd_indx]=nchan;
      pci_state[bd_indx].dev_chan = nchan;
      pci_state[bd_indx].dev_id = QdevID[btype-1];
      dev_id[bd_indx] = QdevID[btype-1];
      for(ci=0;ci<nchan;ci++)
      {
         cardNumberIndex[chndx] = bd_indx;
         chndx++;
      }
      dprint("btype = %x\n",btype);
      dprint("nchan = %d\n",nchan);
      dprint("dev_id = %x\n", dev_id[bd_indx]);
   } 
   else
   {
      dprint("1553 Board host interface = %x\n",host_interface);
      btype = (host_interface & 0xc000)>>14;
      nchan = (host_interface & 0x3000)>>12;
      if(nchan > 1)
         nchan = 2;
      pci_state[bd_indx].dev_chan = nchan;
      dev_chan[bd_indx]=nchan;
      pci_state[bd_indx].dev_id = devID[btype];
      for(ci=0;ci<nchan;ci++)
      {
         cardNumberIndex[chndx] = bd_indx;
         chndx++;
      }
      dprint("btype = %x\n",btype);
      dprint("nchan = %d\n",nchan);
      dprint("dev_id = %x\n", dev_id[bd_indx]);
   }

   dprint("gefes_config_pci_devices Complete for device %d\n",bd_indx);
   fflush(stdout);
   return 0;   
}

int gefes_config_AR15_device(int bd_indx)
{
   UINT nchan;
   int ci;  
   UINT chndx=0;
   
   dprint("gefes_config_Ar15_device %d\n",bd_indx);
   dprint("base addr = %x\n",pci_state[bd_indx].membase[pci_state[bd_indx].mem_base_reg]);
   dprint("Condor Device ID = %x\n",pci_state[bd_indx].CondorDeviceID);
     
   nchan = 10;
   dev_chan[bd_indx]=nchan;
   pci_state[bd_indx].dev_chan = nchan;
   pci_state[bd_indx].dev_id = 0x280;
   dev_id[bd_indx] = 0x280;
   for(ci=0;ci<nchan;ci++)
   {
      cardNumberIndex[chndx] = bd_indx;
      chndx++;
   }
   dprint("nchan = %d\n",nchan);
   dprint("dev_id = %x\n", dev_id[bd_indx]);

   dprint("gefes_config_pci_devices Complete for device %d\n",bd_indx);
   fflush(stdout);
   return 0;   
}


/*******************************************************************************
*
* gefesPCIDrvInit - initialize the device driver
*
* This routine initializes and installs the driver.  It must be called
* before any GEFES_PCI devices are created.  
*
* RETURNS: OK, or ERROR if the driver installation fails.
*/

STATUS gefesPCIDrvInit (void)
{
    /* check if driver already installed */
   
   if(gefesPCIDrvNum == -1)
      gefesPCIDrvNum = 0;
   
   if (gefesPCIDrvNum == 0)
   {
      gefesPCIDrvNum = iosDrvInstall ( gefesPCIDrvCreate, 
                                       gefesPCIDrvDelete, 
                                       gefesPCIDrvOpen,
                                       gefesPCIDrvClose, 
                                       gefesPCIDrvRead, 
                                       gefesPCIDrvWrite, 
                                       gefesPCIDrvIoctl);
   }
   
   if(gefesPCIDrvNum == ERROR)
   {
      return -1; /* ERROR;*/
   }
   else
      return 0;
}
/*******************************************************************************
*
* gefesPCIDrvCreate - create a gefesPCI device
*
* This routine creates a gefesPCI device.
* It allocates memory for the necessary structures and initializes the device.
*   
* RETURNS: OK, or ERROR if the call fails.
*/

static STATUS gefesPCIDrvCreate (DEV_HDR *hdr, char *name, void * mode)      /* name of device to be created      */
{

   CEI_INT devnum;          
   char device_name[80];
   
   if (gefesPCIDrvNum < 1)
   {
      errno = S_ioLib_NO_DRIVER;
      return (ERROR);
   }
 
   devnum = *(int *)name;      
   dprint("gefesPCIDrvCreate for device %d\n",devnum);
   
   sprintf(device_name,"/gefesDev/%d",devnum);
   dprint("device_name = %s\n",device_name);
   
   pgefesPCIDev[devnum] = (GEFES_PCI_DEV *) malloc (sizeof (GEFES_PCI_DEV));

   if (pgefesPCIDev[devnum] == NULL)
      return (ERROR);

   /* add instance of I/O device to system */
   if (iosDevAdd (&pgefesPCIDev[devnum]->devHdr, device_name, gefesPCIDrvNum) != OK)
   {
      free ((char *) pgefesPCIDev);
     return (ERROR);
   }
   pgefesPCIDev[devnum]->thisDevice = devnum;
   pgefesPCIDev[devnum]->pci_state = &pci_state[devnum];
   pgefesPCIDev[devnum]->read_target = 0;

   return(OK);  
}

/*******************************************************************************
*
* gefesPCIDrvDelete- Destroy a gefesPCI device
*
*/

static STATUS gefesPCIDrvDelete(char * name) /* name of gefesPCI driver to be deleted */
{
   GEFES_PCI_DEV *pgefesPCIDev;
   const char * pTail = NULL;

    /* driver must be initialized */

   if (gefesPCIDrvNum == ERROR)
   {
      errno = S_ioLib_NO_DRIVER;
      return (ERROR);
   }

   /* get pointer to gefesPCI device descriptor */

   if ((pgefesPCIDev = (GEFES_PCI_DEV *) iosDevFind (name, &pTail)) == NULL)
   {
      return (ERROR);
   }

   /* remove the instance of the I/O device from the system */
   iosDevDelete (&pgefesPCIDev->devHdr);

   /* free memory */
   free ((char *) pgefesPCIDev);

   return (OK);
}

/*******************************************************************************
*
* gefesPCIDrvOpen - open a gefesPCI device file
*
* This routine is called to open a gefesPCI driver.
*    
* Whatever you return, the system will associate with the file
* descriptor and hand to your read and write function when they are called.
* For a simple device with no files simply return the device descriptor.  
* More complex devices will have an array of file specific structures 
* that includes  a pointer back to device descriptor.
*
* If no resources are avalible you can also return ERROR, which will cause 
* the open() call in the code to return error also.    
*
* NOMANUAL
*/

LOCAL int gefesPCIDrvOpen
(
   GEFES_PCI_DEV *pgefesPCIDev,         /* gefesPCI descriptor */
   char *name,                          /* name of the file with device stipped off */ 
   int flags
)
{ 
   return ((int)pgefesPCIDev);
} 

/*******************************************************************************
*
* gefesPCIWrite - write bytes to a gefesPCI
*
* This routine writes a message of `nbytes' to the gefesPCI device.
*
* RETURNS: number of bytes written or ERROR
*
* NOMANUAL
*/

LOCAL int gefesPCIDrvWrite
(
   FAST GEFES_PCI_DEV *pgefesPCIDev,    /* pointer to gefesPCI descriptor */
   char *buffer,               /* buffer from which to copy bytes */
   int nbytes                  /* number of bytes to copy from buffer */
)
{
   /* TODO: do somthing with the data */
   return (nbytes);
}

/*******************************************************************************
*
* gefesPCIIoctl - do device specific control function
*
* The ioctl requests recognized are entirely up to the designer, though
* select support requires implimentation of FIOSELECT and FIOUNSELECT
*
* The ioctl command also provides an excellent way to pass out of band
* status mode, and configuration information to or from the driver using
* non-standard ioctl codes.
*
* RETURNS:
*  OK 
*  ERROR if request is not supported
*
* NOMANUAL
*/

LOCAL STATUS gefesPCIDrvIoctl
(
   FAST GEFES_PCI_DEV *gefesPCI_DEV,    /* pointer to gefesPCI descriptor */
   int request,                         /* ioctl code */
   int *argptr                          /* where to send answer */
)
{
   STATUS   status = OK;
   int offset;

   switch (request)
   {
      case PCI_GET_REGION_SIZE:
         *argptr = (int)gefesPCI_DEV->pci_state->size[gefesPCI_DEV->pci_state->region];
         break;
      case PCI_GET_REGION_MEM:
         *argptr = (int)gefesPCI_DEV->pci_state->membase[gefesPCI_DEV->pci_state->region];
         break;
      case PCI_GET_INTERRUPT:
         *argptr = (int)gefesPCI_DEV->pci_state->irq;
         break;
      case PCI_SET_REGION:
         gefesPCI_DEV->pci_state->region = *argptr; 
         break;
      case PCI_SET_CNIC_ISR:
         /* params is pointer to three 32-bit ints:  [0] = pPdx, [1] = pPdx->sys_board_num, [2] = pointer to ISR */
         cnicInterruptConnect(argptr); 
         break;
      case PCI_GET_NUM_REGS:
         *argptr = gefesPCI_DEV->pci_state->num_regions;
      case PCI_GET_HOST_INTRF:
         break;
      case PCI_GET_MSG_Q_ID:
         *argptr = (unsigned)gefesPCI_DEV->pci_state->msg_q_id;
         break;
      case PCI_GET_PCIREGION_BYTE:
         offset = *argptr;
         pciConfigInByte(gefesPCI_DEV->pci_state->busNumber,
                         gefesPCI_DEV->pci_state->deviceNumber,
                         gefesPCI_DEV->pci_state->functionNumber,
                         offset,
                        (UINT8 *)argptr);
         break;
      case PCI_GET_PCIREGION_WORD:
         offset = *argptr;
         pciConfigInWord(gefesPCI_DEV->pci_state->busNumber,
                         gefesPCI_DEV->pci_state->deviceNumber,
                         gefesPCI_DEV->pci_state->functionNumber,
                         offset,
                        (UINT16 *)argptr);
         break;
      case PCI_GET_PCIREGION_DWORD:
         offset = *argptr;
         pciConfigInLong(gefesPCI_DEV->pci_state->busNumber,
                         gefesPCI_DEV->pci_state->deviceNumber,
                         gefesPCI_DEV->pci_state->functionNumber,
                         offset,
                        (UINT32 *)argptr);
         break;
      case PCI_GET_BOARD_TYPE:
         *argptr = (int)gefesPCI_DEV->pci_state->dev_id;
         break;
      case PCI_GET_DEVICE_ID:
         *argptr = (int)gefesPCI_DEV->pci_state->CondorDeviceID;
         break;    
      case PCI_GET_NUM_CHAN:
         *argptr = (int)gefesPCI_DEV->pci_state->dev_chan;
         break;
      case PCI_SYS_CLK_SET:
         {
            int crate;                    
            crate = (int)*argptr;
            if(sysClkRateSet(crate)== ERROR) /* This gives a accurate millisecond timer */
               *argptr = BTD_CLK_RATE_NOT_SET;  
            else
               *argptr = BTD_OK;
            break;  
         }
      case PCI_GET_DEV_MAP:
         {
            PDEVMAP_T devPTR;
            CEI_INT mindx,device,i;
            CEI_UINT membase,region_count,size;
          
            devPTR = (PDEVMAP_T)argptr;
            device = devPTR->device;

            /*Clear the region_count and struct*/
            region_count=0;
            for ( i = 0; i < MAX_MEMORY; i++ )
            {
               devPTR->memStartPhy[i]    = 0;    /*Physical address is null.*/
               devPTR->memLengthBytes[i] = 0;    /*Length is null.*/
               devPTR->memHostBase[i]    = NULL; /*Host address is null.*/
               devPTR->flagMapToHost[i]  = 0;
               
            }
          
            devPTR->busType = pci_state[device].bustype;
            devPTR->interruptNumber = pci_state[device].irq;
            devPTR->memSections = pci_state[device].region_count;
            devPTR->VendorID = 0x13c6;
            devPTR->DeviceID = pci_state[device].CondorDeviceID;

            for(mindx=0;mindx<MAX_MEMORY; mindx++)
            {
#if defined(VXW_SYS_PCI_MAP)
               membase = (unsigned long)sysPciMemToLocalAdrs(pci_state[device].busNumber,pci_state[device].membase[i]);
#elif defined(VXW_SYS_BUS_MAP)
               {
                  pCEI_UCHAR addr_base[1];
                  status = (unsigned long)sysBusToLocalAdrs(PCI_SPACE_MEM_PRI,(char *)pci_state[device].membase[i],addr_base);
                  membase = (unsigned long)addr_base[0];
               }
#else
               membase = pci_state[device].membase[mindx];
#endif 
               size = pci_state[device].size[mindx];

               if(size>0)
               {
                  devPTR->memStartPhy[region_count] = membase;
                  devPTR->memHostBase[region_count] = (pCEI_CHAR)membase;
                  devPTR->memLengthBytes[region_count] = size;
                  devPTR->flagMapToHost[region_count] = 1;
                  region_count++;
               }  
            }
            break;
         }
     default:
       return ERROR;
   }
    return (status); 
}


/*******************************************************************************
*
* gefesPCIRead - does the reading for each file descritor
*
*
* RETURNS: number of bytes read or ERROR
*
* NOMANUAL
*/

static int gefesPCIDrvRead 
(
   GEFES_PCI_DEV *pgefesPCIDev, 
   char *  pBuf,    /* addr of destination buffer */
   int     maxBytes /* maximum bytes to read into buffer */
)
{
   unsigned request, offset;
   
   request = *(unsigned *)pBuf;
   
   switch (request)
   {
      case PCI_GET_REGION_SIZE:
         *(int *)pBuf = (int)pgefesPCIDev->pci_state->size[pgefesPCIDev->pci_state->region];
         break;
      case PCI_GET_REGION_MEM:
         *(unsigned *)pBuf = (int)pgefesPCIDev->pci_state->membase[pgefesPCIDev->pci_state->region];
         break;
      case PCI_GET_INTERRUPT:
         *(int *)pBuf = (int)pgefesPCIDev->pci_state->irq;
         break;
      case PCI_GET_NUM_REGS:
         *(int *)pBuf = pgefesPCIDev->pci_state->num_regions;
         break;
      case PCI_GET_HOST_INTRF:
         break;
      case PCI_GET_MSG_Q_ID:
         *(int *)pBuf = (unsigned)pgefesPCIDev->pci_state->msg_q_id;
         break;
      case PCI_GET_PCIREGION_BYTE:
         pBuf++;
         offset = *(unsigned *)pBuf;
         pciConfigInByte(pgefesPCIDev->pci_state->busNumber,
                         pgefesPCIDev->pci_state->deviceNumber,
                         pgefesPCIDev->pci_state->functionNumber,
                         offset,
                         (UINT8 *)pBuf);
         break;
      case PCI_GET_PCIREGION_WORD:
         pBuf++;
         offset = *(unsigned *)pBuf;
         pciConfigInWord(pgefesPCIDev->pci_state->busNumber,
                         pgefesPCIDev->pci_state->deviceNumber,
                         pgefesPCIDev->pci_state->functionNumber,
                         offset,
                         (UINT16 *)pBuf);
         break;
      case PCI_GET_PCIREGION_DWORD:
         pBuf++;
         offset = *(unsigned *)pBuf;
         pciConfigInLong(pgefesPCIDev->pci_state->busNumber,
                         pgefesPCIDev->pci_state->deviceNumber,
                         pgefesPCIDev->pci_state->functionNumber,
                         offset,
                         (UINT32 *)pBuf);
         break;
      case PCI_GET_BOARD_TYPE:
         *(int *)pBuf = (int)pgefesPCIDev->pci_state->dev_id;
         break;
      case PCI_GET_DEVICE_ID:
         *(int *)pBuf = (int)pgefesPCIDev->pci_state->CondorDeviceID;
         break;    
      case PCI_GET_NUM_CHAN:
         *(int *)pBuf = (int)pgefesPCIDev->pci_state->dev_chan;
         break;
      case PCI_GET_DEV_MAP:
      {
         PDEVMAP_T devPTR;
         CEI_INT mindx,device,i;
         CEI_UINT membase,region_count,size;
        
         devPTR = (PDEVMAP_T)pBuf;
         device = pgefesPCIDev->thisDevice;
        
         /*Clear the region_count and struct*/
         region_count=0;
         for (i = 0; i < MAX_MEMORY; i++)
         {
            devPTR->memStartPhy[i]    = 0;    /*Physical address is null.*/
            devPTR->memLengthBytes[i] = 0;    /*Length is null.*/
            devPTR->memHostBase[i]    = NULL; /*Host address is null.*/
            devPTR->flagMapToHost[i]  = 0;              
         }
        
         devPTR->busType = pci_state[device].bustype;
         devPTR->interruptNumber = pci_state[device].irq;
         devPTR->memSections = pci_state[device].region_count;
         devPTR->VendorID = pci_state[device].manufacturerID;       /* don't assume this is a GE Goleta board */
         devPTR->DeviceID = pci_state[device].CondorDeviceID;
         devPTR->mapping = pci_state[device].mapping;
        
         for (mindx = 0; mindx < MAX_MEMORY; mindx++)
         {
#if defined (VXW_SYS_PCI_MAP)
            membase = (unsigned long)sysPciMemToLocalAdrs(pci_state[device].busNumber,pci_state[device].membase[i]);
#elif defined (VXW_SYS_BUS_MAP)
            {
               pCEI_UCHAR addr_base[1];
               int status;

               status = (unsigned long)sysBusToLocalAdrs(PCI_SPACE_MEM_PRI,(char *)pci_state[device].membase[i],addr_base);
               membase = (unsigned long)addr_base[0];
            }
#else
            membase = pci_state[device].membase[mindx];
#endif
            size = pci_state[device].size[mindx];

            if (size > 0)
            {
               devPTR->memStartPhy[region_count] = membase;
               devPTR->memHostBase[region_count] = (pCEI_CHAR)membase;
               devPTR->memLengthBytes[region_count] = size;
               devPTR->flagMapToHost[region_count] = 1;
               region_count++;
            }   
         }
      }
      break;
   }
   return maxBytes;
}    
     
/*******************************************************************************
*   
* gefesPCIDrvClose - called when the device is closed
*   
* For a simple device there are no resources to reclaim and you may specify 
* a NULL pointer instead of createing an empty function. For more complex
* devices the resources allocated in your open function should be freed here. 
*   
* RETURNS: OK or ERROR
*   
* NOMANUAL
*/  
    
static int gefesPCIDrvClose (GEFES_PCI_DEV *pgefesPCIDev)
{   
  return(OK);
}   
    
int gefes_pci_driver_control(int flag,int devnum)
{   
   STATUS status;
   int i;
   char device_name[80];
   
   if (flag)
   {
      dprint("gefes_pci_driver_control add driver %d\n",number_of_gefes_device);
      if(gefesPCIDrvInit() == ERROR)
      {
         return ERROR;
      }
   
      for(i = 0; i < number_of_gefes_device; i++)
      {
         dprint("calling gefesPCIDrvCreate for device %d\n",i);
         status = gefesPCIDrvCreate(NULL,(char *)&i,0);
         if (status == ERROR)
         {
            dprint("gefesPCIDrvCreate error %d\n",status);
            continue;
         }
      }
      return number_of_gefes_device;
   }
   else
   {
      sprintf(device_name,"/gefesDev/%d",devnum);   
      status = gefesPCIDrvDelete(device_name);
      return status;
   }
}

void gefesInitPCI()
{
   int num_devices,dev_indx, status;   
   
   if (!bd_mapped)
   {
      status = mapCondorPCIAddress();   /* detect and map all GE Intelligent Platforms Avionic boards */ 
   } 
  
   dprint("gefesInitPCI %d\n",0);
   num_devices = gefes_pci_driver_control(DRV_INSTALL,0); 
   
   for(dev_indx =0; dev_indx<num_devices; dev_indx++)
   {
      switch (pci_state[dev_indx].CondorDeviceID)
      {
         case CONDOR_AR15_VPX:
            dprint("calling gefes_config_AR15_device for device %d\n",dev_indx);
            gefes_config_AR15_device(dev_indx);
            if(rtpInterruptConnect(dev_indx) == ERROR)
               logMsg("Error setting the ISR for device %d\n",dev_indx,0,0,0,0,0);
            break;
         case CONDOR_PCI_1553:   /* QPMC-1553 */
         case CONDOR_QPCI_1553:
         case CONDOR_QCP_1553: 
         case CONDOR_QPCX_1553:
         case CONDOR_RXMC_1553:
            dprint("calling gefes_config_1553_devices for device %d\n",dev_indx);
            gefes_config_1553_devices(dev_indx);
#ifdef USE_HW_INTERRUPTS
            dprint("calling rtpInterruptConnect for device %d\n",dev_indx);
            if(rtpInterruptConnect(dev_indx) == ERROR)
               logMsg("Error setting the ISR for device %d\n",dev_indx,0,0,0,0,0);
#endif /* USE_HW_INTERRUPTS */
            break;
         case CONDOR_CEI_520:
         case CONDOR_CEI_620:
         case CONDOR_CEI_820:
         case CONDOR_CEI_820TX:
            /* do x20 init stuff here */
            break;
         case CONDOR_CEI_830:
         case CONDOR_R830RX:
         case CONDOR_CEI_430:
         case CONDOR_CEI_530:
         case CONDOR_RAR_CPCI:
         case CONDOR_CEI_A30:
         case CONDOR_RAR_PCIE:
            /* do x30 init stuff here */
            break;
         case CONDOR_P708:
         case CONDOR_PSER:
         case CONDOR_PMIO:  /* P-MIO   */
         case CONDOR_PDIS:  /* P-DIS   */
               /* do init stuff here */
            break;
         case CONDOR_CNIC:
            /* do CNIC init stuff here */
            break;
         case CONDOR_PMC_D1:
            /* do PMC-D1 init stuff here */
            break;
         default:
            logMsg("Unknown device type %x for device %d\n",pci_state[dev_indx].CondorDeviceID,dev_indx,0,0,0,0);
      }
   }
}

void geipBoardShow(void)
{
   CEI_INT unit;
   char mapstr[80];    

   if (number_of_gefes_device > 0)
   {
      printf("%d GE Intelligent Platforms Avionic Device(s) Detected on the PCI Buses\n\n", number_of_gefes_device);
      printf("Unit  Bus  Dev  Func  Device ID        addr0        addr1     irq     Mapping\n");
      printf("-----------------------------------------------------------------------------\n");
   
      for (unit = 0; unit < number_of_gefes_device; unit++)
      {
         if (pci_state[unit].mapping)
            strcpy(mapstr,"  Shared");
         else
            strcpy(mapstr,"physical");
         
         printf("%2d   %4p %4p   %p     %4p   %10p   %10p    %4p    %8s\n",
                unit,
                pci_state[unit].busNumber,
                pci_state[unit].deviceNumber,
                pci_state[unit].functionNumber,
                pci_state[unit].CondorDeviceID,
                pci_state[unit].membase[0],
                pci_state[unit].membase[2],
                pci_state[unit].irq,
                mapstr);
      }
      printf("\n");
   }
   else
      printf("\nNo GE Intelligent Platforms Avionic Boards installed\n");
}

void memDescShow ()
{
   int i;

   for (i = 0; i < sysPhysMemDescNumEnt; ++i)
   {
      printf ("\n\n entry %d", i);
      printf ("\n virtual addr %p", sysPhysMemDesc[i].virtualAddr);
      printf ("\nphysical addr %p", sysPhysMemDesc[i].physicalAddr);
      printf ("\nsize in bytes %u", sysPhysMemDesc[i].len);
      printf ("\ninitialStateMask %p", sysPhysMemDesc[i].initialStateMask);
      printf ("\ninitialState %p", sysPhysMemDesc[i].initialState);
   }
   putchar ('\n');
}

#ifdef _RTP_SHARED_MEM_MAP_ /* special interrupt handling for shared memory */
void isrProcess(unsigned device)
{
#ifdef VXW_PCI_X86
   sysIntDisablePIC(irq_value[device]);
#else
   intDisable(int_level[device]);
#endif
   semGive(isrSemaphore[device]);   
}

static void intProcess(int base,int nchans,int btype,int qID,int isrSemaphore,int device,int f,int g,int h,int j)
{
   unsigned short  control_data;
   short *HIR_Addr, *HWR_Addr[4]; 
   unsigned base_addr;
   int chan;
   MSG_Q_ID mq_id;
   volatile int interrupts_running=1;
   
   unsigned channel_1=0;       
   unsigned channel_2=0;      
   unsigned channel_3=0;      
   unsigned channel_4=0;       
   unsigned DATA_RAM=0;    
   unsigned HW_REG=0;      
 
#ifdef NON_INTEL_WORD_ORDER
   #define int_bit 0x02
#else
   #define int_bit 0x0200
#endif

   base_addr = (unsigned)base;
   mq_id = (MSG_Q_ID)qID;

   switch(btype)
   {
      case 0x110:
      case 0x160:
      case 0x180:
      case 0x210:
      case 0x220: 
         channel_1 = 0x00000000;   /* Offset to channel 1 specific fncts  */
         channel_2 = 0x00200000;   /* Offset to channel 2 specific fncts  */
         channel_3 = 0x00400000;   /* Offset to channel 3 specific fncts  */
         channel_4 = 0x00600000;   /* Offset to channel 4 specific fncts  */
         DATA_RAM  = 0x00100000;   /* Offset to 1553 data RAM area        */
         HW_REG    = 0x00000800;   /* Offset to 1553 Control Register     */
         break;
      case 0x80:
         channel_1 = 0x00000000;   /* Offset to channel 1 specific fncts  */
         channel_2 = 0x00002000;   /* Offset to channel 2 specific fncts  */
         channel_3 = 0x00000000;   /* Not Used                            */
         channel_4 = 0x00000000;   /* Not Used                            */
         DATA_RAM  = 0x00000800;   /* Offset to 1553 data RAM area        */
         HW_REG    = 0x00001000;   /* Offset to 1553 Control Register     */ 
         break;
      case 0x170:
         channel_1 = 0x00000000;   /* Offset to channel 1 specific fncts  */
         channel_2 = 0x00001000;   /* Offset to channel 2 specific fncts  */
         channel_3 = 0x00002000;   /* Not Used                            */
         channel_4 = 0x00003000;   /* Not Used                            */
         DATA_RAM  = 0x00000800;   /* Offset to 1553 data RAM area        */
         HW_REG    = 0x00000200;   /* Offset to 1553 Control Register     */ 
         break;
      case 0x20:
      case 0x40:
      case 0x50:
         channel_1 = 0x00000000;   /* Offset to channel 1 specific fncts  */
         channel_2 = 0x01000000;   /* Offset to channel 2 specific fncts  */
         channel_3 = 0x00000000;   /* Not Used                            */
         channel_4 = 0x00000000;   /* Not Used                            */
         DATA_RAM  = 0x00100000;   /* Offset to 1553 data RAM area        */
         HW_REG    = 0x00400000;   /* Offset to 1553 Control Register     */ 
         break;
      default:
         return;
   }
 
   HIR_Addr    = (short *)(base_addr + channel_1);          /* Host Interface Registers */
   HWR_Addr[0] = (short *)(base_addr + channel_1 + HW_REG); /* HW Registers             */
   HWR_Addr[1] = (short *)(base_addr + channel_2 + HW_REG); /* HW Registers             */
   HWR_Addr[2] = (short *)(base_addr + channel_3 + HW_REG); /* HW Registers             */
   HWR_Addr[3] = (short *)(base_addr + channel_4 + HW_REG); /* HW Registers             */

   /* Clear the interrupt */
   /* read the CSC control reg */
   /* read the 1553_control_register on channel 1 */
   
   while(interrupts_running)
   {
      semTake((SEM_ID)isrSemaphore,WAIT_FOREVER);
      for (chan = 0; chan < nchans; chan++)
      {
         control_data = *HWR_Addr[chan];    
         if(control_data & int_bit)
         {
            /* write 0 to the write_interrupt_bit */        
            (HWR_Addr[chan])[0xb] = 0x0;    /* Clear the interrupt on Channel 1 */
            msgQSend(mq_id,(char *)&mq_data[chan],sizeof(unsigned),NO_WAIT, MSG_PRI_NORMAL);
         }
      }  
#ifdef VXW_PCI_X86
      sysIntEnablePIC(irq_value[device]);
#else
      intEnable(int_level[device]);
#endif    
   }  
}

#else /* _RTP_SHARED_MEM_MAP_ */

static void intProcess(unsigned device)
{
   unsigned short  control_data;
   short *HIR_Addr, *HWR_Addr[10]; 
   unsigned base_addr;
   int chan;
   MSG_Q_ID mq_id;
   
   unsigned channel_1=0;       
   unsigned channel_2=0;      
   unsigned channel_3=0;      
   unsigned channel_4=0;
   unsigned channel_5=0;       
   unsigned channel_6=0;      
   unsigned channel_7=0;      
   unsigned channel_8=0;     
   unsigned channel_9=0;      
   unsigned channel_10=0;
   unsigned DATA_RAM=0;    
   unsigned HW_REG=0;      
 
#ifdef NON_INTEL_WORD_ORDER
   #define int_bit 0x02
#else
   #define int_bit 0x0200
#endif
   
   base_addr = pci_state[device].membase[pci_state[device].mem_base_reg];
   mq_id = pci_state[device].msg_q_id;
                                                      
   switch(pci_state[device].dev_id)
   {
      case 0x110:
      case 0x160:
      case 0x180:
      case 0x210:
      case 0x220: 
      case 0x250:
      case 0x260:
         channel_1 = 0x00000000;   /* Offset to channel 1 specific fncts  */
         channel_2 = 0x00200000;   /* Offset to channel 2 specific fncts  */
         channel_3 = 0x00400000;   /* Offset to channel 3 specific fncts  */
         channel_4 = 0x00600000;   /* Offset to channel 4 specific fncts  */
         DATA_RAM  = 0x00100000;   /* Offset to 1553 data RAM area        */
         HW_REG    = 0x00000800;   /* Offset to 1553 Control Register     */
         break;
      case 0x280: 
         channel_1 = 0x00000000;   /* Offset to channel 1 specific fncts  */
         channel_2 = 0x00200000;   /* Offset to channel 2 specific fncts  */
         channel_3 = 0x00280000;   /* Offset to channel 3 specific fncts  */
         channel_4 = 0x00300000;   /* Offset to channel 4 specific fncts  */
         channel_5 = 0x00380000;   /* Offset to channel 5 specific fncts  */
         channel_6 = 0x00400000;   /* Offset to channel 6 specific fncts  */
         channel_7 = 0x00480000;   /* Offset to channel 7 specific fncts  */
         channel_8 = 0x00500000;   /* Offset to channel 8 specific fncts  */
         channel_9 = 0x00580000;   /* Offset to channel 9 specific fncts  */
         channel_10 =0x00600000;   /* Offset to channel 10 specific fncts  */
         HW_REG    = 0x00000800;   /* Offset to 1553 Control Register     */
         break;
      case 0x80:
         channel_1 = 0x00000000;   /* Offset to channel 1 specific fncts  */
         channel_2 = 0x00002000;   /* Offset to channel 2 specific fncts  */
         channel_3 = 0x00000000;   /* Not Used                            */
         channel_4 = 0x00000000;   /* Not Used                            */
         DATA_RAM  = 0x00000800;   /* Offset to 1553 data RAM area        */
         HW_REG    = 0x00001000;   /* Offset to 1553 Control Register     */ 
         break;
      case 0x170:
         channel_1 = 0x00000000;   /* Offset to channel 1 specific fncts  */
         channel_2 = 0x00001000;   /* Offset to channel 2 specific fncts  */
         channel_3 = 0x00002000;   /* Not Used                            */
         channel_4 = 0x00003000;   /* Not Used                            */
         DATA_RAM  = 0x00000800;   /* Offset to 1553 data RAM area        */
         HW_REG    = 0x00000200;   /* Offset to 1553 Control Register     */ 
         break;
      case 0x20:
      case 0x40:
      case 0x50:
         channel_1 = 0x00000000;   /* Offset to channel 1 specific fncts  */
         channel_2 = 0x01000000;   /* Offset to channel 2 specific fncts  */
         channel_3 = 0x00000000;   /* Not Used                            */
         channel_4 = 0x00000000;   /* Not Used                            */
         DATA_RAM  = 0x00100000;   /* Offset to 1553 data RAM area        */
         HW_REG    = 0x00400000;   /* Offset to 1553 Control Register     */ 
         break;
      default:
         return;
   }
 
   HIR_Addr    = (short *)(base_addr + channel_1);           /* Host Interface Registers */
   HWR_Addr[0] = (short *)(base_addr + channel_1 + HW_REG);  /* HW Registers             */
   HWR_Addr[1] = (short *)(base_addr + channel_2 + HW_REG);  /* HW Registers             */
   HWR_Addr[2] = (short *)(base_addr + channel_3 + HW_REG);  /* HW Registers             */
   HWR_Addr[3] = (short *)(base_addr + channel_4 + HW_REG);  /* HW Registers             */
#if 0
   HWR_Addr[4] = (short *)(base_addr + channel_5 + HW_REG);  /* HW Registers             */
   HWR_Addr[5] = (short *)(base_addr + channel_6 + HW_REG);  /* HW Registers             */
   HWR_Addr[6] = (short *)(base_addr + channel_7 + HW_REG);  /* HW Registers             */
   HWR_Addr[7] = (short *)(base_addr + channel_8 + HW_REG);  /* HW Registers             */
   HWR_Addr[8] = (short *)(base_addr + channel_9 + HW_REG);  /* HW Registers             */
   HWR_Addr[9] = (short *)(base_addr + channel_10 + HW_REG); /* HW Registers             */
#endif

   /* Clear the interrupt */
   /* read the CSC control reg */
   /* read the 1553_control_register on channel 1 */

#ifdef SPIN_LOCK_PROTECT
   spinLockIsrTake(&spinlock[device]);
#endif                            
   for(chan=0;chan<pci_state[device].dev_chan;chan++)
   {
      control_data = *HWR_Addr[chan]; 
      if(control_data & int_bit)
      {
         /* write 0 to the write_interrupt_bit */ 
         (HWR_Addr[chan])[0xb] = 0x0;    /* Clear the interrupt on Channel 1 */	
         msgQSend(mq_id,(char *)&mq_data[chan],sizeof(unsigned),NO_WAIT, MSG_PRI_NORMAL);
      }
   }
#ifdef SPIN_LOCK_PROTECT
   spinLockIsrGive(&spinlock[device]);
#endif
}
#endif /*_RTP_SHARED_MEM_MAP_ */

