/*============================================================================*
 * FILE:                A R _ E R R O R . H
 *============================================================================*
 *               
 * COPYRIGHT (C) 1997-2011 BY
 *          GE INTELLIGENT PLATFORMS, INC. SANTA BARBARA, CALIFORNIA
 *          ALL RIGHTS RESERVED.
 *
 *          THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *          COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *          THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *          OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *          AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *          SOFTWARE IS HEREBY TRANSFERRED.
 *
 *          THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *          NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY GE 
 *          INTELLIGENT PLATFORMS.
 *
 *===========================================================================*
 *
 * FUNCTION:    Error strings for CEI-x30 utilities.
 *
 * HISTORY:
 *
 *   Date    By   Comments
 * -------- ----- ------------------------------------------------------------
 * 11/14/05  BDW  Added P-708 error codes.
 * 11/07/07  BDW  Removed 'WinRT' reference from driver error status string
 * 02/02/09  BDW  Modified the global error array names.
 *
 *===========================================================================*/
                   
#ifndef AR_ERROR_H
#define AR_ERROR_H

#define MAX_830_1ST_ERRORS        6
#define MAX_830_2ND_ERRORS        52
#define MAX_830_INTFC_ERRORS      23
#define MAX_830_WMS_ERRORS        26
#define MAX_830_ERRORS            MAX_2ND_ERRORS  
                         
/*---------------------------------------------------------------------------*
 *  These are general RCVR status codes.
 *---------------------------------------------------------------------------*/

pCEI_CHAR ar_830_errors_1[MAX_830_1ST_ERRORS] = {
   "No data received",                     /*  0 */
   "Normal successful completion",         /*  1 */
   "Unknown Error Code",                   /*  2 */
   "Unknown Error Code",                   /*  3 */
   "No Error, Received data"               /*  4 */
   "Invalid ARINC 429 msg detected",       /*  5 */
};
                                                       
/*---------------------------------------------------------------------------*
 *  The following is the standard ARINC error message group.  The 
 *  corresponding values are offset from 1000
 *---------------------------------------------------------------------------*/

CEI_CHAR *ar_830_errors_2[MAX_830_2ND_ERRORS] = {
   "Unknown Error Code",                    /*  0 */
   "Unknown Error Code",                    /*  1 */
   "Invalid configuration command",         /*  2 */
   "Invalid configuration value",           /*  3 */
   "Transmit buffer overflow",              /*  4 */
   "Invalid board argument",                /*  5 */
   "Timeout synchronizing with the ARINC",  /*  6 */
   "Bad load",                              /*  7 */
   "Board has not been loaded",             /*  8 */
   "Error initializing timer",              /*  9 */
   "Host CPU can't keep up with data rate", /* 10 */
   "Can't initialize timer on slave",       /* 11 */
   "Memory address error",                  /* 12 */
   "Memory Test failure",                   /* 13 */
   "Invalid string",                        /* 14 */
   "Invalid equipment ID",                  /* 15 */
   "Checksum error loading slave",          /* 16 */
   "No response on receiver",               /* 17 */
   "Translation not available",             /* 18 */
   "Invalid argument",                      /* 19 */
   "Invalid label",                         /* 20 */
   "Driver load failed",                    /* 21 */
   "Device driver error encountered",       /* 22 */
   "Channel timeout on receive function",   /* 23 */
   "Function not supported by driver",      /* 24 */
   "Improperly specified DAC value",        /* 25 */
   "Firmware failure",                      /* 26 */
   "Memory Test failure",                   /* 27 */
   "Illegal Error Code",                    /* 28 */
   "Not programmed for Enhanced Operation", /* 29 */
   "Supported device not detected",         /* 30 */
   "Wrap Test data mismatch failure",       /* 31 */
   "Wrap Test time-tag/baud rate failure",  /* 32 */
   "Incomplete ARINC 708 data frame    ",   /* 33 */
   "Too much data in ARINC 708 frame   ",   /* 34 */
   "BIT terminated due to external data",   /* 35 */
   "Wrap Test missing data failure",        /* 36 */
   "Failed to install/uninstall ISR",       /* 37 */
   "Failed to acquire/release board lock",  /* 38 */
   "Failed to acqire/rel board cfg lock",   /* 39 */
   "Feature not currently supported",       /* 40 */
   "Feature not supported under this o/s",  /* 41 */
   "Unable to grant exclusive load",        /* 42 */
   "Int not supported by board and/or o/s", /* 43 */
   "Int support has not been enabled",      /* 44 */
   "Other process already installed ISR",   /* 45 */
   "Thread waiting for data on this rx ch", /* 46 */
   "Fail waiting for data on this rx chan", /* 47 */
   "Failed to signal waiting thread",       /* 48 */
   "Requested oper not supported on chan",  /* 49 */
   "Failed to allocate shared mem region",  /* 50 */
   "Failed to allocate shared mem region"   /* 51 */

};
                                   
/*---------------------------------------------------------------------------*
 *  These are general interface library failures
 *---------------------------------------------------------------------------*/

pCEI_CHAR ar_830_intfc_errors[MAX_830_INTFC_ERRORS] = {
   "Library Does Not Exist"      ,         /*  0 */
   "Function Not Available in Library",    /*  1 */
   "Board Not Installed",                  /*  2 */
   "Installed Board in Not Supported",     /*  3 */
   "Unknown Error Code",                   /*  4 */
   "Unknown Error Code",                   /*  5 */
   "Unknown Error Code",                   /*  6 */
   "Unknown Error Code",                   /*  7 */
   "Unknown Error Code",                   /*  8 */
   "Unknown Error Code",                   /*  9 */
   "Unknown Error Code",                   /* 10 */
   "Unknown Error Code",                   /* 11 */
   "Unknown Error Code",                   /* 12 */
   "Unknown Error Code",                   /* 13 */
   "VISA Error - Invalid Object",          /* 14 */
   "VISA Error - Resource Locked",         /* 15 */
   "VISA Error - Invalid Expression",      /* 16 */
   "VISA Error - Resource Not Found",      /* 17 */
   "VISA Error - Invalid Resource Name",   /* 18 */
   "VISA Error - Invalid Access Mode",     /* 19 */
   "Unknown Error Code",                   /* 20 */
   "VISA Error - Timeout Expired",         /* 21 */
   "VISA Error - Deallocation Problem"     /* 22 */
};
                                                       
/*---------------------------------------------------------------------------* 
 *  The following error messages are specific to the Williamsburg API and can 
 *  be ignored for general usage. 
 *---------------------------------------------------------------------------*/

pCEI_CHAR arw_830_errors[MAX_830_WMS_ERRORS] =
{
   "Unknown Error Code",                  /* 2000 */
   "ARS_INVVERS",                         /* 2001 */
   "ARS_TIMEOUT",                         /* 2002 */
   "ARS_INVFILE",                         /* 2003 */
   "ARS_INVCOMMVERS",                     /* 2004 */
   "ARS_FILE_FAIL",                       /* 2005 */
   "ARS_INV_CHANNEL",                     /* 2006 */
   "ARS_NCTS",                            /* 2007 */
   "ARS_BUSY",                            /* 2008 */
   "ARS_FILETOOBIG",                      /* 2009 */
   "ARS_NO_RECVR_RESP",                   /* 2010 */
   "ARS_SEQUENCE_ERROR",                  /* 2011 */
   "ARS_WRITE_BUFF_OVERFL",               /* 2012 */
   "ARS_UNEXPECTED_WORD",                 /* 2013 */
   "ARS_INVALID_CASE",                    /* 2014 */
   "ARS_DUPLICATE_LDU",                   /* 2015 */
   "ARS_GOT_ALO",                         /* 2016 */
   "ARS_GOT_TEST",                        /* 2017 */
   "ARS_PATTERN_FAIL",                    /* 2018 */
   "ARS_MEM_ALLOC_ERR",                   /* 2019 */
   "ARS_NOT_OPENED",                      /* 2020 */
   "ARS_EXCESS_NAK",                      /* 2021 */
   "ARS_EXCESS_SYN",                      /* 2022 */
   "ARS_NAK",                             /* 2023 */
   "ARS_SYN"                              /* 2024 */
   "ARS_EOF"                              /* 2025 */
};

#endif /* AR_ERROR_H */

