/*===========================================================================*
 * FILE:                    C D E V _ A P I . H
 *===========================================================================*
 *
 * COPYRIGHT (C) 2002-2011
 *          GE INTELLIGENT PLATFORMS, INC. SANTA BARBARA, CALIFORNIA
 *          ALL RIGHTS RESERVED.
 *
 *          THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *          COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *          THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *          OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *          AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *          SOFTWARE IS HEREBY TRANSFERRED.
 *
 *          THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *          NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY GE 
 *          INTELLIGENT PLATFORMS.
 *
 *===========================================================================*
 *
 * FUNCTION:    Header file for all CEI-x30 API public contants, public
 *              and local API data structure definitions, and public 
 *              API function prototypes.
 * 
 *---------------------------------------------------------------------------*
 *          
 * HISTORY:   
 *
 *   Date    By   Comments
 * -------- ----- ------------------------------------------------------------
 * 10/26/01  BDW  Created.
 * 11/06/03  BDW  Added CEI-830 channel types.
 * 09/14/05  BDW  Added support for Linux via inclusion of CDEV_LNX.H
 *                Included relocating some compiler directive defines from
 *                CDEV_API.C
 * 10/21/05  BDW  Added support for the CEI-430 firmware load:        
 *                 - constants for new channel types DIFFERENTIAL_IN_CHANNEL
 *                   and DIFFERENTIAL_OUT_CHANNEL (API internal usage only)
 *                   modifying the value of NUMBER_OF_CHANNEL_TYPES
 *                 - added the new h/w type CEIDEV_CHANNEL_TYPE_DIFFERENTIAL
 *                   modifying the value of CEIDEV_CHANNEL_TYPE_MAX
 *                 - added new board identifier CEIDEV_ALT_CONFIGURATION_430
 * 12/06/05  BDW  Added channel count report options for AR_Get_Device_Config
 *                   ARU_CHAN_COUNT_429        448
 *                   ARU_CHAN_COUNT_573        449
 *                   ARU_CHAN_COUNT_DISC       450
 *                   ARU_CHAN_COUNT_DIFF       451
 * 09/19/06  BDW  Added the AMC-A30 board type constant AMCA30
 *                Increased the VxWorks & Linux MAX_BOARDS definition to 
 *                   support up to 8 devices
 * 05/11/07  BDW  Added support for the EC-A30, CEI-530, -630 and -830RX boards
 * 07/25/07  BDW  Support for R830RX ARU_IRIG_OUTPUT_ENABLE option added
 *                   to ar_set_device_config/ar_get_device_config
 * 10/24/07  BDW  Added the configuration values CEIDEV_CONFIG_RAR_CPCI (12) 
 *                                               CEIDEV_CONFIG_RAR_EC (13)
 *                Increased CEIDEV_NUM_CONFIGURATIONS from 12 to 14
 * 01/07/08  BDW  Added the member "serialNumber" to the ARINC_DEVICE_TYPE
 *                   data structure, supporting RAR-EC PPROM serial number
 * 02/18/08  BDW  Removed the data structure type def CDEV_CHANNEL_IDENT_TYPE
 * 03/19/08  BDW  Modified the prototype for ar_get_boardnameLV.
 * 11/19/08  BDW  Updated compiler directive usage in support of Integrity o/s.
 * 06/23/09  BDW  Added Enhanced CEI-x30 Interface support constants used in
 *                ar_get/set_device_config, ARU_HW_ENHANCE_CHECK & ARU_HW_ENHANCE_UPDATE 
 * 05/19/10  BDW  Added support for the RAR-PCIE to the following items:
 *                   define CEIDEV_CONFIG_RAR_PCIE to the RAR-PCIE Configuration code
 *                   set CEIDEV_LAST_CEIX30_CONFIG to CEIDEV_CONFIG_RAR_PCIE                       
 * 11/01/10  BDW  Modified the return type for ar_get_base_addr to be a pointer
 * 09/16/11  BDW  Added CEI-430A support with CEIDEV_CONFIG_CEI430A and CEI430A definitions
 *                                          
 *===========================================================================*/
#ifndef CEIDEV_API_H
#define CEIDEV_API_H

#ifndef LABVIEW_RT
#define API_VERSION             "3.10"
#else
#define API_VERSION             "RT_3.10"
#endif

/*------------------------------------------------------------------------*
 *  Microsoft C protected mode program
 *------------------------------------------------------------------------*/

#if defined (_WINDOWS)  
   #define _Windows
#elif defined(_WINDLL)
   #define _Windows
#elif defined(WIN32)
   #define _Windows
#endif 
                      
/*------------------------------------------------------------------------* 
 *  For CVI and Borland C++ Compilers
 *------------------------------------------------------------------------*/
                   
#if defined (__WIN32__)    
   #if !defined(_CVI_)
      #if !defined(_WIN32)
         #define _WIN32
      #endif
   #endif
#endif 

#if !defined (_WIN32)
   #define EXPORT __export
   #define EXPORT32
#else
   #include <windows.h>
   #define EXPORT32  

   #if !defined (_CVI_)
      #include <windowsx.h>
   #endif
   #if defined(__BORLANDC__)
      #define EXPORT __export
   #else         
      #define EXPORT
   #endif
#endif

#ifndef LABVIEW_RT
   #if defined(_LINUX_X86_) || defined(_LINUX_PPC_)
      #include <stdio.h>
      #include <stdlib.h>
      #include <time.h>
      #include <sys/types.h>
      #include <sys/timeb.h>
   #else
      #include <ctype.h>
      #include <time.h>
      #include <stdio.h>
   #endif
#endif

#include "cei_types.h"

#if defined (VXWORKS) || defined (_LINUX_X86_) || defined (_LINUX_PPC_) || defined (INTEGRITY_PCI_PPC)
   #ifndef MAX_BOARDS
      #define MAX_BOARDS  8
   #endif
#else
   #define MAX_BOARDS 16
#endif

#define TIME_TAG_TYPE CEI_UINT64       /* __int64 */

#include "cdev_hw.h"


/*===========================================================================*
 * Internal API Usage - Device Description Structures and Definitions
 *===========================================================================*
 */
typedef float CEI_FLOAT;
typedef CEI_UINT32 ARINC;

/* GE Intelligent Platforms, Santa Barbara Division Products,
   Product Identification Assignments
 */
#define UNSUPPORTED   1

/* this next group are from UTILDEFS.H and are used universally */
#define CEI520        6 
#define CEI620       10 
#define CEI820       11 
#define CEI715       16 
#define CEI830       19     /* supported by this API */
#define PMC708       20
#define CEI430       21     /* supported by this API */
#define AMCA30       22     /* supported by this API */
#define CEI530       26     /* supported by this API */
#define R830RX       27     /* supported by this API */
#define RARCPCI      28     /* supported by this API */
#define RAREC        29     /* supported by this API */
#define RARPCIE      30
#define CEI430A      31
#define MAC          32

/* this group is defined solely for use with ar_query_device()  */
#define CNIC        100
#define PMCD1       101
#define PMCD2       102
#define EPMC        103
#define PSER        104
#define PMIO        105
#define PDIS        106
#define P10SF       107
#define AMC1553     108
#define PCx1553     109
#define QPCI        110
#define QCP         111
#define QPCX        112
#define CEI630       28     /* redundant board type for the RAR-CPCI using new board name format */


#define MAX_LABEL               256
#define MAX_SDI                 4
#define MAX_ESSM                8
#define MAX_CHANNEL             16
#define MAX_SCHEDULED_MSGS      1024
#define NUM_SUBFRAME_SYNC_WORDS 4

/*---------------------------------------------------------------------------*
 * The ARINC_LOCAL_TYPE structure contains local-process parameters
 *   regarding specific device mapping and API usage
 *---------------------------------------------------------------------------*/

typedef struct ARINC_LOCAL {
/*   CEI_HANDLE  handle; */
   pCEI_UINT32 baseAddr;          /* Plug'n'Play Address to which the device is mapped */
   pCEI_UINT32 devicePtr;         /* Pointer to location at which the device is mapped */
   pCEI_VOID   sharedMemoryHandle;/* handle to multi-process shared memory region      */
   CEI_UINT32  multiThreadProtect;/* multi-thread/process Mutex protection flag        */
} ARINC_LOCAL_TYPE;

/*---------------------------------------------------------------------------*
 * The ARINC_LOCAL_TYPE structure contains global configuration and  
 *   operational information regarding the specific device
 *---------------------------------------------------------------------------*/

typedef struct ARINC_GLOBAL {
   CEI_UINT16  numRx;                      /* Total number of Receivers on device      */
   CEI_UINT16  numTx;                      /* Total number of Transmitters on device   */
   CEI_UINT32  boardInitialized;           /* Indicates this board f/w has been loaded */
   CEI_UINT32  attachedProcessCount;       /* Number of processes actively attached to this device */
   CEI_UINT32  receiveMode;                /* Receive buffer operational mode, BUFFERED or MERGED */
   CEI_UINT32  transmitMode;               /* Transmitter configuration as CSDB, 573-HBP, or 573-BPRZ */
   CEI_UINT32  serialNumber;               /* RAR-EC Serial Number, stored in EEPROM */
   CEI_UINT32  firmwareVersion;            /* Firmware version loaded on this device */
   CEI_UINT32  configuration;              /* Board configuration for this device */
   CEI_UINT32  alternateJconfiguration;    /* ARINC 717 configuration flag for this device */
   CEI_UINT32  irigInstalled;              /* Indication that IRIG is installed on the device   */
   CEI_UINT32  deviceTimeTagReference;     /* The active time-tag reference selected for this device */
   CEI_UINT32  deviceX20CompatibleResolution; /* Selected x20-compatible time-ref resolution for this device */
   CEI_UINT32  numberOfChannels[NUMBER_OF_CHANNEL_TYPES]; /* the following 3 items track channel types and */
   CEI_UINT32  firstChannel[NUMBER_OF_CHANNEL_TYPES];     /* channel counts installed on this device       */
   CEI_UINT32  lastChannel[NUMBER_OF_CHANNEL_TYPES];
   CEI_UINT32  receiveChannelMessageCount[CEIDEV_MAX_RX_CHAN]; /* current msg count logged via ar_clr_rx_count */
   CEI_UINT32  ceiPciInterruptEnabled;        /* Set when PCI Interrupts & ISR are Enabled */
   TIME_TAG_TYPE userProgrammableIrigOffset;  /* IRIG sample time offset used to coordinate with other IRIG devices */
#if defined (NO_INT64)
   double currentIrigTimeUsec;             /* 64-bit 1usec IRIG time-of-year placeholder  */
   double irigReferenceTimeTag;
#else
   TIME_TAG_TYPE currentIrigTimeUsec;
   TIME_TAG_TYPE irigReferenceTimeTag;  
#endif
} ARINC_GLOBAL_TYPE;


/*---------------------------------------------------------------------------*
 * Hardware Configuration Definitions (as defined by the hardware)
 *---------------------------------------------------------------------------*/
#define CEIDEV_NUM_CONFIGURATIONS 16    /* number of configurations avail.   */
#define CEIDEV_NUM_ALT_CONFIGURATIONS 3 /* number of alt. configs avail.     */
#define CEIDEV_CONFIGURATION_NULL 0     /* configurations 1-6 are CEI-715    */
#define CEIDEV_CONFIG_CEI830      7     /* Unspecified CEI-830 configuration */
#define CEIDEV_CONFIG_CEI430      8     /* Unspecified CEI-430 configuration */
#define CEIDEV_CONFIG_AMCA30      9     /* Unspecified AMC-A30 configuration */
#define CEIDEV_CONFIG_CEI530     10     /* Unspecified CEI-530 configuration */
#define CEIDEV_CONFIG_R830RX     11     /* CEI-830RX - fixed configuration   */
#define CEIDEV_CONFIG_RAR_CPCI   12     /* Unspecified RAR-CPCI configuration*/
#define CEIDEV_CONFIG_RAR_EC     13     /* Unspecified RAR-EC configuration  */
#define CEIDEV_CONFIG_RAR_PCIE   14     /* Unspecified RAR-PCIE configuration*/
#define CEIDEV_CONFIG_CEI430A    15     /* Unspecified CEI-430 configuration */
#define CEIDEV_FIRST_CEIX30_CONFIG CEIDEV_CONFIG_CEI830
#define CEIDEV_LAST_CEIX30_CONFIG CEIDEV_CONFIG_CEI430A
#define CEIDEV_CONFIGURATION_MASK       0x000F

/* Pre-430 legacy defines */
#define CEIDEV_FIRST_CEI830_CONFIG CEIDEV_CONFIG_CEI830
#define CEIDEV_LAST_CEI830_CONFIG CEIDEV_CONFIG_CEI830

/*===========================================================================*
 * External API Usage - Device Description Definitions and Structures
 *===========================================================================*

 *----------------------------------------------------------------------------*
 *  Utility return status codes
 *----------------------------------------------------------------------------*
 *  IMPORTANT NOTE: These constants mirror those defined in INTERFACE.H for 
 *                  the IP-Avionics BusTools/LabVIEW error definitions.
 *----------------------------------------------------------------------------*/

#define ARS_FAILURE             -1    /* General API routine failed           */
#define ARS_NODATA               0    /* No data returned from "get" routine  */
#define ARS_NORMAL               1    /* Normal successful completion         */
#define ARS_MOREDATA             3    /* not used                             */
#define ARS_GOTDATA              4    /* Some data returned from "get" routine*/
#define ARS_BAD_MESSAGE          5    /* Invalid ARINC 429 message detected   */
#define ARS_INVHARCMD         1002    /* Requested operation not supported    */
#define ARS_INVHARVAL         1003    /* Specified attribute was invalid      */
#define ARS_XMITOVRFLO        1004    /* Transmit buffer overflow             */
#define ARS_INVBOARD          1005    /* Invalid device ID was specified      */
#define ARS_NOSYNC            1006    /* Transmitter flush failed completion  */
#define ARS_BADLOAD           1007    /* Device firmware load failed          */
#define ARS_BRDNOTLOAD        1008    /* not used                             */
#define ARS_SYNCTIMEOUT       1009    /* not used                             */
#define ARS_SYNCOVERRUN       1010    /* not used                             */
#define ARS_BADINIT           1011    /* not used                             */
#define ARS_MEMADERR          1012    /* not used                             */
#define ARS_MEMWRERR          1013    /* Memory Test failure                  */
#define ARS_INVSTRING         1014    /* not used                             */
#define ARS_INVEQID           1015    /* not used                             */
#define ARS_CHECKSUM          1016    /* not used                             */
#define ARS_NORESPONSE        1017    /* not used                             */
#define ARS_UNSUPTYPE         1018    /* not used                             */
#define ARS_INVARG            1019    /* Specified function arg was invalid   */
#define ARS_INVLABEL          1020    /* not used                             */
#define ARS_DRIVERFAIL        1021    /* PCI Interrupt initialization fail    */
#define ARS_WINRTFAIL         1022    /* Windows device driver failure        */
#define ARS_CHAN_TIMEOUT      1023    /* Time out while waiting for data      */
#define ARS_NO_HW_SUPRT       1024    /* Function not supported by hardware   */
#define ARS_BAD_DAC_VAL       1025    /* Specified DAC value bad (<0 or >255) */
#define ARS_BAD_FPGA          1026    /* Bad num of transmitters or receivers */
#define ARS_BAD_STATIC        1027    /* Internal memory read/write failure   */
#define ARS_LAST_ERROR        1028    /* Returns addn'l info about prev error */
#define ARS_HW_CONSISTENCY    1029    /* Hardware config is not consistant    */
#define ARS_HW_DETECT         1030    /* Failed to detect CEI-xxx hardware    */
#define ARS_WRAP_DATA_FAIL    1031    /* Internal wrap test data failure      */ 
#define ARS_WRAP_RATE_FAIL    1032    /* Wrap Test timetag/baud rate mismatch */
#define ARS_SHORT_DATA_FRAME  1033    /* Incomplete ARINC-708 data frame      */
#define ARS_LONG_DATA_FRAME   1034    /* Too much data in ARINC-708 frame     */
#define ARS_WRAP_FLUSH_FAIL   1035    /* BIT cannot execute due to extern rx  */
#define ARS_WRAP_DROP_FAIL    1036    /* Wrap Test missing data failure       */
#define ARS_INT_ISR           1037    /* Failed to install/uninstall ISR      */
#define ARS_BOARD_MUTEX       1038    /* Failed to acquire/release board lock */
#define ARS_CONFIG_MUTEX      1039    /* Failed to acquire/release cfg lock   */
#define ARS_NOT_SUPPORTED     1040    /* Requested feature not supported      */
#define ARS_NO_OS_SUPPORT     1041    /* No oper sys support for req feature  */
#define ARS_NOT_EXCLUSIVE     1042    /* Cannot grant exclusive load request  */
#define ARS_NO_INT_SUPPORT    1043    /* Interrupt handling not supported     */
#define ARS_NO_INT_ENABLED    1044    /* Interrupt handling not enabled       */
#define ARS_ISR_ALREADY_INST  1045    /* Another proc already installed ISR   */
#define ARS_THREAD_WAITING    1046    /* Already thread waiting for rcv data  */
#define ARS_WAIT_FAILED       1047    /* Failed to wait for rcv data          */
#define ARS_SIGNAL_FAILED     1048    /* Failed to signal waiting thread      */
#define ARS_CHANNEL_TYPE      1049    /* Invalid channel type for req func    */
#define ARS_ERR_SH_MEM_OBJ    1050    /* Failed to allocate shared mem region */
#define ARS_ERR_SH_MEM_MAP    1051    /* Failed to allocate shared mem region */

#define INTFC_BASE_ERR_CODES  1900

/*---------------------------------------------------------------------------*
 *  This set of configuration constants are specifically used with calls to 
 *  satisfy the LabVIEW VIs and LabWindows/CVI Applications.  Invocations of
 *  ar_set_arinc_config() (in INTERFACE.DLL) are translated by the function
 *  ar_set_device_config() within the API DLL.  Direct access to this
 *  DLL should use this function with the following constants or use
 *  ar_set_config() using other constants defined below.
 *---------------------------------------------------------------------------*/
#define ARU_RX_BITRATE               1
#define ARU_RX_PARITY                3
#define ARU_RECV_MODE                5
#define ARU_RX_STORAGE_MODE          7
#define ARU_RX_DISABLE               9
                                     
#define ARU_TX_BITRATE               2
#define ARU_TX_PARITY                4
#define ARU_TX_BIT_ERROR             6
#define ARU_TX_GAP_ERROR             8
#define ARU_TX_DISABLE               10

/* ar_set_raw_mode channel types */
#define ARU_XMIT                     34
#define ARU_RECV                     35
#define TRANSMIT_CHANNEL             0
#define RECEIVE_CHANNEL              1

/*---------------------------------------------------------------------------*
 *  This set of configuration "value parameter" constants are used with calls 
 *  to either the function ar_set_config() or ar_set_device_config()
 *---------------------------------------------------------------------------*/
#define AR_WRAP_ON                   0  /* Receiver Internal wrap enable        */
#define AR_WRAP_OFF                  1  /* Receiver Internal wrap disable       */
#define AR_ON                        7  /* General "Enable" flag                */
#define AR_OFF                       8  /* General "Disable" flag               */
#define AR_LO                        0  /* Discrete & digital I/O states        */
#define AR_HI                        1  
                                     
#define ARU_SPEED_HIGH               0  /* Bus speed fixed 100Kbps assignment   */
#define ARU_SPEED_LOW                1  /* Bus speed fixed 12.5Kbps assignment  */
#define AR_HIGH                      0  /* Used to set HIGH speed (100 Kbaud)      */
#define AR_LOW                       1  /* Used to set LOW speed (12.5 Kbaud)      */
#define ARU_PARITY_ODD               0  
#define AR_ODD                       0
#define ARU_PARITY_EVEN              1
#define AR_EVEN                      1
#define ARU_PARITY_NONE              2
#define AR_RAW                       0x2000

#define ARU_LABEL_ONLY               0
#define ARU_LABEL_WITH_SDI           1

/*----------------------------------------------------------------------------*
 * ar_set_preload_config constants
 * used along with ON/OFF in ar_set_multithread_protect
 *----------------------------------------------------------------------------*/
#define ARU_CONCURRENCY_MODE         1
#define ARU_INSTALL_INT_HANDLER      2
#define AR_CONC_NONE                 0
#define AR_CONC_MULTITHRD            1
#define AR_CONC_MULTIPROC            2

/*---------------------------------------------------------------------------*
 *  This set of configuration constants are specifically used for the 
 *  "item" parameter with invocations to the ar_set/get_config() and 
 *  ar_set/get_device_config() functions for CEI-x30-specific operations.
 *---------------------------------------------------------------------------*/
#define ARU_CHANNEL_ENABLE            11
#define ARU_DISCRETE_OUT              12
#define ARU_DIGITAL_OUT               13
#define ARU_DISCRETE_IN               14
#define ARU_DIGITAL_IN                15
#define ARU_RX_FIFO_ENABLE            16
#define ARU_TX_FIFO_ENABLE            17
#define ARU_RX_MERGED_MODE            18
#define ARU_TX_FIFO_COUNT             19
#define ARU_FW_VERSION                20
#define ARU_CONFIGURATION             21
#define ARU_DIFFERENTIAL_IN           22
#define ARU_DIFFERENTIAL_OUT          23
#define ARU_RX_RESET                  24
#define ARU_TX_RESET                  25
#define ARU_IRIG_OUTPUT_ENABLE        26
#define ARU_IRIG_INPUT_TIME           27
#define ARU_RX_FIFO_COUNT             28
#define ARU_HW_INTERRUPT_ENABLE       29
#define ARU_HW_ENHANCE_CHECK          30
#define ARU_HW_ENHANCE_UPDATE         31
#define ARU_BYPASS_FW_LOAD            32
#define ARU_WAIT_ON_EVENT             33
#define ARU_RX_MSG_COUNT              35
#define ARU_TX_MSG_COUNT              36
#define ARU_INSERT_INT_Q_ENTRY        37
#define ARU_ACCESS_SNAPSHOT_BUFFER    38
#define ARU_DEVICE_DISABLE            39

#define ARU_DISCRETE_IN1             260
#define ARU_DISCRETE_IN2             261
#define ARU_DISCRETE_IN3             262
#define ARU_DISCRETE_IN4             263
#define ARU_DIGITAL_IN1              264
#define ARU_DIGITAL_IN2              265
#define ARU_DIGITAL_IN3              266
#define ARU_DIGITAL_IN4              267
#define ARU_DISCRETE_OUT1            268
#define ARU_DISCRETE_OUT2            269
#define ARU_DISCRETE_OUT3            270
#define ARU_DISCRETE_OUT4            271
#define ARU_DIGITAL_OUT1             272
#define ARU_DIGITAL_OUT2             273
#define ARU_DIGITAL_OUT3             274
#define ARU_DIGITAL_OUT4             275
#define ARU_DIGITAL_OUT5             276
#define ARU_DIGITAL_OUT6             277
#define ARU_DIGITAL_OUT7             278
#define ARU_DIGITAL_OUT8             279
#define ARU_DIGITAL_OUT9             280
#define ARU_DIGITAL_OUT10            281
#define ARU_DIGITAL_OUT11            282
#define ARU_DIGITAL_OUT12            283

#define ARU_BYPASS_INIT_WRAP_TEST    320
#define ARU_MULTITHREAD_PROTECT      321
#define ARU_DISCRETE_RAW             322
#define ARU_FAST_SLEW_RATE           323

#define ARU_RX_TIMETAG_MODE          440
#define ARU_IRIG_WRAP_ENABLE         441           
#define ARU_IRIG_INPUT_THRESHOLD     442
#define ARU_IRIG_ADJUST_THRESHOLD    443
#define ARU_IRIG_QUICK_ADJUSTMENT    444
#define ARU_IRIG_AVAILABLE           445
#define ARU_IRIG_SET_BIAS            446
#define ARU_IRIG_CALIBRATED          447
#define ARU_CHAN_COUNT_429           448
#define ARU_CHAN_COUNT_573           449
#define ARU_CHAN_COUNT_DISC          450
#define ARU_CHAN_COUNT_DIFF          451
#define ARU_SERIAL_NUMBER            452
#define ARU_HW_FPGA_TEMPERATURE      453
#define ARU_HW_1PT0V_PWR_SUPPLY      454
#define ARU_HW_2PT5V_PWR_SUPPLY      455


                                     
/*---------------------------------------------------------------------------*
 *  This HUGE set of configuration constants are specifically used for the 
 *  "item" parameter with invocations to ar_set_config() to satisfy the board
 *  -level configuration usage with BusTools/ARINC and CEI-x20 interfaces.
 *---------------------------------------------------------------------------*/
#define ARU_XMIT_RATE                1 /* AR_SPEED_HIGH/_LOW for 100/12.5 Kb    */
#define ARU_RECV_RATE                2 /* AR_SPEED_HIGH/_LOW for 100/12.5 Kb    */
#define ARU_PARITY                   3 /* To assign all channels AR_PARITY_**** */
#define ARU_INTERNAL_WRAP            4 /* Internal wrap a single 429 channel    */
#define ARU_EXTERNAL_WRAP            5 /* External wrap a single 429 channel    */

#define ARU_RX_CH01_BIT_RATE         500 /* Configure receive ch one bit rate    */
#define ARU_RX_CH02_BIT_RATE         501 /* AR_HIGH or AR_LOW                    */
#define ARU_RX_CH03_BIT_RATE         502 /* AR_HIGH or AR_LOW select 100Kbaud    */
#define ARU_RX_CH04_BIT_RATE         503 /*  and 12.5 Kbaud respectively.        */
#define ARU_RX_CH05_BIT_RATE         504
#define ARU_RX_CH06_BIT_RATE         505
#define ARU_RX_CH07_BIT_RATE         506
#define ARU_RX_CH08_BIT_RATE         507
#define ARU_RX_CH09_BIT_RATE         508
#define ARU_RX_CH10_BIT_RATE         509
#define ARU_RX_CH11_BIT_RATE         510
#define ARU_RX_CH12_BIT_RATE         511
#define ARU_RX_CH13_BIT_RATE         512
#define ARU_RX_CH14_BIT_RATE         513
#define ARU_RX_CH15_BIT_RATE         514
#define ARU_RX_CH16_BIT_RATE         515
#define ARU_RX_CH17_BIT_RATE         516
#define ARU_RX_CH18_BIT_RATE         517
#define ARU_RX_CH19_BIT_RATE         518
#define ARU_RX_CH20_BIT_RATE         519
#define ARU_RX_CH21_BIT_RATE         520
#define ARU_RX_CH22_BIT_RATE         521
#define ARU_RX_CH23_BIT_RATE         522
#define ARU_RX_CH24_BIT_RATE         523
#define ARU_RX_CH25_BIT_RATE         524
#define ARU_RX_CH26_BIT_RATE         525
#define ARU_RX_CH27_BIT_RATE         526
#define ARU_RX_CH28_BIT_RATE         527
#define ARU_RX_CH29_BIT_RATE         528
#define ARU_RX_CH30_BIT_RATE         529
#define ARU_RX_CH31_BIT_RATE         530
#define ARU_RX_CH32_BIT_RATE         531

#define ARU_TX_CH01_BIT_RATE         700 /* Configure transmit ch one bit rate   */
#define ARU_TX_CH02_BIT_RATE         701 /* AR_HIGH or AR_LOW select 100Kbaud    */
#define ARU_TX_CH03_BIT_RATE         702 /*  and 12.5 Kbaud respectively.        */
#define ARU_TX_CH04_BIT_RATE         703
#define ARU_TX_CH05_BIT_RATE         704
#define ARU_TX_CH06_BIT_RATE         705
#define ARU_TX_CH07_BIT_RATE         706
#define ARU_TX_CH08_BIT_RATE         707
#define ARU_TX_CH09_BIT_RATE         708
#define ARU_TX_CH10_BIT_RATE         709
#define ARU_TX_CH11_BIT_RATE         710
#define ARU_TX_CH12_BIT_RATE         711
#define ARU_TX_CH13_BIT_RATE         712
#define ARU_TX_CH14_BIT_RATE         713
#define ARU_TX_CH15_BIT_RATE         714
#define ARU_TX_CH16_BIT_RATE         715
#define ARU_TX_CH17_BIT_RATE         716
#define ARU_TX_CH18_BIT_RATE         717
#define ARU_TX_CH19_BIT_RATE         718
#define ARU_TX_CH20_BIT_RATE         719
#define ARU_TX_CH21_BIT_RATE         720
#define ARU_TX_CH22_BIT_RATE         721
#define ARU_TX_CH23_BIT_RATE         722
#define ARU_TX_CH24_BIT_RATE         723
#define ARU_TX_CH25_BIT_RATE         724
#define ARU_TX_CH26_BIT_RATE         725
#define ARU_TX_CH27_BIT_RATE         726
#define ARU_TX_CH28_BIT_RATE         727
#define ARU_TX_CH29_BIT_RATE         728
#define ARU_TX_CH30_BIT_RATE         729
#define ARU_TX_CH31_BIT_RATE         730
#define ARU_TX_CH32_BIT_RATE         731
                                     
#define ARU_RX_CH01_PARITY           900 /* Configure receive ch one parity     */
#define ARU_RX_CH02_PARITY           901 /* Parity detection AR_ON or AR_OFF    */
#define ARU_RX_CH03_PARITY           902 /* Defaults to AR_ON, where parity     */
#define ARU_RX_CH04_PARITY           903 /*  detection is enabled.              */
#define ARU_RX_CH05_PARITY           904 /* If parity enabled, MSB of received  */
#define ARU_RX_CH06_PARITY           905 /*  word is 1 if odd parity detected,  */
#define ARU_RX_CH07_PARITY           906 /*  and 0 if even parity was detected. */
#define ARU_RX_CH08_PARITY           907 
#define ARU_RX_CH09_PARITY           908
#define ARU_RX_CH10_PARITY           909
#define ARU_RX_CH11_PARITY           910
#define ARU_RX_CH12_PARITY           911
#define ARU_RX_CH13_PARITY           912
#define ARU_RX_CH14_PARITY           913
#define ARU_RX_CH15_PARITY           914
#define ARU_RX_CH16_PARITY           915
#define ARU_RX_CH17_PARITY           916
#define ARU_RX_CH18_PARITY           917
#define ARU_RX_CH19_PARITY           918
#define ARU_RX_CH20_PARITY           919
#define ARU_RX_CH21_PARITY           920
#define ARU_RX_CH22_PARITY           921
#define ARU_RX_CH23_PARITY           922
#define ARU_RX_CH24_PARITY           923
#define ARU_RX_CH25_PARITY           924
#define ARU_RX_CH26_PARITY           925
#define ARU_RX_CH27_PARITY           926
#define ARU_RX_CH28_PARITY           927
#define ARU_RX_CH29_PARITY           928
#define ARU_RX_CH30_PARITY           929
#define ARU_RX_CH31_PARITY           930
#define ARU_RX_CH32_PARITY           931
                                     
#define ARU_TX_CH01_PARITY           1100 /* Configure transmit ch one parity   */
#define ARU_TX_CH02_PARITY           1101 /* ARU_PARITY_ODD, ARU_PARITY_EVEN or */
#define ARU_TX_CH03_PARITY           1102 /*   ARU_PARITY_NONE (or AR_OFF)      */
#define ARU_TX_CH04_PARITY           1103 /* Defaults to ARU_PARITY_ODD         */
#define ARU_TX_CH05_PARITY           1104 /* If parity enabled, MSB of transmit */
#define ARU_TX_CH06_PARITY           1105 /*  word is overwritten with the      */
#define ARU_TX_CH07_PARITY           1106 /*  parity bit value calculated by hw.*/
#define ARU_TX_CH08_PARITY           1107
#define ARU_TX_CH09_PARITY           1108
#define ARU_TX_CH10_PARITY           1109
#define ARU_TX_CH11_PARITY           1110
#define ARU_TX_CH12_PARITY           1111
#define ARU_TX_CH13_PARITY           1112
#define ARU_TX_CH14_PARITY           1113
#define ARU_TX_CH15_PARITY           1114
#define ARU_TX_CH16_PARITY           1115
#define ARU_TX_CH17_PARITY           1116
#define ARU_TX_CH18_PARITY           1117
#define ARU_TX_CH19_PARITY           1118
#define ARU_TX_CH20_PARITY           1119
#define ARU_TX_CH21_PARITY           1120
#define ARU_TX_CH22_PARITY           1121
#define ARU_TX_CH23_PARITY           1122
#define ARU_TX_CH24_PARITY           1123
#define ARU_TX_CH25_PARITY           1124
#define ARU_TX_CH26_PARITY           1125
#define ARU_TX_CH27_PARITY           1126
#define ARU_TX_CH28_PARITY           1127
#define ARU_TX_CH29_PARITY           1128
#define ARU_TX_CH30_PARITY           1129
#define ARU_TX_CH31_PARITY           1130
#define ARU_TX_CH32_PARITY           1131
                                 
#define ARU_RX_CH01_SDI_FILTER       1300  /* Enable SDI prefilter ch 1           */
#define ARU_RX_CH02_SDI_FILTER       1301  /* AR_ON or AR_OFF                     */
#define ARU_RX_CH03_SDI_FILTER       1302
#define ARU_RX_CH04_SDI_FILTER       1303
#define ARU_RX_CH05_SDI_FILTER       1304
#define ARU_RX_CH06_SDI_FILTER       1305
#define ARU_RX_CH07_SDI_FILTER       1306
#define ARU_RX_CH08_SDI_FILTER       1307
#define ARU_RX_CH09_SDI_FILTER       1308
#define ARU_RX_CH10_SDI_FILTER       1309
#define ARU_RX_CH11_SDI_FILTER       1310
#define ARU_RX_CH12_SDI_FILTER       1311
#define ARU_RX_CH13_SDI_FILTER       1312
#define ARU_RX_CH14_SDI_FILTER       1313
#define ARU_RX_CH15_SDI_FILTER       1314
#define ARU_RX_CH16_SDI_FILTER       1315
#define ARU_RX_CH17_SDI_FILTER       1316
#define ARU_RX_CH18_SDI_FILTER       1317
#define ARU_RX_CH19_SDI_FILTER       1318
#define ARU_RX_CH20_SDI_FILTER       1319
#define ARU_RX_CH21_SDI_FILTER       1320
#define ARU_RX_CH22_SDI_FILTER       1321
#define ARU_RX_CH23_SDI_FILTER       1322
#define ARU_RX_CH24_SDI_FILTER       1323
#define ARU_RX_CH25_SDI_FILTER       1324
#define ARU_RX_CH26_SDI_FILTER       1325
#define ARU_RX_CH27_SDI_FILTER       1326
#define ARU_RX_CH28_SDI_FILTER       1327
#define ARU_RX_CH29_SDI_FILTER       1328
#define ARU_RX_CH30_SDI_FILTER       1329
#define ARU_RX_CH31_SDI_FILTER       1330
#define ARU_RX_CH32_SDI_FILTER       1331
                                 
#define ARU_RX_CH01_SDI_VALUE        1500  /* SDI prefilter value for ch 1 (0-3)  */
#define ARU_RX_CH02_SDI_VALUE        1501
#define ARU_RX_CH03_SDI_VALUE        1502
#define ARU_RX_CH04_SDI_VALUE        1503
#define ARU_RX_CH05_SDI_VALUE        1504
#define ARU_RX_CH06_SDI_VALUE        1505
#define ARU_RX_CH07_SDI_VALUE        1506
#define ARU_RX_CH08_SDI_VALUE        1507
#define ARU_RX_CH09_SDI_VALUE        1508
#define ARU_RX_CH10_SDI_VALUE        1509
#define ARU_RX_CH11_SDI_VALUE        1510
#define ARU_RX_CH12_SDI_VALUE        1511
#define ARU_RX_CH13_SDI_VALUE        1512
#define ARU_RX_CH14_SDI_VALUE        1513
#define ARU_RX_CH15_SDI_VALUE        1514
#define ARU_RX_CH16_SDI_VALUE        1515
#define ARU_RX_CH17_SDI_VALUE        1516
#define ARU_RX_CH18_SDI_VALUE        1517
#define ARU_RX_CH19_SDI_VALUE        1518
#define ARU_RX_CH20_SDI_VALUE        1519
#define ARU_RX_CH21_SDI_VALUE        1520
#define ARU_RX_CH22_SDI_VALUE        1521
#define ARU_RX_CH23_SDI_VALUE        1522
#define ARU_RX_CH24_SDI_VALUE        1523
#define ARU_RX_CH25_SDI_VALUE        1524
#define ARU_RX_CH26_SDI_VALUE        1525
#define ARU_RX_CH27_SDI_VALUE        1526
#define ARU_RX_CH28_SDI_VALUE        1527
#define ARU_RX_CH29_SDI_VALUE        1528
#define ARU_RX_CH30_SDI_VALUE        1529
#define ARU_RX_CH31_SDI_VALUE        1530
#define ARU_RX_CH32_SDI_VALUE        1531
                                          /* DAC values to read or write          */
#define ARU_RX1_4_THRESH_VALUE       160  /* Thresh = 10 * (2.5 - Vdac)           */
#define ARU_RX5_8_THRESH_VALUE       161  /* Vdac = 5 * n / 256 Volts             */
#define ARU_RX9_10_THRESH_VALUE      162  /* Thresh = 0x72 = 2.73 Volts default   */
#define ARU_RX11_12_THRESH_VALUE     163
#define ARU_INPUT_THRESH_VALUE       164  /* Thresh = 20 * n / 256 Volts = 1.56V  */
#define ARU_TRANSMIT_VOLT_VALUE      165  /* Level  = 10 * n / 256 Volts = 9.96V  */
                                     
#define ARU_PARAMETERIC_SUPPORT      170  /* H/W supports variable parametrics?   */
                                          /* This includes Rcvr Thresh Adj, Xmt   */
                                          /*  Output Adj and Extra/Too Few bits   */
                                          /* Returns AR_ON or AR_OFF              */
                                     
#define ARU_DISCRETE_VALUES          175  /* Read/Write Discrete Inputs/Outputs   */
                                          /* Discretes are active high.           */
#define ARU_DISCRETE_OUTPUTS         176  /* Read/Write discrete output values    */
#define ARU_DISCRETE_INPUTS          177  /* Read discrete input values           */

#define ARU_TX_CH01_SHUT_OFF         1700  /* Shut off the ARINC transmitter       */
#define ARU_TX_CH02_SHUT_OFF         1701  /* AR_ON or AR_OFF                      */
#define ARU_TX_CH03_SHUT_OFF         1702  /* AR_ON turns the transmitter OFF      */
#define ARU_TX_CH04_SHUT_OFF         1703
#define ARU_TX_CH05_SHUT_OFF         1704
#define ARU_TX_CH06_SHUT_OFF         1705
#define ARU_TX_CH07_SHUT_OFF         1706
#define ARU_TX_CH08_SHUT_OFF         1707
#define ARU_TX_CH09_SHUT_OFF         1708
#define ARU_TX_CH10_SHUT_OFF         1709
#define ARU_TX_CH11_SHUT_OFF         1710
#define ARU_TX_CH12_SHUT_OFF         1711
#define ARU_TX_CH13_SHUT_OFF         1712
#define ARU_TX_CH14_SHUT_OFF         1713
#define ARU_TX_CH15_SHUT_OFF         1714
#define ARU_TX_CH16_SHUT_OFF         1715
#define ARU_TX_CH17_SHUT_OFF         1716  
#define ARU_TX_CH18_SHUT_OFF         1717  
#define ARU_TX_CH19_SHUT_OFF         1718  
#define ARU_TX_CH20_SHUT_OFF         1719
#define ARU_TX_CH21_SHUT_OFF         1720
#define ARU_TX_CH22_SHUT_OFF         1721
#define ARU_TX_CH23_SHUT_OFF         1722
#define ARU_TX_CH24_SHUT_OFF         1723
#define ARU_TX_CH25_SHUT_OFF         1724
#define ARU_TX_CH26_SHUT_OFF         1725
#define ARU_TX_CH27_SHUT_OFF         1726
#define ARU_TX_CH28_SHUT_OFF         1727
#define ARU_TX_CH29_SHUT_OFF         1728
#define ARU_TX_CH30_SHUT_OFF         1729
#define ARU_TX_CH31_SHUT_OFF         1730
#define ARU_TX_CH32_SHUT_OFF         1731

#define ARU_TX_CH01_HB_INJ           3300  /* Enable Extra Bit error injection     */
#define ARU_TX_CH02_HB_INJ           3301  /* AR_ON or AR_OFF                      */
#define ARU_TX_CH03_HB_INJ           3302  /* This function when enabled increases */
#define ARU_TX_CH04_HB_INJ           3303  /*  the normal number of bits send per  */
#define ARU_TX_CH05_HB_INJ           3304  /*  label from 32 to 33.                */
#define ARU_TX_CH06_HB_INJ           3305
#define ARU_TX_CH07_HB_INJ           3306
#define ARU_TX_CH08_HB_INJ           3307
#define ARU_TX_CH09_HB_INJ           3308
#define ARU_TX_CH10_HB_INJ           3309
#define ARU_TX_CH11_HB_INJ           3310
#define ARU_TX_CH12_HB_INJ           3311
#define ARU_TX_CH13_HB_INJ           3312
#define ARU_TX_CH14_HB_INJ           3313
#define ARU_TX_CH15_HB_INJ           3314
#define ARU_TX_CH16_HB_INJ           3315
#define ARU_TX_CH17_HB_INJ           3316
#define ARU_TX_CH18_HB_INJ           3317
#define ARU_TX_CH19_HB_INJ           3318
#define ARU_TX_CH20_HB_INJ           3319
#define ARU_TX_CH21_HB_INJ           3320
#define ARU_TX_CH22_HB_INJ           3321
#define ARU_TX_CH23_HB_INJ           3322
#define ARU_TX_CH24_HB_INJ           3323
#define ARU_TX_CH25_HB_INJ           3324
#define ARU_TX_CH26_HB_INJ           3325
#define ARU_TX_CH27_HB_INJ           3326
#define ARU_TX_CH28_HB_INJ           3327
#define ARU_TX_CH29_HB_INJ           3328
#define ARU_TX_CH30_HB_INJ           3329
#define ARU_TX_CH31_HB_INJ           3330
#define ARU_TX_CH32_HB_INJ           3331

#define ARU_TX_CH01_LB_INJ           3500  /* Enable Too Few Bits error injection  */
#define ARU_TX_CH02_LB_INJ           3501  /* AR_ON or AR_OFF                      */
#define ARU_TX_CH03_LB_INJ           3502  /* This function when enabled reduces   */
#define ARU_TX_CH04_LB_INJ           3503  /*  the normal number of bits send per  */
#define ARU_TX_CH05_LB_INJ           3504  /*  label from 32 to 31.                */
#define ARU_TX_CH06_LB_INJ           3505
#define ARU_TX_CH07_LB_INJ           3506
#define ARU_TX_CH08_LB_INJ           3507
#define ARU_TX_CH09_LB_INJ           3508
#define ARU_TX_CH10_LB_INJ           3509
#define ARU_TX_CH11_LB_INJ           3510
#define ARU_TX_CH12_LB_INJ           3511
#define ARU_TX_CH13_LB_INJ           3512
#define ARU_TX_CH14_LB_INJ           3513
#define ARU_TX_CH15_LB_INJ           3514
#define ARU_TX_CH16_LB_INJ           3515
#define ARU_TX_CH17_LB_INJ           3516
#define ARU_TX_CH18_LB_INJ           3517
#define ARU_TX_CH19_LB_INJ           3518
#define ARU_TX_CH20_LB_INJ           3519
#define ARU_TX_CH21_LB_INJ           3520
#define ARU_TX_CH22_LB_INJ           3521
#define ARU_TX_CH23_LB_INJ           3522
#define ARU_TX_CH24_LB_INJ           3523
#define ARU_TX_CH25_LB_INJ           3524
#define ARU_TX_CH26_LB_INJ           3525
#define ARU_TX_CH27_LB_INJ           3526
#define ARU_TX_CH28_LB_INJ           3527
#define ARU_TX_CH29_LB_INJ           3528
#define ARU_TX_CH30_LB_INJ           3529
#define ARU_TX_CH31_LB_INJ           3530
#define ARU_TX_CH32_LB_INJ           3531
                                     
#define ARU_TX_CH01_GAP_INJ          3700  /* Enable Short Gap error injection     */
#define ARU_TX_CH02_GAP_INJ          3701  /* AR_ON or AR_OFF                      */
#define ARU_TX_CH03_GAP_INJ          3702  /* This function when enabled reduces   */
#define ARU_TX_CH04_GAP_INJ          3703  /*  the normal 4 bit time gap between   */
#define ARU_TX_CH05_GAP_INJ          3704  /*  labels to 2 bit times.              */
#define ARU_TX_CH06_GAP_INJ          3705
#define ARU_TX_CH07_GAP_INJ          3706
#define ARU_TX_CH08_GAP_INJ          3707
#define ARU_TX_CH09_GAP_INJ          3708
#define ARU_TX_CH10_GAP_INJ          3709
#define ARU_TX_CH11_GAP_INJ          3710
#define ARU_TX_CH12_GAP_INJ          3711
#define ARU_TX_CH13_GAP_INJ          3712
#define ARU_TX_CH14_GAP_INJ          3713
#define ARU_TX_CH15_GAP_INJ          3714
#define ARU_TX_CH16_GAP_INJ          3715
#define ARU_TX_CH17_GAP_INJ          3716
#define ARU_TX_CH18_GAP_INJ          3717
#define ARU_TX_CH19_GAP_INJ          3718
#define ARU_TX_CH20_GAP_INJ          3719
#define ARU_TX_CH21_GAP_INJ          3720
#define ARU_TX_CH22_GAP_INJ          3721
#define ARU_TX_CH23_GAP_INJ          3722
#define ARU_TX_CH24_GAP_INJ          3723
#define ARU_TX_CH25_GAP_INJ          3724
#define ARU_TX_CH26_GAP_INJ          3725
#define ARU_TX_CH27_GAP_INJ          3726
#define ARU_TX_CH28_GAP_INJ          3727
#define ARU_TX_CH29_GAP_INJ          3728
#define ARU_TX_CH30_GAP_INJ          3729
#define ARU_TX_CH31_GAP_INJ          3730
#define ARU_TX_CH32_GAP_INJ          3731

/*---------------------------------------------------------------------------*
 *  This set of configuration constants are specifically used for the 
 *  "item" parameter with invocations to ar_set_config() to satisfy the 
 *  legacy BusTools/ARINC & CEI-x20 interface.
 *---------------------------------------------------------------------------*/
#define ARU_OLD_RX_CH01_BIT_RATE     40  /* Configure receiver bit rate      */
#define ARU_OLD_RX_CH16_BIT_RATE     55
#define ARU_OLD_TX_CH01_BIT_RATE     60  /* Configure transmitter bit rate   */
#define ARU_OLD_TX_CH16_BIT_RATE     75
#define ARU_OLD_RX_CH01_PARITY       80  /* Configure receiver parity        */
#define ARU_OLD_RX_CH16_PARITY       95
#define ARU_OLD_TX_CH01_PARITY       100 /* Configure transmitter parity     */
#define ARU_OLD_TX_CH16_PARITY       115
#define ARU_OLD_RX_CH01_SDI_FILTER   120 /* Enable SDI prefilter             */
#define ARU_OLD_RX_CH16_SDI_FILTER   135
#define ARU_OLD_RX_CH01_SDI_VALUE    140 /* SDI prefilter value              */ 
#define ARU_OLD_RX_CH16_SDI_VALUE    155
#define ARU_OLD_TX_CH01_HB_INJ       180 /* Enable Extra Bit error inj       */
#define ARU_OLD_TX_CH16_HB_INJ       195
#define ARU_OLD_TX_CH01_LB_INJ       200 /* Enable Too Few Bits error inj    */ 
#define ARU_OLD_TX_CH16_LB_INJ       215
#define ARU_OLD_TX_CH01_GAP_INJ      220 /* Enable Short Gap error inj       */
#define ARU_OLD_TX_CH16_GAP_INJ      235
#define ARU_OLD_TX_CH01_SHUT_OFF     240 /* Shut off the ARINC transmitter   */
#define ARU_OLD_TX_CH16_SHUT_OFF     255


/* ARINC 717/573-specific Item parameters for Set/Get 573 Config */
#define ARU_573_RX_ENABLE            300
#define ARU_573_RX_AUTO_DETECT       301
#define ARU_573_RX_BPRZ_SELECT       302
#define ARU_573_RX_SUBFRAME_SIZE     304
#define ARU_573_TX_SLEW_RATE         305
#define ARU_573_SYNC_WORD1           307
#define ARU_573_SYNC_WORD2           308
#define ARU_573_SYNC_WORD3           309
#define ARU_573_SYNC_WORD4           310
#define ARU_573_TX_BPRZ_SELECT       313
#define ARU_573_TX_HBP_SELECT        314

/* ARINC 573 specific value parameters for Set/Get Config Items */
#define ARU_573_RATE_SIZE_384_32      0
#define ARU_573_RATE_SIZE_768_64      1   
#define ARU_573_RATE_SIZE_1536_128    2    
#define ARU_573_RATE_SIZE_3072_256    3    
#define ARU_573_RATE_SIZE_6144_512    4    
#define ARU_573_RATE_SIZE_12288_1024  5     
#define ARU_573_RATE_SIZE_24576_2048  6
#define ARU_573_RATE_SIZE_49152_4096  7     
#define A717_NUM_SUPPORTED_BUS_SPEEDS 8

#define ARU_573_TX_SLEW_10PT0        0
#define ARU_573_TX_SLEW_1PT5         1

/* These ARINC 573 items are only supported for backward compatibility
 * with the CEI-710 API, as their is no seperate subframe size attribute
 * on this device
 */
#define ARU_573_SPEED_384            384
#define ARU_573_SPEED_768            768
#define ARU_573_SPEED_1536           1536
#define ARU_573_SPEED_3072           3072
#define ARU_573_SPEED_6144           6144
#define ARU_573_SPEED_12288          12288
#define ARU_573_SPEED_24576          24576
#define DEFAULT_BUS_SPEED            ARU_573_SPEED_768

#define ARU_573_SUB_FRAME_32         32
#define ARU_573_SUB_FRAME_64         64
#define ARU_573_SUB_FRAME_128        128
#define ARU_573_SUB_FRAME_256        256
#define ARU_573_SUB_FRAME_512        512
#define ARU_573_SUB_FRAME_1024       1024
#define ARU_573_SUB_FRAME_2048       2048
#define ARU_573_SUB_FRAME_4096       4096

#define DEFAULT_FRAME_SIZE           ARU_573_SUB_FRAME_64
#define ARU_717_AUTOSYNC             0x0000   /* Autosync mode                     */
#define ARU_717_RAW                  0x2000   /* RAW mode                          */
#define ARU_717_H_BI_PHASE           0x2000   /* Harvard Bi-Phase Encoding (HBP)   */
#define ARU_717_BIPOLAR              0x1000   /* Bipolar R-T-Z Encoding    (BPRZ)  */

/*------------------------------------------------------------------------* 
 * General Function Constants used for various things
 *------------------------------------------------------------------------*/
#define EXIT_KEY                   27    /* ASCII <ESC> key for exit            */

#ifndef TRUE
#define TRUE                       1
#define FALSE                      0
#endif

#define ARU_ENABLE_TIMETAG         1     /* Enable receiver time tagging         */
#define ARU_DISABLE_TIMETAG        0     /* Disable receiver time tagging        */
#define ARU_ENABLE_INT             1     /* Enable interrupts to the host        */
#define ARU_DISABLE_INT            2     /* Disable interrupts to the host       */
#define ARU_RECEIVER               0     /* Apply parameter to receiver          */
#define ARU_TRANSMITTER            1     /* Apply parameter to transmitter       */
#define ARU_SCHED_MSG_INFINITE     CEIDEV_MSG_SCHEDULER_TX_ALWAYS
#define CHAN_NOT_IMPLEMENTED       0x00  /* Channel is not implemented           */
#define CHAN_ARINC429              0x01  /* Channel is an ARINC-429 channel      */
#define CHAN_ARINC717              0x02  /* Channel is an ARINC-573/717 channel  */
#define ARU_PARAMETRIC_SUPPORT     170  /* H/W supports variable parametrics?   */

#define INTERNAL                   0
#define EXTERNAL                   1
#define EXIT_ROUTINE               9999

/* Time-tag source selection */
#define AR_TIMER_REGISTER_SET          0
#define AR_RECEIVER_FIFO_TIMETAG       1
#define AR_BYPASS_TIMER_READ           2

/* Time-tag format selection */
#define AR_TIMETAG_EXT_IRIG_64BIT      0
#define AR_TIMETAG_INT_USEC_64BIT      1
#define AR_TIMETAG_HOST_USEC_64BIT     2
#define AR_TIMETAG_INT_20USEC_32BIT    3
#define AR_TIMETAG_INT_MSEC_32BIT      4
#define AR_TIMETAG_EXT_IRIG_REF_64BIT  5
#define AR_TIMER_X20_COMPAT_32BIT      6 
#define AR_TIMETAG_LAST_OPTION         (CEI_INT32)(AR_TIMER_X20_COMPAT_32BIT+1)
/* The next format selection is valid only for LV-x30 support (LabVIEW R/T OS) */
#define AR_TIMETAG_HOST_SEC_32BIT      7
#define AR_TIMETAG_REF_IRIG_64BIT      8
#define AR_LOG_EXT_IRIG_SAMPLE         9
#define AR_TIMETAG_EXT_IRIG_RAW        10
#define AR_DEFAULT_RECEIVE_DATA_TIMETAG_FORMAT   AR_TIMETAG_INT_MSEC_32BIT

#if defined (_LINUX_X86_) || defined (_LINUX_PPC_)
   #define AR_MAX_TIMETAG_VALUE           (TIME_TAG_TYPE)0xFFFFFFFFFFFFFFFFULL 
   #define AR_X20_MAX_TIMETAG_VALUE       (TIME_TAG_TYPE)0x3FFFFFFFFFFFFFFFULL
   #define AR_IRIG_ROLLOVER_NON_LEAP_YEAR (TIME_TAG_TYPE)(31536000000000ULL)
   #define AR_IRIG_ROLLOVER_LEAP_YEAR     (TIME_TAG_TYPE)(31622400000000ULL)
#else
   #define AR_MAX_TIMETAG_VALUE           (TIME_TAG_TYPE)0xFFFFFFFFFFFFFFFF
   #define AR_X20_MAX_TIMETAG_VALUE       (TIME_TAG_TYPE)0x3FFFFFFFFFFFFFFF
   #define AR_IRIG_ROLLOVER_NON_LEAP_YEAR (TIME_TAG_TYPE)31536000000000
   #define AR_IRIG_ROLLOVER_LEAP_YEAR     (TIME_TAG_TYPE)31622400000000
#endif

#define AR_IRIG_TIMETAG_BIAS           (TIME_TAG_TYPE)1010000
#define AR_X20_SCALE_TIMETAG_VALUE     (TIME_TAG_TYPE)4
#define AR_X20_QTR_MICROSEC_PER_MSEC   (CEI_UINT32)4000
#define AR_MICROSECONDS_PER_SECOND     (TIME_TAG_TYPE)(1000000)
#define AR_MICROSEC_PER_MILLISEC       (TIME_TAG_TYPE)(1000)
#define AR_SECONDS_PER_DAY             86400
#define AR_LOCAL_TIME_YEAR_BASIS       1900
#define AR_LEAP_YEAR_DIVISOR           4
#define AR_LEAP_YEAR_EXCEPTION         2100


/* Time-tag string conversion format selection and array size */
#define AR_TD_DATE                 0
#define AR_TD_IRIG                 1
#define AR_TD_REL_MIDNIGHT         2
#define AR_TD_STRING_SIZE          24

/* BIT execution selection */
#define AR_BIT_BASIC_STARTUP       0
#define AR_BIT_FULL_STARTUP        1
#define AR_BIT_PERIODIC            2
#define AR_BIT_INT_LOOPBACK        3
#define AR_BIT_EXT_LOOPBACK        4
#define AR_BIT_PARTIAL_SRAM        8
#define AR_BIT_FULL_SRAM           9
#define AR_BIT_SELECT_SRAM_MIN     100
#define AR_BIT_SELECT_SRAM_MAX     1123

/* the following constants are used with the function ceidev_memory_test() */
#define FULL_SRAM                  0
#define UNUSED_SRAM                1
#define SELECT_SRAM                2

/* the following constants are used with the function ar_board_test() */
#define EXTERNAL_WRAP              0
#define INTERNAL_WRAP              1

#define BAUD_100_TIMETAG_DEV       18
#define BAUD_125_TIMETAG_DEV       144
#define BAUD_11_TIMETAG_DEV        150

#define MAX_OUTPUT_DISCRETES       4
#define MAX_DIGITAL_OUTPUT         12
#define MAX_DIGITAL_INPUT          4

/* the following constants are used with the function ar_set_storage_mode() */
#define ARU_BUFFERED               0
#define ARU_DEDICATED              1
#define ARU_MERGED                 2

/* the following constants are used with the function ar_set_transmitter_mode() */
#define ARU_XMIT_573_HBP           1
#define ARU_XMIT_573_BPRZ          2

/* channel set/get config internal validation definitions */
#define ARU_GET_DATA               34
#define ARU_PUT_DATA               35
#define ARU_FILTER_LABEL           36
#define ARU_ENHANCED_FILTER_LABEL  37
#define ARU_ADD_SCHEDULED_MSG      39

/* the following constants are used with the function ar_label_filter()      */
#define ARU_ALL_LABELS             0x1FF /* Enable/disable ALL label filtering  */
#define ARU_ALL_SDI                4     /* Enable/disable all sdi filtering for this label */
#define ARU_ALL_ESSM               8     /* Enable/disable all ssm filtering for this label */
#define ARU_FILTER_OFF             0     /* Filter off--pass specified label(s) */
#define ARU_FILTER_ON              1     /* Filter on--delete specified label(s)*/
#define FILTER_SEQUENTIAL          0x10
#define FILTER_SNAPSHOT            0x20
#define FILTER_INTERRUPT           0x40
#define ARINC_LABEL_SDI_MASK       0x000003ff
#define ARINC_LABEL_MASK           0x000000ff
#define ARINC_SDI_MASK             0x00000300

#define PARITY_BIT                 0x80000000
#define DATA_BITS                  0x7FFFFFFF
#define MERGED_CHANNEL_MASK        (TIME_TAG_TYPE)0x7FFFFFF
#define MERGED_CHANNEL_SHIFT       27
#define A573_AUTOSYNC_TIMEOUT      1000
#define A573_RAW_TIMEOUT           2


/*---------------------------------------------------------------------------*
 * Message Scheduler API routine data structure for the definition and
 * parameter usage of CEI-x30 periodic message scheduling data
 *---------------------------------------------------------------------------*/
typedef struct MESSAGE_SCHEDULER_ENTRY {
   CEI_UINT32 messageIndex;
   CEI_UINT32 board; 
   CEI_UINT32 channel;
   CEI_UINT32 rate;
   CEI_UINT32 start;
   CEI_UINT32 txCount;
   CEI_UINT32 data;
} AR_SCHEDULED_MSG_ENTRY_TYPE, *pAR_SCHEDULED_MSG_ENTRY_TYPE;

typedef struct LV_MESSAGE_SCHEDULER_ENTRY {
  CEI_UINT32 array_size;
  AR_SCHEDULED_MSG_ENTRY_TYPE msgEntries[CEIDEV_CHAN_MSG_SCH_ENTRY_MAX];
} LV_AR_SCHEDULED_MSG_ENTRY_TYPE, *pLV_AR_SCHEDULED_MSG_ENTRY_TYPE;

/*---------------------------------------------------------------------------*
 * Time-tag definition structure for any routine passing a 64-bit time-tag
 * parameter supporting the IRIG time-reference (routines with an _xt suffix)
 * LabVIEW-RT support doesn't provide 64-bit integer capability, so limit the
 * time-tag support to 32-bit integer
 *---------------------------------------------------------------------------*/
typedef struct TIMETAG_PARAMETER_STRUCT {
   TIME_TAG_TYPE timeTagFormat;
   TIME_TAG_TYPE timeTag;
   TIME_TAG_TYPE referenceTimeTag;
} AR_TIMETAG_TYPE, *pAR_TIMETAG_TYPE;

/*---------------------------------------------------------------------------*
 * The following definitions and structure were added to support compilation 
 * for legacy CEI-520/620/820 applications.
 * This structure is not used in any way, shape, or form by the CEI-830 API.
 *---------------------------------------------------------------------------*/
#ifndef UTILDEFS_H
#define IS_6WIRE              0x1000 /* CEI-x20 board with 6-wire support    */
#define IS_717                0x2000 /* CEI-x20 board with -717/573 support  */
#define IS_CSDB               0x4000 /* CEI-x20 board with CSDB support      */
typedef struct ar_channel_parms 
{
   CEI_INT32  BaudRate;     
   CEI_INT32  ChannelType;  
   CEI_INT32  InternalWrap; 
   CEI_INT32  IntCount;     
   CEI_INT32  Parity;       
   CEI_INT32  AutoSync;     
   CEI_INT32  HWEncoding;   
   CEI_INT32  Sync1;        
   CEI_INT32  Sync2;        
   CEI_INT32  Sync3;        
   CEI_INT32  Sync4;        
   CEI_INT32  NumSubFrames; 
   CEI_INT32  OperateMode;  
   CEI_INT32  TimeTagMode;  
   CEI_UINT32 Parametrics; 
   CEI_INT32  Reserved2;
} AR_CHANNEL_PARMS, *pAR_CHANNEL_PARMS;
#endif

/*===========================================================================*
 *  F U N C T I O N   P R O T O T Y P E S 
 *===========================================================================*/
 
#ifdef __cplusplus 

   /*------------------------------------------------------------------------*  
    *  handle .cpp files
    *------------------------------------------------------------------------*/

   extern "C" {
#endif

#if defined (_Windows)                 
  #if !defined (_WIN32)
    #define DLL_EXPORTED FAR PASCAL EXPORT 
  #else
    #define DLL_EXPORTED _stdcall           
  #endif
#else  
  #define DLL_EXPORTED
  #define EXPORT32
#endif

      
/*===========================================================================*
 *     A P I     E X T E R N A L   F U N C T I O N   P R O T O T Y P E S 
 *===========================================================================*/
                     
EXPORT32 CEI_INT16 DLL_EXPORTED ar_board_test 
(
   CEI_INT16 board, 
   CEI_INT16 testType
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_bypass_wrap_test
(
   CEI_INT16 board, 
   CEI_INT16 bypass
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_close
(
   CEI_INT16 board
);

EXPORT32 CEI_VOID DLL_EXPORTED ar_convert_time_to_string 
( 
   CEI_INT16 board,
   CEI_INT16 displayFormat,
   pAR_TIMETAG_TYPE timeTagIn,
   pCEI_CHAR timeString 
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_define_msg 
(
   CEI_INT16 board, 
   CEI_INT16 channel,
   CEI_INT16 rate, 
   CEI_UINT16 start,
   CEI_INT32 data
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_define_msg_block
(
   CEI_INT32 numberOfEntries, 
   pAR_SCHEDULED_MSG_ENTRY_TYPE messageEntry
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_define_msg_block_lv
(
   CEI_INT32 numberOfEntries, 
   pLV_AR_SCHEDULED_MSG_ENTRY_TYPE messageEntry
);

EXPORT32 CEI_INT32 DLL_EXPORTED ar_enh_label_filter 
(
   CEI_INT16 board, 
   CEI_INT16 channel, 
   CEI_UINT16 label, 
   CEI_UINT16 sdi, 
   CEI_UINT16 essm, 
   CEI_INT16 action
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_execute_bit 
(
   CEI_INT16 board, 
   CEI_INT16 testType
);

EXPORT32 pCEI_UINT32 DLL_EXPORTED ar_get_base_addr
(
   CEI_INT16 board
);

EXPORT32 CEI_INT32 DLL_EXPORTED ar_getblock
(
   CEI_UINT32 board,
   CEI_UINT32 channel,
   CEI_INT32  maxMessages,
   CEI_INT32  offset,
   pCEI_INT32 actualCount,
   pCEI_INT32 data,
   pCEI_INT32 timeTags
);

EXPORT32 CEI_INT32 DLL_EXPORTED ar_getblock_t
(
   CEI_UINT32  board,
   CEI_UINT32  channel,
   CEI_INT32   maxMessages,
   pCEI_INT32  actualCount,
   pCEI_UINT32 msgChan,
   pCEI_INT32  data,
   pCEI_INT32  timeTagMsw,
   pCEI_INT32  timeTagLsw
);

EXPORT32 CEI_INT32 DLL_EXPORTED ar_get_boardnameLV 
(
   CEI_UINT32 board, 
   pCEI_CHAR boardName
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_get_boardtype
(
   CEI_INT16 board
);

EXPORT32 CEI_INT32 DLL_EXPORTED ar_get_config
(
   CEI_INT16 board, CEI_INT16 item 
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_get_data
(
   CEI_INT16 board,
#if defined (LABVIEW_RT)
   CEI_INT16 channel, 
#else   
   pCEI_INT16 channel, 
#endif
   pCEI_UINT32 data, 
   pCEI_UINT32 timeTagLo,
   pCEI_UINT32 timeTagHi 
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_get_data_xt
(
   CEI_INT16 board,
   pCEI_INT16 channel, 
   pCEI_UINT32 data, 
   pAR_TIMETAG_TYPE timeTagRef
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_get_device_config
(
   CEI_INT16 board,
   CEI_INT16 channel, 
   CEI_INT16 item, 
   pCEI_INT16 value 
);


#ifdef NIVISA_PXI
EXPORT32 void DLL_EXPORTED ar_get_error
(
   CEI_INT16 error_num,
   pCEI_CHAR errorStringOut
);
#else
EXPORT32 pCEI_CHAR DLL_EXPORTED ar_get_error
(
   CEI_INT16 error_num
);
#endif

EXPORT32 CEI_INT16 DLL_EXPORTED ar_get_429_message 
(
   CEI_INT16 board,
   CEI_INT16 channel,
   CEI_INT16 waitState, 
   pCEI_VOID data,
   pCEI_VOID timeTag 
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_get_573_config
(
   CEI_INT16 board,  
   CEI_INT16 item,   
   pCEI_INT32 value 
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_get_573_frame
(
   CEI_INT16 board,
   pCEI_UINT32 numberWords,
   pCEI_UINT16 destination 
);

EXPORT32 CEI_INT32 DLL_EXPORTED ar_getfilter
(
   CEI_UINT32 board,
   CEI_UINT32 channel, 
   pCEI_CHAR filterTable
);
   
EXPORT32 CEI_VOID DLL_EXPORTED ar_get_latest 
(
   CEI_INT16 board, 
   CEI_INT16 channel,
   CEI_UINT16 label, 
   pCEI_VOID data,
   pCEI_CHAR seq_num
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_getnext
(
   CEI_INT16 board, 
   CEI_INT16 channel, 
   pCEI_VOID destination 
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_getnextt
(
   CEI_INT16 board,
   CEI_INT16 channel, 
   pCEI_VOID destination, 
   pCEI_VOID timetag 
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_getnext_xt
(
   CEI_INT16 board,
   CEI_INT16 channel, 
   pCEI_UINT32 data, 
   pAR_TIMETAG_TYPE timeTagRef
);

EXPORT32 CEI_INT32 DLL_EXPORTED ar_get_snap_data 
(
   CEI_INT16 board, 
   CEI_INT16 channel,
   CEI_UINT16 label, 
   CEI_UINT16 sdi, 
   pCEI_INT32 data
);

EXPORT32 CEI_INT32 DLL_EXPORTED ar_get_latest_t 
(
   CEI_INT16   board,       
   CEI_INT16   channel,     
   CEI_UINT16  label,      
   pCEI_UINT32 data,        
   TIME_TAG_TYPE * timeTag 
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_get_status
(
   CEI_INT16 board, 
   pCEI_INT16 state 
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_get_storage_mode
(
   CEI_INT16 board, 
   pCEI_INT16 mode
); 

EXPORT32 CEI_INT16 DLL_EXPORTED ar_get_time 
(
   CEI_INT16 board,
   CEI_INT16 format,
   pAR_TIMETAG_TYPE timeTag
);

EXPORT32 CEI_UINT32 DLL_EXPORTED ar_get_timercntl
(
   CEI_INT16 board
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_get_transmitter_mode
(
   CEI_INT16 board, 
   pCEI_INT16 mode
); 

#ifndef UTILDEFS_H
EXPORT32 CEI_INT16 DLL_EXPORTED ar_getword
(
   CEI_INT16 board,
   CEI_INT16 channel, 
   pCEI_VOID destination
);
#endif

EXPORT32 CEI_INT16 DLL_EXPORTED ar_getwordt
(
   CEI_INT16 board,
   CEI_INT16 channel, 
   pCEI_VOID destination, 
   pCEI_VOID timetag
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_getword_xt
(
   CEI_INT16 board,
   CEI_INT16 channel, 
   pCEI_UINT32 data, 
   pAR_TIMETAG_TYPE timeTagRef
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_go
(
   CEI_INT16 board
);
      
EXPORT32 CEI_INT16 DLL_EXPORTED ar_initialize_api
(
   CEI_INT16 board
); 

EXPORT32 CEI_INT16 DLL_EXPORTED ar_initialize_device 
(
   CEI_INT16 board
);

EXPORT32 CEI_INT32 DLL_EXPORTED ar_hw_interrupt_buffer_read 
(
   CEI_INT32   board,
   pCEI_UINT32 numberOfWords,
   pCEI_UINT32 data 
);

EXPORT32 CEI_INT32 DLL_EXPORTED ar_interrupt_queue_read 
(
   CEI_INT32   board,
   pCEI_UINT32 numberOfWords,
   pCEI_UINT32 data 
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_label_filter
(
   CEI_INT16 board, 
   CEI_INT16 channel,
   CEI_UINT16 label,
   CEI_INT16 action 
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_loadslv
(
   CEI_INT16 board,
   CEI_UINT32 base_seg, 
   CEI_INT32 base_port, 
   CEI_UINT16 ram_size
);
 
EXPORT32 CEI_INT16 DLL_EXPORTED ar_modify_msg 
(
   CEI_INT16 board, 
   CEI_INT16 channel,
   CEI_INT16 msgNumber,
   CEI_INT16 rate, 
   CEI_INT32 data
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_modify_msg_block 
(
   CEI_INT32 numberOfEntries,
   pAR_SCHEDULED_MSG_ENTRY_TYPE messageEntry
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_num_rchans
(
   CEI_INT16 board
);
                
EXPORT32 CEI_INT16 DLL_EXPORTED ar_num_xchans
(
   CEI_INT16 board
);  

EXPORT32 CEI_INT32 DLL_EXPORTED ar_putblock
(
   CEI_UINT32 board,
   CEI_UINT32 channel,
   CEI_INT32  maxMessages,
   CEI_INT32  offset,
   pCEI_INT32 data,
   pCEI_INT32 actualCount
);

EXPORT32 CEI_INT32 DLL_EXPORTED ar_putblock_multi_chan
(
   CEI_UINT32  board,
   CEI_INT32   maxMessages,
   pCEI_UINT32 channels,
   pCEI_INT32  data,
   pCEI_INT32  actualCount
);

EXPORT32 CEI_INT32 DLL_EXPORTED ar_putfilter
(
   CEI_UINT32 board,
   CEI_UINT32 channel, 
   pCEI_CHAR filterTable
);
   
EXPORT32 CEI_INT16 DLL_EXPORTED ar_put_429_message 
(
   CEI_INT16 board,
   CEI_INT16 channel,
   CEI_INT32 data 
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_put_573_frame
(
   CEI_INT16 board,
   CEI_UINT32 numberWords, 
   pCEI_UINT32 transmitCount,
   pCEI_UINT16 arincData 
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_query_device (CEI_INT16 board, pCEI_INT16 boardType);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_putword
(
   CEI_INT16 board, 
   CEI_INT16 channel, 
   CEI_INT32 arincdata
);   
         
EXPORT32 CEI_UINT32 DLL_EXPORTED ar_get_rx_count (CEI_INT16 board, CEI_INT16 channel);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_read_message_schedule_table_entry 
(
   CEI_INT16   board,
   CEI_UINT32  messageIndex,
   pCEI_UINT32 msgSchedTableEntry
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_reset
(
   CEI_INT16 board
);

EXPORT32 CEI_VOID DLL_EXPORTED ar_reset_timercnt (CEI_INT16 board);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_set_config
(
   CEI_INT16 board,
   CEI_INT16 item, 
   CEI_UINT32 value 
);
               
EXPORT32 CEI_INT16 DLL_EXPORTED ar_set_device_config
(
   CEI_INT16 board,
   CEI_INT16 channel, 
   CEI_INT16 item, 
   CEI_INT16 value 
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_set_573_config
(
   CEI_INT16 board,  
   CEI_INT16 item,   
   CEI_INT32 value 
);  

EXPORT32 CEI_INT16 DLL_EXPORTED ar_set_multithread_protect
(
   CEI_INT16 board, 
   CEI_INT16 state 
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_set_preload_config
(
   CEI_INT16  board,
   CEI_INT16  item,
   CEI_UINT32 value
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_set_raw_mode
(
   CEI_INT16 board,
   CEI_INT16 direction,
   CEI_INT16 channel, 
   CEI_INT16 control 
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_set_storage_mode
(
   CEI_INT16 board, 
   CEI_INT16 mode
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_set_time 
(
   CEI_INT16 board,
   pAR_TIMETAG_TYPE timeTag
);

EXPORT32 CEI_VOID DLL_EXPORTED ar_set_timerrate 
(
   CEI_INT16 board, 
   CEI_INT16 rate
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_set_transmitter_mode
(
   CEI_INT16 board, 
   CEI_INT16 mode
);

EXPORT32 CEI_INT16 DLL_EXPORTED ar_stop
(
   CEI_INT16 board
);

EXPORT32 CEI_VOID DLL_EXPORTED ar_version
(
   pCEI_CHAR verstr
);

EXPORT32 CEI_VOID DLL_EXPORTED ar_wait
(
   CEI_FLOAT nsecs
);

EXPORT32 CEI_INT32 DLL_EXPORTED ar_set_isr_function 
(
   CEI_INT32 deviceIndex,             /* the device identifier               */
   pCEI_VOID function                 /* multithread protect state selection */
);
EXPORT32 CEI_VOID DLL_EXPORTED ar_sleep (CEI_UINT32 sleep_ms);

#if defined (_LINUX_X86_) || defined (_LINUX_PPC_)
EXPORT32 CEI_INT32 DLL_EXPORTED ar_set_base_addresses 
(
   CEI_INT16  board,                 /* (input) board to reference (0 to 15) */
   CEI_UINT32 bar0Address,           /* (input) virtual address for BAR0     */
   CEI_UINT32 bar2Address            /* (input) virtual address for BAR2     */
);
#endif
/*===========================================================================*  
 *  CEI-x20 Link-Compatibility API Function prototypes 
 *===========================================================================*/             
EXPORT32 CEI_VOID   DLL_EXPORTED ar_clr_rx_count(CEI_INT16 board,CEI_INT16 channel);
EXPORT32 CEI_INT32  DLL_EXPORTED ar_dump_dp(CEI_INT16 board, pCEI_CHAR file_name, pCEI_CHAR msg);
EXPORT32 CEI_INT16  DLL_EXPORTED ar_force_version (CEI_INT16 board, CEI_INT16 number);
EXPORT32 CEI_VOID   DLL_EXPORTED ar_formatarinclabel (CEI_UINT32 label_data, pCEI_CHAR labelstr);
EXPORT32 CEI_VOID   DLL_EXPORTED ar_formatbinarylabel (CEI_UINT32 label_data, CEI_INT32 trunc_label, CEI_CHAR *labelstr);
EXPORT32 pCEI_CHAR  DLL_EXPORTED ar_get_boardname (CEI_UINT32 board, pCEI_CHAR boardName);
EXPORT32 CEI_VOID   DLL_EXPORTED ar_get_errorLV (pCEI_CHAR string, CEI_INT16 status);
EXPORT32 CEI_INT32  DLL_EXPORTED ar_get_harris (CEI_INT16 board, CEI_INT16 item);
EXPORT32 CEI_INT16  DLL_EXPORTED ar_get_label_filter (CEI_INT16 board, CEI_UINT16 label);
EXPORT32 CEI_INT16  DLL_EXPORTED ar_get_raw_mode (CEI_INT16 board, CEI_INT16 direction, CEI_INT16 channel);
EXPORT32 CEI_UINT16 DLL_EXPORTED ar_get_timercnt (CEI_INT16 board);
EXPORT32 CEI_INT16  DLL_EXPORTED ar_init_dual_port (CEI_INT16 board);
EXPORT32 CEI_INT16  DLL_EXPORTED ar_init_slave (CEI_INT16 board);
EXPORT32 CEI_INT16  DLL_EXPORTED ar_int_control (CEI_INT16 board, CEI_INT16 channel, CEI_INT16 flag);
EXPORT32 CEI_INT16  DLL_EXPORTED ar_int_set (CEI_INT16 board, CEI_INT32 int_num);
EXPORT32 CEI_INT16  DLL_EXPORTED ar_int_slave (CEI_INT16 board);
EXPORT32 CEI_INT16  DLL_EXPORTED ar_loadslv_init_discrete (CEI_INT16 board, CEI_UINT32 base_seg, CEI_INT32 base_port, CEI_UINT16 ram_size, CEI_UINT32 disc_init);
EXPORT32 CEI_INT16  DLL_EXPORTED ar_msg_control (CEI_INT16 board, CEI_INT16 control);
EXPORT32 CEI_INT16  DLL_EXPORTED ar_putword2x16 (CEI_INT16 board, CEI_INT16 channel, CEI_INT16 lsw, CEI_INT16 msw);
EXPORT32 CEI_VOID   DLL_EXPORTED ar_recreate_parity (pCEI_UCHAR word);
EXPORT32 CEI_VOID   DLL_EXPORTED ar_reformat (pCEI_VOID lsw, pCEI_VOID msw);
EXPORT32 CEI_INT16  DLL_EXPORTED ar_reset_int (CEI_INT16 board);
EXPORT32 CEI_INT32  DLL_EXPORTED ar_setchparms (CEI_UINT32 cardnum, CEI_UINT32 channel, CEI_INT32 TransRcv, pAR_CHANNEL_PARMS def);
EXPORT32 CEI_INT32  DLL_EXPORTED ar_setinterrupts (CEI_UINT32 cardnum, CEI_UINT32 channel, CEI_INT32 TransRcv, CEI_UINT32 Count, CEI_UINT32 Mask);
EXPORT32 CEI_INT16  DLL_EXPORTED ar_set_arinc_config (CEI_INT16 board, CEI_INT16 channel, CEI_INT16 item, CEI_UINT32 value);
EXPORT32 CEI_INT16  DLL_EXPORTED ar_set_harris (CEI_INT16 board, CEI_INT16 item, CEI_UINT32 value);
EXPORT32 CEI_INT16  DLL_EXPORTED ar_timetag_control (CEI_INT16 board, CEI_INT16 flag);
EXPORT32 CEI_VOID   DLL_EXPORTED ar_unformat (pCEI_INT16 lsword, pCEI_INT16 msword);
EXPORT32 CEI_INT16  DLL_EXPORTED ar_xmit_sync (CEI_INT16 board, CEI_INT16 channel);


#ifdef __cplusplus
}
#endif
#endif /* CEIDEV_API_H */
