/*===========================================================================*
 * FILE:                     C D E V _ G L B . H                    
 *===========================================================================*
 *
 * COPYRIGHT (C) 2003-2011 BY
 *          GE INTELLIGENT PLATFORMS, INC. SANTA BARBARA, CALIFORNIA
 *          ALL RIGHTS RESERVED.
 *
 *          THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *          COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *          THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *          OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *          AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *          SOFTWARE IS HEREBY TRANSFERRED.
 *
 *          THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *          NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY GE 
 *          INTELLIGENT PLATFORMS.
 *
 *===========================================================================*
 *
 * FUNCTION:    Application Programmer's Interface for a CEI-x30 ARINC device.
 *              Header File for API Global Variables and Data Structures
 *
 *---------------------------------------------------------------------------*
 *          
 * HISTORY:   
 *
 *   Date    By   Comments
 * -------- ----- ------------------------------------------------------------
 * 10/21/05  BDW  Added support for the new CEI-430-supported API channel types
 *                   DIFFERENTIAL_IN_CHANNEL and DIFFERENTIAL_OUT_CHANNEL
 *                   to the channelTypeCrossRef[][] table
 * 09/25/06  BDW  Added prototype for download routine a429_utl_program_xilinx_9056_32bit
 * 10/29/07  BDW  Added prototype for open error handling, cei_get_open_error
 * 02/18/08  BDW  Removed the data struct array channelIdent[]; structure members
 *                   relocated in the new data structure array "deviceGlobal"
 *                Removed the following global arrays, relocated as structure
 *                   members in the new data structure array "deviceGlobal":
 *                      alternateJconfiguration      
 *                      deviceTimeTagReference       
 *                      deviceX20CompatibleResolution
 *                      userProgrammableIrigOffset   
 *                      currentIrigTimeUsec
 *                      irigReferenceTimeTag
 *                Removed the variable multiThreadProtect, relocated it as a 
 *                   structure member in the data structure array "device".
 *                Added prototypes for multi-process support: 
 *                   cei_open_shared_lock and cei_map_shared_memory
 *                Moved the remaining API local data arrays to CDEV_API.C.
 * 04/15/09  BDW  Incorporated support for the CEI-x30 Enhanced Firmware Design.
 *                adding prototypes for low-level hardware interrupt routines.
 * 06/16/09  BDW  Added prototype for cei_open_preload_lock.
 * 06/23/09  BDW  Added prototype for a429_utl_enh.
 * 07/01/09  BDW  Added prototype for cei_delay_milliseconds.
 *
 *===========================================================================*/ 
#ifndef CEIDEV_GLB_H
#define CEIDEV_GLB_H
#include "cdev_api.h"

#define AR_USE_PROGRAMMED_TIMETAG_MODE      AR_TIMETAG_LAST_OPTION
#define AR_BIT_TIMER_DEVIATION              1000
#define AR_WRAP_TEST_MSG_COUNT              20
#define AR_SIXTEEN_MHZ                      16000000L
#define AR_STATIC                           0
#define AR_SHIFT                            1
#define AR_ADD                              2
#define AR_NUM_PATTERNS                     6
#define AR_NUM_UNUSED_SRAM_LOCATIONS        6
#define AR_LOWER                            0
#define AR_UPPER                            1
#define LOCK_IDENT                          (CEI_INT16)board

#define INTERRUPT_QUEUE_SIZE    2048
/*========================================================================* 
 * CEI_INT_QUEUE_TYPE is used to store the interrupt queue
 * entries for access from a separate processing thread.
 *========================================================================*/
typedef struct CEI_INT_QUEUE {
   CEI_UINT32 intQueueHeadPtr;
   CEI_UINT32 intQueueTailPtr;
   CEI_UINT32 intQueueEntries[INTERRUPT_QUEUE_SIZE];
} CEI_INT_QUEUE_TYPE;


/* Define the basis for a time delay using the system clock resolution */
#if defined (LABVIEW_RT) || defined (_WIN32)    
   #define TIMER_CLOCK_TICKS_PER_SECOND CLOCKS_PER_SEC
#elif defined (VXWORKS)  || defined (_LINUX_X86_) || defined (_LINUX_PPC_) || defined (INTEGRITY_PCI_PPC)
   #define TIMER_CLOCK_TICKS_PER_SECOND 1000
#endif

#ifndef IN_LNX_SOURCE
/*===========================================================================*  
 * API Local (Private) Function prototypes
 *===========================================================================*/  
CEI_INT32 a429_utl_adjust_irig_threshold (CEI_INT16 board, CEI_INT32 displayDacSettings);
CEI_INT32 a429_utl_acquire_formatted_time_tag (CEI_INT16 board, 
                                               pCEI_UINT32 chanMemPtr,
                                               CEI_INT32 timeSource, 
                                               CEI_INT32 timeTagFormat, 
                                               CEI_INT32 mergedMode, 
                                               pAR_TIMETAG_TYPE timeTagRef);
CEI_INT16 a429_utl_memory_test (CEI_INT16 board, CEI_INT32 testType, CEI_INT32 testOption);
CEI_INT16 a429_utl_wrap_test (CEI_INT16 board, CEI_INT32 wrap, CEI_INT32 msgCount);
CEI_VOID  a429_utl_determine_channel_support ( CEI_INT16 board );
CEI_INT16 a429_utl_enh (CEI_INT16 board, CEI_INT32 action);
CEI_INT16 a429_utl_enh_check (CEI_INT16 board, CEI_INT32 deviceIdent);
CEI_INT32 a429_utl_find_dac_threshold (CEI_INT16 board,
                                       CEI_INT32 useActualTimeSample,
                                       CEI_INT32 displayDacSettings,
                                       CEI_INT32 thresholdType,
                                       CEI_UINT32 voltageLimit,
                                       CEI_UINT32 initialDacAdjustment,
                                       pCEI_UINT32 finalDacValue,
                                       pCEI_UINT32 dacBoudaryCushionValue);
CEI_VOID a429_utl_initialize_global_data (CEI_INT16 board);
CEI_INT32 a429_utl_is_irig_calibrated (CEI_INT16 board);
CEI_INT32 a429_utl_program_xilinx_9030 (CEI_INT16 board, CEI_INT32 deviceIdent, CEI_UINT32 programSize, pCEI_UCHAR fwloadPtr);
CEI_INT32 a429_utl_program_xilinx_9030_32bit (CEI_INT16 board, CEI_INT32 deviceIdent, CEI_UINT32 programSize, pCEI_UINT32 fwloadPtr);
CEI_INT32 a429_utl_program_xilinx_9056 (CEI_INT16 board, CEI_INT32 deviceIdent, CEI_UINT32 programSize, pCEI_UCHAR fwloadPtr);
CEI_INT32 a429_utl_program_xilinx_9056_32bit (CEI_INT16 board, CEI_INT32 deviceIdent, CEI_UINT32 programSize, pCEI_UINT32 fwloadPtr);
CEI_INT32 a429_utl_quick_adjust_irig_threshold (CEI_INT16 board, CEI_INT32 displayDacSettings);
CEI_INT32 a429_utl_store_message_scheduler_entry (CEI_INT32 modifyEntryFlag, CEI_INT32 numberOfEntries, pAR_SCHEDULED_MSG_ENTRY_TYPE messageEntry);
CEI_INT16 a429_utl_validate_channel_request ( CEI_INT16 board, CEI_UINT32 channel, CEI_INT16 item );
CEI_INT32 a429_utl_validate_scheduled_message_parameters (CEI_UINT32 board, CEI_UINT32 channel, pCEI_UINT32 hwChannel);

CEI_INT32 cei_acquire_thread_lock (CEI_INT16 deviceIndex);
CEI_INT32 cei_release_thread_lock (CEI_INT16 deviceIndex);
CEI_INT32 cei_acquire_device (CEI_INT16 deviceIndex, ARINC_LOCAL_TYPE * device);
CEI_INT32 cei_release_device (CEI_INT16 deviceIndex, ARINC_LOCAL_TYPE * device);
CEI_INT32 cei_read_32bit (CEI_INT16 board,  pCEI_UINT32 address, pCEI_UINT32 data);
CEI_INT32 cei_write_32bit (CEI_INT16 board, pCEI_UINT32 address, CEI_UINT32 data);
CEI_INT32 cei_delay_seconds (CEI_FLOAT numberOfSeconds);
CEI_INT32 cei_delay_milliseconds (CEI_UINT32 numberOfMilliseconds);
#ifdef INTEGRITY_PCI_PPC
pCEI_CHAR cei_get_open_error ();
#else
pCEI_CHAR cei_get_open_error (CEI_VOID);
#endif
CEI_INT32 cei_map_shared_memory ( CEI_UCHAR    blockId,
                            CEI_UINT32   blockSize,
                            CEI_VOID  ** blockPtr,
                            CEI_VOID  ** blockHandle,
                            CEI_UINT32 * alreadyExists,
                            CEI_UINT32 * isExclusive );
CEI_INT32 cei_get_host_system_time (pCEI_UINT32 hostTime);
CEI_VOID  cei_get_host_system_time_x (TIME_TAG_TYPE * hostTime);
CEI_VOID  cei_get_irig_initial_time (pCEI_UINT32 time);
CEI_INT32 cei_acquire_device_identity (CEI_INT32 deviceIndex, pCEI_INT32 vendorId, pCEI_INT32 deviceId);
CEI_INT32 cei_open_shared_lock (CEI_INT16 sharedLockId, pCEI_INT32 sharedLockState);
CEI_INT32 cei_open_preload_lock (CEI_INT16 device);
CEI_INT32 cei_init_interrupt ( CEI_INT32   deviceIndex,
#if defined (_WIN32)    
                               CEI_INT32   (_stdcall *isr)(CEI_INT32), 
#else
                               pCEI_VOID   isr, 
#endif
                               CEI_INT32   priority,
                               pCEI_UINT32 deviceBaseAddress,
                               CEI_INT32   memRegion,
                               CEI_UINT32  disOffset,
                               CEI_UINT32  interruptBit,
                               pCEI_UINT32 isrEnabled );

CEI_INT32 cei_terminate_interrupt(CEI_INT32 deviceIndex, 
                                  CEI_UINT32 interruptBit,
                                  pCEI_UINT32 deviceBaseAddress);
CEI_INT32 a429_utl_plx_eeprom_read (CEI_INT16 board, 
                                    CEI_INT32 pciInterfaceType, 
                                    pCEI_UINT32 pciConfigPtr, 
                                    pCEI_UINT32 promValues);
CEI_INT32 a429_utl_plx_eeprom_write_enable (CEI_INT16 board, 
                                            CEI_INT32 pciInterfaceType, 
                                            pCEI_UINT32 pciConfigPtr);
CEI_INT32 a429_utl_plx_eeprom_write_disable (CEI_INT16 board, 
                                             CEI_INT32 pciInterfaceType, 
                                             pCEI_UINT32 pciConfigPtr);
CEI_INT32 a429_utl_plx_eeprom_write (CEI_INT16 board, 
                                     CEI_INT32 pciInterfaceType, 
                                     pCEI_UINT32 pciConfigPtr, 
                                     CEI_UINT32 address, 
                                     CEI_UINT32 data);


/*===========================================================================* 
 *  For LabVIEW Real-Time with full VISA support, writes to BAR0 require a 
 *  different argument to the viOut32 invocation, so we use a different write 
 *  access routine, see cdev_lrt.h for details; select via compiler directive.
 *===========================================================================*/
#if defined (NIVISA_PXI)    
CEI_INT32 cei_read_plx_32bit (CEI_INT16 board,  pCEI_UINT32 address, pCEI_UINT32 data);
CEI_INT32 cei_write_plx_32bit (CEI_INT16 board, pCEI_UINT32 address, CEI_UINT32 data);
#endif

#endif

#endif /* CEIDEV_GLB_H */

