/*============================================================================*
 * FILE:                      L O W L E V E L . H
 *============================================================================*
 *
 * COPYRIGHT (C) 1997 - 2008 BY
 *          GE INTELLIGENT PLATFORMS, INC. SANTA BARBARA, CALIFORNIA
 *          ALL RIGHTS RESERVED.
 *
 *          THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *          COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *          THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *          OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *          AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *          SOFTWARE IS HEREBY TRANSFERRED.
 *
 *          THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *          NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY GE 
 *          INTELLIGENT PLATFORMS.
 *
 *===========================================================================*
 *
 * FUNCTION:    Header file for low level WinRT device driver access.
 *              This file defines the error codes returned from the WinRT
 *              memory mapping functions.
 *
 *===========================================================================*
 *
 * HISTORY:
 *
 * Date     Int Ver  Comments
 * -------- --- ---- --------------------------------------------------------
 * 12/23/97 AJH  X   Created file from busapi.h to support CEI-220 program 
 *                   loader.
 * 01/05/98 AJH  X   Added function prototypes to make this a reusable 
 *                   component.  This file matches LOWLEVEL.C version 2.43
 * 01/06/99 DSZ 100  Included the lowlevel c and h files into a library, 
 *                   CEI.DLL.  Existing function prototypes were modified to 
 *                   export/import functions appropriately.  New functionality 
 *                   for the CEI_LL API was added with functions prefixed by 
 *                   ceill_ and their implementation can be found in cei_ll.c.
 * 04/28/99 BDW 101  Added support for Bus Type PCMCIA - (CEI-710) 
 * 09/29/99 DSZ 102  Removed reference to 1553 device in vbtMapBoardAddress()
 *                   error message.  Added ceill_unmapHwDevice()
 * 09/03/02 BDW 201  Added support for TEWS PCI9030 IP Carrier
 * 08/24/06 BDW 300  Incorporated CEI_TYPES.H data type definitions in 
 *                     support of 64-bit Linux operating systems.
 * 07/10/07 BCH 301  Added vbtGetPCIConfigRegister
 *
 *===========================================================================*/

#ifndef _LOWLEVEL_H_
#define _LOWLEVEL_H_
               
//---------------------------------------------------------------------------*
//
//
// Constants common to all products supported by this low-level interface
//
//
//---------------------------------------------------------------------------*

#ifndef WIN32
 #ifndef HANDLE
  #define HANDLE CEI_INT32
 #endif
 #ifndef CEI_UINT32
  #define CEI_UINT32 unsigned int
 #endif
 #ifndef CEI_INT32
  #define CEI_INT32  int
 #endif
 #define _stdcall
 #define MAX_PATH    256
#endif


/**********************************************************************
*  Error return codes from LOWLEVEL routines
**********************************************************************/
#define BTD_OK                0     // success
#define BTD_ERR_PARAM         1     // invalid parameter
#define BTD_ERR_BADADDR       4     // invalid address
#define BTD_ERR_BADADDRMAP    12    // bad initial mapping of address
#define BTD_NO_DRV_MOD        30    // no Driver Module found
#define BTD_IOCTL_SET_REG     32
#define BTD_IOCTL_REG_SIZE    33
#define BTD_IOCTL_GET_REG     34
#define BTD_BAD_SIZE          35
#define BTD_NO_HASH_ENTRY     38
#define BTD_ERR_NOWINRT       50    // WinRT driver not loaded/started
#define BTD_ERR_BADREGISTER   51    // WinRT parameters don't match registry
#define BTD_ERR_BADOPEN       52    // WinRT device open failed
#define BTD_UNKNOWN_BUS       53    // Bus is not PCI, ISA or VME
#define BTD_BAD_LL_VERSION    54    // Unsupported lowlevel driver installed
#define BTD_BAD_INT_EVENT     55    // Unable to create interrupt event
#define BTD_ISR_SETUP_ERROR   56    // Error setting up the ISR driver
#define BTD_CREATE_ISR_THREAD 57    // Error creating the ISR thread
#define BTD_NO_REGIONS_TO_MAP 58
#define BTD_BAD_DEVICE_NUM    59
#define BTD_DRV_READ_FAIL     60 
#define BTD_DRV_WRITE_FAIL    61
#define BTD_DRV_IOCTL_FAIL    62
#define BTD_SYSFS_FAIL        63        
#define BTD_SYSFS_ATTR_FAIL   64
#define BTD_TOO_MANY_RGNS     65    // invalid hardware memory region count
#define BTD_ERR_ONCLOSE       76    // Error closing device handle.

#define BTD_NEXT_INTERRUPT    0
#define BTD_EXIT_INTERRUPT    1

/*---------------------------------------------------------------------------*
 *  Product and Registry information.
 *---------------------------------------------------------------------------*/
      
//enumerate carrier bus types
#define CEWRT_BUSTYPE_PCI       70
#define CEWRT_BUSTYPE_ISA       71
#define CEWRT_BUSTYPE_PCMCIA    72
#define CEWRT_BUSTYPE_NOT_FOUND 75

#define P_SER                   0x1004
#define HWREG_CONTROL           0x0000
#define HWREG_WRITE_INTERRUPT   0x0000

/**********************************************************************
*  Lowlevel address mapping structure
**********************************************************************/
#define MAX_MEMORY   6     /* Number of supported memory regions */
#define MAX_PORTS    2     /* Number of supported IO regions     */
#define MAX_MEMORY_REGION_SIZE 0x3000000L

// define: CEI_ENUM_MAX_DEVS
// usage:  The maximum number Condor devices that can be physically installed 
//         in a machine at any one time.
#define CEI_ENUM_MAX_DEVS         16

// type:   CEI_ENUM_DEV_DESC
// usage:  This structure contains the 'DeviceDesc' and 'LocationInformation'
//         registry fields associated with a particular installed Condor 
//         device.
typedef struct 
{
   CEI_CHAR devDesc[MAX_PATH];
   CEI_CHAR locInfo[MAX_PATH];
} CEI_ENUM_DEV_DESC;


// type:   CEI_ENUM_DEV_LIST
// usage:  This structure contains a list of Condor devices that are currently
//         installed on the target machine. 
typedef struct 
{
   CEI_INT32 numDevs;
   CEI_ENUM_DEV_DESC devs[CEI_ENUM_MAX_DEVS];
} CEI_ENUM_DEV_LIST;

typedef enum _BUS_TYPE     /* Host bus types supported           */
{
    BUS_INTERNAL,
    BUS_ISA,
    BUS_PCI,
    BUS_VME,
    BUS_PCMCIA,
    BUS_OTHER
} BUS_TYPE;           // These are the bus types supported by lowlevel.c

   // Structure for communicating memory mapping information about a device.
typedef struct _DEVMAP_T {
   CEI_INT32   busType;                    // One of BUS_TYPE.
   CEI_INT32   interruptNumber;            // Interrupt number.
   CEI_INT32   memSections;                // Number of memory regions defined.
   CEI_INT32   flagMapToHost[MAX_MEMORY];  // Set to map region into host space.
   pCEI_CHAR   memHostBase[MAX_MEMORY];    // Base address of region in host space.
   unsigned long  memStartPhy[MAX_MEMORY];    // Physical base address of region.
   CEI_UINT32  memLengthBytes[MAX_MEMORY]; // Length of region in bytes.
   CEI_INT32   portSections;               // Number of I/O port regions.
   unsigned long  portStart[MAX_PORTS];       // I/O Address of first byte.
   CEI_UINT32  portLength[MAX_PORTS];      // Number of bytes in region.
   CEI_INT32   llDriverVersion;            // Low Level driver version.
   CEI_INT32   KernelDriverVersion;        // Kernel driver version.
   HANDLE      hKernelDriver;              // Handle to the kernel driver.
} DEVMAP_T, *PDEVMAP_T;

/**********************************************************************
*  Functions defined in the LOWLEVEL.C source file
**********************************************************************/
CEI_VOID _stdcall vbtOutpb_32    (CEI_UINT32 device, CEI_INT32 address, CEI_CHAR frame);
CEI_VOID _stdcall vbtOutpw_32    (CEI_UINT32 device, CEI_INT32 address, CEI_INT32 frame);
CEI_VOID _stdcall vbtOutpd_32    (CEI_UINT32 device, CEI_INT32 address, CEI_INT32 frame);

CEI_INT32 _stdcall vbtInpd_32     (CEI_UINT32 device, CEI_INT32 address);
CEI_INT32 _stdcall vbtInpw_32     (CEI_UINT32 device, CEI_INT32 address);
CEI_CHAR  _stdcall vbtInpb_32     (CEI_UINT32 device, CEI_INT32 address);
CEI_INT32 _stdcall vbtInpwOutpw_32(CEI_UINT32 device, CEI_INT32 frame);

CEI_INT32 _stdcall vbtGetInterrupt (CEI_UINT32 device);

CEI_INT32 vbtHookInterrupt
(
   CEI_UINT32 device, 
   CEI_INT32 (_stdcall *fn)(CEI_INT32),
   CEI_INT32 iPriority,
   HANDLE * pHandleIntThread
);

CEI_VOID   _stdcall vbtCloseInterrupt(CEI_UINT32 device);

CEI_INT32  _stdcall vbtGetBusType (HANDLE hWinRT1);

CEI_INT32  _stdcall vbtFreeBoardAddresses(PDEVMAP_T pMap);

CEI_INT32  _stdcall vbtMapBoardAddresses(CEI_UINT32 device,
                                         PDEVMAP_T  pMap); 

HANDLE _stdcall vbtOpenHandle(CEI_UINT32 device);

CEI_INT32 _stdcall vbtMapBoardAddress(CEI_UINT32  cardnum,
                                      CEI_UINT32  device,
                                      pCEI_CHAR   *addr,
                                      pCEI_UINT32 MemoryMapSize);

CEI_INT32 _stdcall vbtFreeBoardAddress(CEI_UINT32 device);

CEI_INT vbtGetPCIConfigRegister(CEI_UINT32 device,
                                CEI_UINT32 offset,
                                CEI_UINT32 length,
                                void* value);


#if defined(__WIN32__) || defined(_WIN32) || defined(VXWORKS) || defined(_CVI_)

BOOL extractInt (pCEI_CHAR text, pCEI_CHAR begText, pCEI_UINT32 value, pCEI_CHAR *nextChar);
BOOL enumBoardsWDM (pCEI_CHAR busType, CEI_ENUM_DEV_LIST *list);
BOOL mapID (CEI_UINT32 installID, pCEI_UINT32 actualID);

#endif

#endif  // #ifndef _LOWLEVEL_H_
